# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.8
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.


"""
PyPercept is the Python interface for the Trilinos STK Percept package.

The following are examples of the actual PyPercept Python interface:

##################################################################################################################

Case 1: Norm of the difference of an Exact Solution and a Mesh Field

pMesh = PerceptMesh() #Create a PerceptMesh database
#Specify the geometric hex mesh to be a 3x3x3 box between 0,0,0 and 2,2,2
pMesh.new_mesh(GMeshSpec('3x3x3|bbox:0,0,0,2,2,2')) 
field = pMesh.add_field('coordinates', 1) #Add a coordinates field
pMesh.commit() #Commits the mesh to the mesh database

input_array = numpy.array([1.0, 0.5, 0.5]) #Create numpy arrays to be evaluated 
input_array_2 = numpy.array([1.0, 1.5, 1.5])
'
#Create a field function with a name, field base, and the mesh and the domain and co-domain dimensions
ff = FieldFunction('ff', field, pMesh, 3, 3) 
ff.addAlias('myalias') #Define another name for the field function
ff_output = ff.evaluate(input_array) #Evaluate the field function with the given array

f2 = FieldFunction('f2', field, pMesh, 3, 3)
f2_output = f2.evaluate(input_array_2)

#Create a function from a string 'x+y+z', with a name, the domain and co-domain dimensions
sf = StringFunction('x+y+z', 'myname', 3, 1) 
sf_output = sf.evaluate(input_array) #Evaluate the function with the given array

sf_diff = StringFunction('ff-f2', 'myname') #Create a function of the difference of the field functions

norm = L1Norm(pMesh.getBulkData()) #Create a norm object to evaluate the norm of the mesh field
value = norm.evaluate(ff) #Evaluate the norm
diffnorm = norm.evaluate(sf_diff) #Evaluate the norm of the difference of the exact solution and the mesh field

#Now use a helper function to evaluate the norm: eval_norm(bulkData, Function, int) 
value1 = eval_norm(pMesh.getBulkData(), ff, 1) 
diffnorm1 = eval_norm(pMesh.getBulkData(), sf_diff, 1)

#################################################################################################################

Case 2: Refine a mesh uniformly

pMesh = PerceptMesh()
pMesh.open('tet-mesh-brick-no-block.e') #Read an existing exodus file

uniform_refiner = Refiner(pMesh, TET4_TET4_8) #Create a refiner on the specified mesh of the specified pattern.
pMesh.commit()

i = 0
while i < 3:
  uniform_refiner.doBreak() #Call the doBreak() method to refine the mesh
  i = i + 1

pMesh.saveAs('tet-mesh-refined-3-times.e') #Save out the refined mesh

#################################################################################################################

Case 3: Norm of the difference of results from two separate simulations

subprocess.call('sierra aria -i tet_mesh.e -o result_0.e') #Run an external program from Python

pMesh = PerceptMesh()
pMesh.open('tet-mesh.e')
uniform_refiner = Refiner(pMesh, TET4_TET4_8)
pMesh.commit()

uniform_refiner.doBreak()
pMesh.saveAs('tet-mesh_refined.e')

subprocess.call('sierra aria -i tet_mesh_refined.e -o result_1.e')

pMesh_0 = PerceptMesh()
pMesh_1 = PerceptMesh()
pMesh_0.openReadOnly('result_0.e')
pMesh_1.openReadOnly('result_1.e')

ff_0 = Field_Function(pMesh_0)
ff_1 = Field_Function(pMesh_1)
diff = StringFunction('ff_0 - ff_1')

diffnorm = eval_norm(pMesh.getBulkData, diff, 2)
print 'diffnorm = ', diffnorm

#################################################################################################################

"""


from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_PerceptMesh', [dirname(__file__)])
        except ImportError:
            import _PerceptMesh
            return _PerceptMesh
        if fp is not None:
            try:
                _mod = imp.load_module('_PerceptMesh', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _PerceptMesh = swig_import_helper()
    del swig_import_helper
else:
    import _PerceptMesh
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_SwigPyIterator
    def value(self): return _PerceptMesh.SwigPyIterator_value(self)
    def incr(self, n=1): return _PerceptMesh.SwigPyIterator_incr(self, n)
    def decr(self, n=1): return _PerceptMesh.SwigPyIterator_decr(self, n)
    def distance(self, *args): return _PerceptMesh.SwigPyIterator_distance(self, *args)
    def equal(self, *args): return _PerceptMesh.SwigPyIterator_equal(self, *args)
    def copy(self): return _PerceptMesh.SwigPyIterator_copy(self)
    def next(self): return _PerceptMesh.SwigPyIterator_next(self)
    def __next__(self): return _PerceptMesh.SwigPyIterator___next__(self)
    def previous(self): return _PerceptMesh.SwigPyIterator_previous(self)
    def advance(self, *args): return _PerceptMesh.SwigPyIterator_advance(self, *args)
    def __eq__(self, *args): return _PerceptMesh.SwigPyIterator___eq__(self, *args)
    def __ne__(self, *args): return _PerceptMesh.SwigPyIterator___ne__(self, *args)
    def __iadd__(self, *args): return _PerceptMesh.SwigPyIterator___iadd__(self, *args)
    def __isub__(self, *args): return _PerceptMesh.SwigPyIterator___isub__(self, *args)
    def __add__(self, *args): return _PerceptMesh.SwigPyIterator___add__(self, *args)
    def __sub__(self, *args): return _PerceptMesh.SwigPyIterator___sub__(self, *args)
    def __iter__(self): return self
SwigPyIterator_swigregister = _PerceptMesh.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class vectorfieldbase(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vectorfieldbase, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vectorfieldbase, name)
    __repr__ = _swig_repr
    def iterator(self): return _PerceptMesh.vectorfieldbase_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _PerceptMesh.vectorfieldbase___nonzero__(self)
    def __bool__(self): return _PerceptMesh.vectorfieldbase___bool__(self)
    def __len__(self): return _PerceptMesh.vectorfieldbase___len__(self)
    def pop(self): return _PerceptMesh.vectorfieldbase_pop(self)
    def __getslice__(self, *args): return _PerceptMesh.vectorfieldbase___getslice__(self, *args)
    def __setslice__(self, *args): return _PerceptMesh.vectorfieldbase___setslice__(self, *args)
    def __delslice__(self, *args): return _PerceptMesh.vectorfieldbase___delslice__(self, *args)
    def __delitem__(self, *args): return _PerceptMesh.vectorfieldbase___delitem__(self, *args)
    def __getitem__(self, *args): return _PerceptMesh.vectorfieldbase___getitem__(self, *args)
    def __setitem__(self, *args): return _PerceptMesh.vectorfieldbase___setitem__(self, *args)
    def append(self, *args): return _PerceptMesh.vectorfieldbase_append(self, *args)
    def empty(self): return _PerceptMesh.vectorfieldbase_empty(self)
    def size(self): return _PerceptMesh.vectorfieldbase_size(self)
    def clear(self): return _PerceptMesh.vectorfieldbase_clear(self)
    def swap(self, *args): return _PerceptMesh.vectorfieldbase_swap(self, *args)
    def get_allocator(self): return _PerceptMesh.vectorfieldbase_get_allocator(self)
    def begin(self): return _PerceptMesh.vectorfieldbase_begin(self)
    def end(self): return _PerceptMesh.vectorfieldbase_end(self)
    def rbegin(self): return _PerceptMesh.vectorfieldbase_rbegin(self)
    def rend(self): return _PerceptMesh.vectorfieldbase_rend(self)
    def pop_back(self): return _PerceptMesh.vectorfieldbase_pop_back(self)
    def erase(self, *args): return _PerceptMesh.vectorfieldbase_erase(self, *args)
    def __init__(self, *args): 
        this = _PerceptMesh.new_vectorfieldbase(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _PerceptMesh.vectorfieldbase_push_back(self, *args)
    def front(self): return _PerceptMesh.vectorfieldbase_front(self)
    def back(self): return _PerceptMesh.vectorfieldbase_back(self)
    def assign(self, *args): return _PerceptMesh.vectorfieldbase_assign(self, *args)
    def resize(self, *args): return _PerceptMesh.vectorfieldbase_resize(self, *args)
    def insert(self, *args): return _PerceptMesh.vectorfieldbase_insert(self, *args)
    def reserve(self, *args): return _PerceptMesh.vectorfieldbase_reserve(self, *args)
    def capacity(self): return _PerceptMesh.vectorfieldbase_capacity(self)
    __swig_destroy__ = _PerceptMesh.delete_vectorfieldbase
vectorfieldbase_swigregister = _PerceptMesh.vectorfieldbase_swigregister
vectorfieldbase_swigregister(vectorfieldbase)

class vectorparts(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vectorparts, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vectorparts, name)
    __repr__ = _swig_repr
    def iterator(self): return _PerceptMesh.vectorparts_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _PerceptMesh.vectorparts___nonzero__(self)
    def __bool__(self): return _PerceptMesh.vectorparts___bool__(self)
    def __len__(self): return _PerceptMesh.vectorparts___len__(self)
    def pop(self): return _PerceptMesh.vectorparts_pop(self)
    def __getslice__(self, *args): return _PerceptMesh.vectorparts___getslice__(self, *args)
    def __setslice__(self, *args): return _PerceptMesh.vectorparts___setslice__(self, *args)
    def __delslice__(self, *args): return _PerceptMesh.vectorparts___delslice__(self, *args)
    def __delitem__(self, *args): return _PerceptMesh.vectorparts___delitem__(self, *args)
    def __getitem__(self, *args): return _PerceptMesh.vectorparts___getitem__(self, *args)
    def __setitem__(self, *args): return _PerceptMesh.vectorparts___setitem__(self, *args)
    def append(self, *args): return _PerceptMesh.vectorparts_append(self, *args)
    def empty(self): return _PerceptMesh.vectorparts_empty(self)
    def size(self): return _PerceptMesh.vectorparts_size(self)
    def clear(self): return _PerceptMesh.vectorparts_clear(self)
    def swap(self, *args): return _PerceptMesh.vectorparts_swap(self, *args)
    def get_allocator(self): return _PerceptMesh.vectorparts_get_allocator(self)
    def begin(self): return _PerceptMesh.vectorparts_begin(self)
    def end(self): return _PerceptMesh.vectorparts_end(self)
    def rbegin(self): return _PerceptMesh.vectorparts_rbegin(self)
    def rend(self): return _PerceptMesh.vectorparts_rend(self)
    def pop_back(self): return _PerceptMesh.vectorparts_pop_back(self)
    def erase(self, *args): return _PerceptMesh.vectorparts_erase(self, *args)
    def __init__(self, *args): 
        this = _PerceptMesh.new_vectorparts(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _PerceptMesh.vectorparts_push_back(self, *args)
    def front(self): return _PerceptMesh.vectorparts_front(self)
    def back(self): return _PerceptMesh.vectorparts_back(self)
    def assign(self, *args): return _PerceptMesh.vectorparts_assign(self, *args)
    def resize(self, *args): return _PerceptMesh.vectorparts_resize(self, *args)
    def insert(self, *args): return _PerceptMesh.vectorparts_insert(self, *args)
    def reserve(self, *args): return _PerceptMesh.vectorparts_reserve(self, *args)
    def capacity(self): return _PerceptMesh.vectorparts_capacity(self)
    __swig_destroy__ = _PerceptMesh.delete_vectorparts
vectorparts_swigregister = _PerceptMesh.vectorparts_swigregister
vectorparts_swigregister(vectorparts)

class vectors(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vectors, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vectors, name)
    __repr__ = _swig_repr
    def iterator(self): return _PerceptMesh.vectors_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _PerceptMesh.vectors___nonzero__(self)
    def __bool__(self): return _PerceptMesh.vectors___bool__(self)
    def __len__(self): return _PerceptMesh.vectors___len__(self)
    def pop(self): return _PerceptMesh.vectors_pop(self)
    def __getslice__(self, *args): return _PerceptMesh.vectors___getslice__(self, *args)
    def __setslice__(self, *args): return _PerceptMesh.vectors___setslice__(self, *args)
    def __delslice__(self, *args): return _PerceptMesh.vectors___delslice__(self, *args)
    def __delitem__(self, *args): return _PerceptMesh.vectors___delitem__(self, *args)
    def __getitem__(self, *args): return _PerceptMesh.vectors___getitem__(self, *args)
    def __setitem__(self, *args): return _PerceptMesh.vectors___setitem__(self, *args)
    def append(self, *args): return _PerceptMesh.vectors_append(self, *args)
    def empty(self): return _PerceptMesh.vectors_empty(self)
    def size(self): return _PerceptMesh.vectors_size(self)
    def clear(self): return _PerceptMesh.vectors_clear(self)
    def swap(self, *args): return _PerceptMesh.vectors_swap(self, *args)
    def get_allocator(self): return _PerceptMesh.vectors_get_allocator(self)
    def begin(self): return _PerceptMesh.vectors_begin(self)
    def end(self): return _PerceptMesh.vectors_end(self)
    def rbegin(self): return _PerceptMesh.vectors_rbegin(self)
    def rend(self): return _PerceptMesh.vectors_rend(self)
    def pop_back(self): return _PerceptMesh.vectors_pop_back(self)
    def erase(self, *args): return _PerceptMesh.vectors_erase(self, *args)
    def __init__(self, *args): 
        this = _PerceptMesh.new_vectors(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _PerceptMesh.vectors_push_back(self, *args)
    def front(self): return _PerceptMesh.vectors_front(self)
    def back(self): return _PerceptMesh.vectors_back(self)
    def assign(self, *args): return _PerceptMesh.vectors_assign(self, *args)
    def resize(self, *args): return _PerceptMesh.vectors_resize(self, *args)
    def insert(self, *args): return _PerceptMesh.vectors_insert(self, *args)
    def reserve(self, *args): return _PerceptMesh.vectors_reserve(self, *args)
    def capacity(self): return _PerceptMesh.vectors_capacity(self)
    __swig_destroy__ = _PerceptMesh.delete_vectors
vectors_swigregister = _PerceptMesh.vectors_swigregister
vectors_swigregister(vectors)

class vectorvs(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vectorvs, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vectorvs, name)
    __repr__ = _swig_repr
    def iterator(self): return _PerceptMesh.vectorvs_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _PerceptMesh.vectorvs___nonzero__(self)
    def __bool__(self): return _PerceptMesh.vectorvs___bool__(self)
    def __len__(self): return _PerceptMesh.vectorvs___len__(self)
    def pop(self): return _PerceptMesh.vectorvs_pop(self)
    def __getslice__(self, *args): return _PerceptMesh.vectorvs___getslice__(self, *args)
    def __setslice__(self, *args): return _PerceptMesh.vectorvs___setslice__(self, *args)
    def __delslice__(self, *args): return _PerceptMesh.vectorvs___delslice__(self, *args)
    def __delitem__(self, *args): return _PerceptMesh.vectorvs___delitem__(self, *args)
    def __getitem__(self, *args): return _PerceptMesh.vectorvs___getitem__(self, *args)
    def __setitem__(self, *args): return _PerceptMesh.vectorvs___setitem__(self, *args)
    def append(self, *args): return _PerceptMesh.vectorvs_append(self, *args)
    def empty(self): return _PerceptMesh.vectorvs_empty(self)
    def size(self): return _PerceptMesh.vectorvs_size(self)
    def clear(self): return _PerceptMesh.vectorvs_clear(self)
    def swap(self, *args): return _PerceptMesh.vectorvs_swap(self, *args)
    def get_allocator(self): return _PerceptMesh.vectorvs_get_allocator(self)
    def begin(self): return _PerceptMesh.vectorvs_begin(self)
    def end(self): return _PerceptMesh.vectorvs_end(self)
    def rbegin(self): return _PerceptMesh.vectorvs_rbegin(self)
    def rend(self): return _PerceptMesh.vectorvs_rend(self)
    def pop_back(self): return _PerceptMesh.vectorvs_pop_back(self)
    def erase(self, *args): return _PerceptMesh.vectorvs_erase(self, *args)
    def __init__(self, *args): 
        this = _PerceptMesh.new_vectorvs(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _PerceptMesh.vectorvs_push_back(self, *args)
    def front(self): return _PerceptMesh.vectorvs_front(self)
    def back(self): return _PerceptMesh.vectorvs_back(self)
    def assign(self, *args): return _PerceptMesh.vectorvs_assign(self, *args)
    def resize(self, *args): return _PerceptMesh.vectorvs_resize(self, *args)
    def insert(self, *args): return _PerceptMesh.vectorvs_insert(self, *args)
    def reserve(self, *args): return _PerceptMesh.vectorvs_reserve(self, *args)
    def capacity(self): return _PerceptMesh.vectorvs_capacity(self)
    __swig_destroy__ = _PerceptMesh.delete_vectorvs
vectorvs_swigregister = _PerceptMesh.vectorvs_swigregister
vectorvs_swigregister(vectorvs)

class vectori(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vectori, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vectori, name)
    __repr__ = _swig_repr
    def iterator(self): return _PerceptMesh.vectori_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _PerceptMesh.vectori___nonzero__(self)
    def __bool__(self): return _PerceptMesh.vectori___bool__(self)
    def __len__(self): return _PerceptMesh.vectori___len__(self)
    def pop(self): return _PerceptMesh.vectori_pop(self)
    def __getslice__(self, *args): return _PerceptMesh.vectori___getslice__(self, *args)
    def __setslice__(self, *args): return _PerceptMesh.vectori___setslice__(self, *args)
    def __delslice__(self, *args): return _PerceptMesh.vectori___delslice__(self, *args)
    def __delitem__(self, *args): return _PerceptMesh.vectori___delitem__(self, *args)
    def __getitem__(self, *args): return _PerceptMesh.vectori___getitem__(self, *args)
    def __setitem__(self, *args): return _PerceptMesh.vectori___setitem__(self, *args)
    def append(self, *args): return _PerceptMesh.vectori_append(self, *args)
    def empty(self): return _PerceptMesh.vectori_empty(self)
    def size(self): return _PerceptMesh.vectori_size(self)
    def clear(self): return _PerceptMesh.vectori_clear(self)
    def swap(self, *args): return _PerceptMesh.vectori_swap(self, *args)
    def get_allocator(self): return _PerceptMesh.vectori_get_allocator(self)
    def begin(self): return _PerceptMesh.vectori_begin(self)
    def end(self): return _PerceptMesh.vectori_end(self)
    def rbegin(self): return _PerceptMesh.vectori_rbegin(self)
    def rend(self): return _PerceptMesh.vectori_rend(self)
    def pop_back(self): return _PerceptMesh.vectori_pop_back(self)
    def erase(self, *args): return _PerceptMesh.vectori_erase(self, *args)
    def __init__(self, *args): 
        this = _PerceptMesh.new_vectori(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _PerceptMesh.vectori_push_back(self, *args)
    def front(self): return _PerceptMesh.vectori_front(self)
    def back(self): return _PerceptMesh.vectori_back(self)
    def assign(self, *args): return _PerceptMesh.vectori_assign(self, *args)
    def resize(self, *args): return _PerceptMesh.vectori_resize(self, *args)
    def insert(self, *args): return _PerceptMesh.vectori_insert(self, *args)
    def reserve(self, *args): return _PerceptMesh.vectori_reserve(self, *args)
    def capacity(self): return _PerceptMesh.vectori_capacity(self)
    __swig_destroy__ = _PerceptMesh.delete_vectori
vectori_swigregister = _PerceptMesh.vectori_swigregister
vectori_swigregister(vectori)

class vectord(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vectord, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vectord, name)
    __repr__ = _swig_repr
    def iterator(self): return _PerceptMesh.vectord_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _PerceptMesh.vectord___nonzero__(self)
    def __bool__(self): return _PerceptMesh.vectord___bool__(self)
    def __len__(self): return _PerceptMesh.vectord___len__(self)
    def pop(self): return _PerceptMesh.vectord_pop(self)
    def __getslice__(self, *args): return _PerceptMesh.vectord___getslice__(self, *args)
    def __setslice__(self, *args): return _PerceptMesh.vectord___setslice__(self, *args)
    def __delslice__(self, *args): return _PerceptMesh.vectord___delslice__(self, *args)
    def __delitem__(self, *args): return _PerceptMesh.vectord___delitem__(self, *args)
    def __getitem__(self, *args): return _PerceptMesh.vectord___getitem__(self, *args)
    def __setitem__(self, *args): return _PerceptMesh.vectord___setitem__(self, *args)
    def append(self, *args): return _PerceptMesh.vectord_append(self, *args)
    def empty(self): return _PerceptMesh.vectord_empty(self)
    def size(self): return _PerceptMesh.vectord_size(self)
    def clear(self): return _PerceptMesh.vectord_clear(self)
    def swap(self, *args): return _PerceptMesh.vectord_swap(self, *args)
    def get_allocator(self): return _PerceptMesh.vectord_get_allocator(self)
    def begin(self): return _PerceptMesh.vectord_begin(self)
    def end(self): return _PerceptMesh.vectord_end(self)
    def rbegin(self): return _PerceptMesh.vectord_rbegin(self)
    def rend(self): return _PerceptMesh.vectord_rend(self)
    def pop_back(self): return _PerceptMesh.vectord_pop_back(self)
    def erase(self, *args): return _PerceptMesh.vectord_erase(self, *args)
    def __init__(self, *args): 
        this = _PerceptMesh.new_vectord(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _PerceptMesh.vectord_push_back(self, *args)
    def front(self): return _PerceptMesh.vectord_front(self)
    def back(self): return _PerceptMesh.vectord_back(self)
    def assign(self, *args): return _PerceptMesh.vectord_assign(self, *args)
    def resize(self, *args): return _PerceptMesh.vectord_resize(self, *args)
    def insert(self, *args): return _PerceptMesh.vectord_insert(self, *args)
    def reserve(self, *args): return _PerceptMesh.vectord_reserve(self, *args)
    def capacity(self): return _PerceptMesh.vectord_capacity(self)
    __swig_destroy__ = _PerceptMesh.delete_vectord
vectord_swigregister = _PerceptMesh.vectord_swigregister
vectord_swigregister(vectord)

RCP_WEAK_NO_DEALLOC = _PerceptMesh.RCP_WEAK_NO_DEALLOC
RCP_UNDEFINED_WEAK_NO_DEALLOC = _PerceptMesh.RCP_UNDEFINED_WEAK_NO_DEALLOC
RCP_UNDEFINED_WITH_DEALLOC = _PerceptMesh.RCP_UNDEFINED_WITH_DEALLOC
class RCPComp(_object):
    """Proxy of C++ Teuchos::RCPComp class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, RCPComp, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, RCPComp, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(Teuchos::RCPComp self) -> RCPComp"""
        this = _PerceptMesh.new_RCPComp()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_RCPComp
RCPComp_swigregister = _PerceptMesh.RCPComp_swigregister
RCPComp_swigregister(RCPComp)

class RCPConstComp(_object):
    """Proxy of C++ Teuchos::RCPConstComp class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, RCPConstComp, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, RCPConstComp, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(Teuchos::RCPConstComp self) -> RCPConstComp"""
        this = _PerceptMesh.new_RCPConstComp()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_RCPConstComp
RCPConstComp_swigregister = _PerceptMesh.RCPConstComp_swigregister
RCPConstComp_swigregister(RCPConstComp)

class RCPFunction(_object):
    """Proxy of C++ Teuchos::RCP<(stk::percept::Function)> class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, RCPFunction, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, RCPFunction, name)
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_RCPFunction
    def swap(self, *args):
        """swap(RCPFunction self, RCPFunction r_ptr)"""
        return _PerceptMesh.RCPFunction_swap(self, *args)

    def is_null(self):
        """is_null(RCPFunction self) -> bool"""
        return _PerceptMesh.RCPFunction_is_null(self)

    def __deref__(self):
        """__deref__(RCPFunction self) -> Function"""
        return _PerceptMesh.RCPFunction___deref__(self)

    def __ref__(self):
        """__ref__(RCPFunction self) -> Function"""
        return _PerceptMesh.RCPFunction___ref__(self)

    def get(self):
        """get(RCPFunction self) -> Function"""
        return _PerceptMesh.RCPFunction_get(self)

    def getRawPtr(self):
        """getRawPtr(RCPFunction self) -> Function"""
        return _PerceptMesh.RCPFunction_getRawPtr(self)

    def ptr(self):
        """ptr(RCPFunction self) -> Teuchos::Ptr< stk::percept::Function >"""
        return _PerceptMesh.RCPFunction_ptr(self)

    def getConst(self):
        """getConst(RCPFunction self) -> Teuchos::RCP< stk::percept::Function const >"""
        return _PerceptMesh.RCPFunction_getConst(self)

    def strength(self):
        """strength(RCPFunction self) -> ERCPStrength"""
        return _PerceptMesh.RCPFunction_strength(self)

    def is_valid_ptr(self):
        """is_valid_ptr(RCPFunction self) -> bool"""
        return _PerceptMesh.RCPFunction_is_valid_ptr(self)

    def strong_count(self):
        """strong_count(RCPFunction self) -> int"""
        return _PerceptMesh.RCPFunction_strong_count(self)

    def weak_count(self):
        """weak_count(RCPFunction self) -> int"""
        return _PerceptMesh.RCPFunction_weak_count(self)

    def total_count(self):
        """total_count(RCPFunction self) -> int"""
        return _PerceptMesh.RCPFunction_total_count(self)

    def set_has_ownership(self):
        """set_has_ownership(RCPFunction self)"""
        return _PerceptMesh.RCPFunction_set_has_ownership(self)

    def has_ownership(self):
        """has_ownership(RCPFunction self) -> bool"""
        return _PerceptMesh.RCPFunction_has_ownership(self)

    def release(self):
        """release(RCPFunction self) -> Teuchos::Ptr< stk::percept::Function >"""
        return _PerceptMesh.RCPFunction_release(self)

    def create_weak(self):
        """create_weak(RCPFunction self) -> RCPFunction"""
        return _PerceptMesh.RCPFunction_create_weak(self)

    def create_strong(self):
        """create_strong(RCPFunction self) -> RCPFunction"""
        return _PerceptMesh.RCPFunction_create_strong(self)

    def assert_not_null(self):
        """assert_not_null(RCPFunction self) -> RCPFunction"""
        return _PerceptMesh.RCPFunction_assert_not_null(self)

    def assert_valid_ptr(self):
        """assert_valid_ptr(RCPFunction self) -> RCPFunction"""
        return _PerceptMesh.RCPFunction_assert_valid_ptr(self)

    def debug_assert_not_null(self):
        """debug_assert_not_null(RCPFunction self) -> RCPFunction"""
        return _PerceptMesh.RCPFunction_debug_assert_not_null(self)

    def debug_assert_valid_ptr(self):
        """debug_assert_valid_ptr(RCPFunction self) -> RCPFunction"""
        return _PerceptMesh.RCPFunction_debug_assert_valid_ptr(self)

    def reset(self):
        """reset(RCPFunction self)"""
        return _PerceptMesh.RCPFunction_reset(self)

    def __init__(self, *args): 
        """
        __init__(Teuchos::RCP<(stk::percept::Function)> self, ENull null_arg=null) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self, Function p, bool has_ownership=True) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self, Function p) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self, RCPFunction r_ptr) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self, Function p, Teuchos::ERCPWeakNoDealloc arg3) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self, Function p, Teuchos::ERCPUndefinedWeakNoDealloc arg3) -> RCPFunction
        __init__(Teuchos::RCP<(stk::percept::Function)> self, Function p, RCPNodeHandle const & node) -> RCPFunction
        """
        this = _PerceptMesh.new_RCPFunction(*args)
        try: self.this.append(this)
        except: self.this = this
    def access_private_ptr(self):
        """access_private_ptr(RCPFunction self) -> Function"""
        return _PerceptMesh.RCPFunction_access_private_ptr(self)

    def nonconst_access_private_node(self):
        """nonconst_access_private_node(RCPFunction self) -> RCPNodeHandle &"""
        return _PerceptMesh.RCPFunction_nonconst_access_private_node(self)

    def access_private_node(self):
        """access_private_node(RCPFunction self) -> RCPNodeHandle const &"""
        return _PerceptMesh.RCPFunction_access_private_node(self)

    def value(self, *args):
        """
        value(RCPFunction self, MDArray domain, MDArray MDOutVal, double time_value_optional=0.0)
        value(RCPFunction self, MDArray domain, MDArray MDOutVal)
        value(RCPFunction self, MDArray domain, double time_value_optional=0.0) -> MDArray
        value(RCPFunction self, MDArray domain) -> MDArray

        MDArray
        stk::percept::Function::value(MDArray &domain, double
        time_value_optional=0.0)

        this version creates a dummy output and returns it based on the
        codomain-dimensions specified at construction 
        """
        return _PerceptMesh.RCPFunction_value(self, *args)

    def derivative(self, *args):
        """
        derivative(RCPFunction self, MDArrayString deriv_spec) -> RCPFunction

        virtual
        Teuchos::RCP<Function >
        stk::percept::Function::derivative(MDArrayString &deriv_spec)

        Return a function that is the derivative of this function. The
        derivative is specified as a rank-2 array of strings that specify what
        derivative to take and how many derivatives. For example, 
        """
        return _PerceptMesh.RCPFunction_derivative(self, *args)

    def gradient(self, spatialDim=3):
        """
        gradient(RCPFunction self, int spatialDim=3) -> RCPFunction
        gradient(RCPFunction self) -> RCPFunction

        virtual
        Teuchos::RCP<Function > stk::percept::Function::gradient(int
        spatialDim=3) 
        """
        return _PerceptMesh.RCPFunction_gradient(self, spatialDim)

    def derivativeAtPoint(self, *args):
        """
        derivativeAtPoint(RCPFunction self, MDArrayString deriv_spec, MDArray domain, MDArray codomain, double time=0.0)
        derivativeAtPoint(RCPFunction self, MDArrayString deriv_spec, MDArray domain, MDArray codomain)

        void stk::percept::Function::derivativeAtPoint(MDArrayString
        &deriv_spec, MDArray &domain, MDArray &codomain, double time=0.0) 
        """
        return _PerceptMesh.RCPFunction_derivativeAtPoint(self, *args)

    def setIntegrationOrderDefault(self, *args):
        """setIntegrationOrderDefault(RCPFunction self, unsigned int integration_order)"""
        return _PerceptMesh.RCPFunction_setIntegrationOrderDefault(self, *args)

    def setSpatialDimDefault(self, *args):
        """setSpatialDimDefault(RCPFunction self, unsigned int spatialDim)"""
        return _PerceptMesh.RCPFunction_setSpatialDimDefault(self, *args)

    def setCodomainDimDefault(self, *args):
        """setCodomainDimDefault(RCPFunction self, unsigned int codomainDim)"""
        return _PerceptMesh.RCPFunction_setCodomainDimDefault(self, *args)

    def add_alias(self, *args):
        """
        add_alias(RCPFunction self, char const * alias) -> Function

        Function *
        stk::percept::Function::add_alias(const char *alias)

        allow this function to have one or more aliases 
        """
        return _PerceptMesh.RCPFunction_add_alias(self, *args)

    def getName(self):
        """
        getName(RCPFunction self) -> std::string &

        std::string&
        stk::percept::Function::getName() 
        """
        return _PerceptMesh.RCPFunction_getName(self)

    def setIntegrationOrder(self, *args):
        """
        setIntegrationOrder(RCPFunction self, unsigned int iord)

        void stk::percept::Function::setIntegrationOrder(unsigned iord) 
        """
        return _PerceptMesh.RCPFunction_setIntegrationOrder(self, *args)

    def getIntegrationOrder(self):
        """
        getIntegrationOrder(RCPFunction self) -> unsigned int

        unsigned stk::percept::Function::getIntegrationOrder(void) 
        """
        return _PerceptMesh.RCPFunction_getIntegrationOrder(self)

    def setDomainDimensions(self, *args):
        """
        setDomainDimensions(RCPFunction self, Dimensions dims)

        void stk::percept::Function::setDomainDimensions(const Dimensions
        dims) 
        """
        return _PerceptMesh.RCPFunction_setDomainDimensions(self, *args)

    def setCodomainDimensions(self, *args):
        """
        setCodomainDimensions(RCPFunction self, Dimensions dims)

        void stk::percept::Function::setCodomainDimensions(const Dimensions
        dims) 
        """
        return _PerceptMesh.RCPFunction_setCodomainDimensions(self, *args)

    __swig_getmethods__["Identity"] = _PerceptMesh.RCPFunction_Identity_get
    if _newclass:Identity = _swig_property(_PerceptMesh.RCPFunction_Identity_get)
    def argsAreValid(self, *args):
        """
        argsAreValid(RCPFunction self, MDArray _in, MDArray out) -> bool

        bool
        stk::percept::Function::argsAreValid(const MDArray &in, const MDArray
        &out)

        Verify that the last dimensions of

        Parameters:
        -----------

        in:  and

        out:  are the same; this allows Functions to be invoked at multiple
        points where the first M indices represent an M-d array of points to
        evaluate the function, while the last N indices should match the
        Functions domain and codomain dimensions. 
        """
        return _PerceptMesh.RCPFunction_argsAreValid(self, *args)

    def isSpatialOperator(self):
        """
        isSpatialOperator(RCPFunction self) -> bool

        bool
        stk::percept::GenericFunction::isSpatialOperator() 
        """
        return _PerceptMesh.RCPFunction_isSpatialOperator(self)

    def setIsSpatialOperator(self, *args):
        """
        setIsSpatialOperator(RCPFunction self, bool so)

        void
        stk::percept::GenericFunction::setIsSpatialOperator(bool so) 
        """
        return _PerceptMesh.RCPFunction_setIsSpatialOperator(self, *args)

    def getDomainDimensions(self):
        """
        getDomainDimensions(RCPFunction self) -> Dimensions

        Dimensions
        stk::percept::GenericFunction::getDomainDimensions() 
        """
        return _PerceptMesh.RCPFunction_getDomainDimensions(self)

    def getCodomainDimensions(self):
        """
        getCodomainDimensions(RCPFunction self) -> Dimensions

        Dimensions
        stk::percept::GenericFunction::getCodomainDimensions() 
        """
        return _PerceptMesh.RCPFunction_getCodomainDimensions(self)

    def getNewDomain(self):
        """
        getNewDomain(RCPFunction self) -> MDArray

        MDArray stk::percept::GenericFunction::getNewDomain() 
        """
        return _PerceptMesh.RCPFunction_getNewDomain(self)

    def getNewCodomain(self):
        """
        getNewCodomain(RCPFunction self) -> MDArray

        MDArray stk::percept::GenericFunction::getNewCodomain() const 
        """
        return _PerceptMesh.RCPFunction_getNewCodomain(self)

    def getNewMDArray(self, *args):
        """getNewMDArray(RCPFunction self, Dimensions dims) -> MDArray"""
        return _PerceptMesh.RCPFunction_getNewMDArray(self, *args)

RCPFunction_swigregister = _PerceptMesh.RCPFunction_swigregister
RCPFunction_swigregister(RCPFunction)


def parallel_machine_null():
  """parallel_machine_null() -> stk::ParallelMachine"""
  return _PerceptMesh.parallel_machine_null()

def parallel_machine_init(*args):
  """parallel_machine_init(int * argc, char *** argv) -> stk::ParallelMachine"""
  return _PerceptMesh.parallel_machine_init(*args)

def parallel_machine_finalize():
  """parallel_machine_finalize()"""
  return _PerceptMesh.parallel_machine_finalize()

def parallel_machine_size(*args):
  """parallel_machine_size(stk::ParallelMachine parallel_machine) -> unsigned int"""
  return _PerceptMesh.parallel_machine_size(*args)

def parallel_machine_rank(*args):
  """parallel_machine_rank(stk::ParallelMachine parallel_machine) -> unsigned int"""
  return _PerceptMesh.parallel_machine_rank(*args)

def parallel_machine_barrier(*args):
  """parallel_machine_barrier(stk::ParallelMachine parallel_machine)"""
  return _PerceptMesh.parallel_machine_barrier(*args)
class WriterThrowSafe(_object):
    """Proxy of C++ stk::diag::WriterThrowSafe class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, WriterThrowSafe, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, WriterThrowSafe, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(stk::diag::WriterThrowSafe self, Writer & writer) -> WriterThrowSafe"""
        this = _PerceptMesh.new_WriterThrowSafe(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_WriterThrowSafe
WriterThrowSafe_swigregister = _PerceptMesh.WriterThrowSafe_swigregister
WriterThrowSafe_swigregister(WriterThrowSafe)

class Writer(_object):
    """Proxy of C++ stk::diag::Writer class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Writer, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Writer, name)
    __repr__ = _swig_repr
    DISABLED = _PerceptMesh.Writer_DISABLED
    ENABLED = _PerceptMesh.Writer_ENABLED
    def __init__(self, *args): 
        """
        __init__(stk::diag::Writer self, std::streambuf * streambuf, PrintMask print_mask=static_cast< PrintMask >(LOG_MEMBERS), 
            stk::diag::Writer::Flags flags=static_cast< stk::diag::Writer::Flags >(ENABLED)) -> Writer
        __init__(stk::diag::Writer self, std::streambuf * streambuf, PrintMask print_mask=static_cast< PrintMask >(LOG_MEMBERS)) -> Writer
        __init__(stk::diag::Writer self, std::streambuf * streambuf) -> Writer
        """
        this = _PerceptMesh.new_Writer(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_Writer
    def getStream(self):
        """getStream(Writer self) -> std::ostream &"""
        return _PerceptMesh.Writer_getStream(self)

    def setFlags(self, *args):
        """setFlags(Writer self, int flags) -> Writer"""
        return _PerceptMesh.Writer_setFlags(self, *args)

    def getFlags(self):
        """getFlags(Writer self) -> int"""
        return _PerceptMesh.Writer_getFlags(self)

    def getDepth(self):
        """getDepth(Writer self) -> int"""
        return _PerceptMesh.Writer_getDepth(self)

    def restoreDepth(self, *args):
        """restoreDepth(Writer self, int depth) -> Writer"""
        return _PerceptMesh.Writer_restoreDepth(self, *args)

    def setPrintMask(self, mask=0):
        """
        setPrintMask(Writer self, PrintMask mask=0) -> Writer
        setPrintMask(Writer self) -> Writer
        """
        return _PerceptMesh.Writer_setPrintMask(self, mask)

    def setLineMask(self, *args):
        """setLineMask(Writer self, PrintMask line_mask) -> Writer"""
        return _PerceptMesh.Writer_setLineMask(self, *args)

    def m(self, *args):
        """m(Writer self, PrintMask line_mask) -> Writer"""
        return _PerceptMesh.Writer_m(self, *args)

    def w(self, *args):
        """w(Writer self, bool on, PrintMask line_mask) -> Writer"""
        return _PerceptMesh.Writer_w(self, *args)

    def t(self, line_mask=0):
        """
        t(Writer self, PrintMask line_mask=0) -> Writer
        t(Writer self) -> Writer
        """
        return _PerceptMesh.Writer_t(self, line_mask)

    def getPrintMask(self):
        """getPrintMask(Writer self) -> PrintMask"""
        return _PerceptMesh.Writer_getPrintMask(self)

    def isEnabled(self):
        """isEnabled(Writer self) -> bool"""
        return _PerceptMesh.Writer_isEnabled(self)

    def isLoggable(self, *args):
        """isLoggable(Writer self, PrintMask line_mask) -> bool"""
        return _PerceptMesh.Writer_isLoggable(self, *args)

    def shouldPrint(self, *args):
        """
        shouldPrint(Writer self) -> bool
        shouldPrint(Writer self, PrintMask line_mask) -> bool
        """
        return _PerceptMesh.Writer_shouldPrint(self, *args)

    def shouldTrace(self, *args):
        """shouldTrace(Writer self, int line_mask) -> bool"""
        return _PerceptMesh.Writer_shouldTrace(self, *args)

    def dflush(self):
        """dflush(Writer self) -> Writer"""
        return _PerceptMesh.Writer_dflush(self)

    def dendl(self):
        """dendl(Writer self) -> Writer"""
        return _PerceptMesh.Writer_dendl(self)

    def push(self):
        """push(Writer self) -> Writer"""
        return _PerceptMesh.Writer_push(self)

    def pop(self):
        """pop(Writer self) -> Writer"""
        return _PerceptMesh.Writer_pop(self)

    def resetLineMask(self):
        """resetLineMask(Writer self) -> Writer"""
        return _PerceptMesh.Writer_resetLineMask(self)

    def incTraceDepth(self):
        """incTraceDepth(Writer self) -> int"""
        return _PerceptMesh.Writer_incTraceDepth(self)

    def decTraceDepth(self):
        """decTraceDepth(Writer self) -> int"""
        return _PerceptMesh.Writer_decTraceDepth(self)

    def isTracing(self):
        """isTracing(Writer self) -> bool"""
        return _PerceptMesh.Writer_isTracing(self)

    def isTraceable(self):
        """isTraceable(Writer self) -> bool"""
        return _PerceptMesh.Writer_isTraceable(self)

Writer_swigregister = _PerceptMesh.Writer_swigregister
Writer_swigregister(Writer)


def dendl(*args):
  """dendl(Writer dout) -> Writer"""
  return _PerceptMesh.dendl(*args)

def dflush(*args):
  """dflush(Writer dout) -> Writer"""
  return _PerceptMesh.dflush(*args)

def push(*args):
  """push(Writer dout) -> Writer"""
  return _PerceptMesh.push(*args)

def pop(*args):
  """pop(Writer dout) -> Writer"""
  return _PerceptMesh.pop(*args)
class _setlinemask(_object):
    """Proxy of C++ stk::diag::_setlinemask class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, _setlinemask, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, _setlinemask, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(stk::diag::_setlinemask self, PrintMask line_mask) -> _setlinemask"""
        this = _PerceptMesh.new__setlinemask(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_setmethods__["m_lineMask"] = _PerceptMesh._setlinemask_m_lineMask_set
    __swig_getmethods__["m_lineMask"] = _PerceptMesh._setlinemask_m_lineMask_get
    if _newclass:m_lineMask = _swig_property(_PerceptMesh._setlinemask_m_lineMask_get, _PerceptMesh._setlinemask_m_lineMask_set)
    __swig_destroy__ = _PerceptMesh.delete__setlinemask
_setlinemask_swigregister = _PerceptMesh._setlinemask_swigregister
_setlinemask_swigregister(_setlinemask)


def setlinemask(*args):
  """setlinemask(PrintMask line_mask) -> _setlinemask"""
  return _PerceptMesh.setlinemask(*args)

def resetlinemask(*args):
  """resetlinemask(Writer dout) -> Writer"""
  return _PerceptMesh.resetlinemask(*args)

def include_entity(*args):
  """include_entity(Ioss::GroupingEntity const * entity) -> bool"""
  return _PerceptMesh.include_entity(*args)

def internal_part_processing(*args):
  """
    internal_part_processing(Ioss::GroupingEntity * entity, FEMMetaData meta)
    internal_part_processing(Ioss::EntityBlock * entity, FEMMetaData meta)
    internal_part_processing(Ioss::GroupingEntity * entity, stk::mesh::MetaData & meta)
    internal_part_processing(Ioss::EntityBlock * entity, stk::mesh::MetaData & meta)
    """
  return _PerceptMesh.internal_part_processing(*args)

def define_output_db(*args):
  """
    define_output_db(Ioss::Region & io_region, mesh::BulkData const & bulk_data, Ioss::Region const * input_region=None, 
        Selector anded_selector=None, bool const sort_stk_parts=False)
    define_output_db(Ioss::Region & io_region, mesh::BulkData const & bulk_data, Ioss::Region const * input_region=None, 
        Selector anded_selector=None)
    define_output_db(Ioss::Region & io_region, mesh::BulkData const & bulk_data, Ioss::Region const * input_region=None)
    define_output_db(Ioss::Region & io_region, mesh::BulkData const & bulk_data)
    """
  return _PerceptMesh.define_output_db(*args)

def write_output_db(*args):
  """
    write_output_db(Ioss::Region & io_region, mesh::BulkData const & bulk, Selector anded_selector=None)
    write_output_db(Ioss::Region & io_region, mesh::BulkData const & bulk)
    """
  return _PerceptMesh.write_output_db(*args)

def is_valid_part_field(*args):
  """
    is_valid_part_field(FieldBase field, stk::mesh::EntityRank const part_type, Part part, Part universal, 
        Ioss::Field::RoleType const filter_role, bool const add_all=False) -> bool
    is_valid_part_field(FieldBase field, stk::mesh::EntityRank const part_type, Part part, Part universal, 
        Ioss::Field::RoleType const filter_role) -> bool
    """
  return _PerceptMesh.is_valid_part_field(*args)

def ioss_add_fields(*args):
  """
    ioss_add_fields(Part part, stk::mesh::EntityRank const part_type, Ioss::GroupingEntity * entity, 
        Ioss::Field::RoleType const filter_role, bool const add_all=False)
    ioss_add_fields(Part part, stk::mesh::EntityRank const part_type, Ioss::GroupingEntity * entity, 
        Ioss::Field::RoleType const filter_role)
    """
  return _PerceptMesh.ioss_add_fields(*args)

def define_io_fields(*args):
  """define_io_fields(Ioss::GroupingEntity * entity, Ioss::Field::RoleType role, Part part, stk::mesh::EntityRank part_type)"""
  return _PerceptMesh.define_io_fields(*args)

def map_topology_ioss_to_cell(*args):
  """map_topology_ioss_to_cell(Ioss::ElementTopology const * topology) -> CellTopologyData const *"""
  return _PerceptMesh.map_topology_ioss_to_cell(*args)

def map_topology_cell_to_ioss(*args):
  """map_topology_cell_to_ioss(CellTopologyData const * cell_top, int spatial_dimension) -> std::string"""
  return _PerceptMesh.map_topology_cell_to_ioss(*args)

def get_entity_list(*args):
  """
    get_entity_list(Ioss::GroupingEntity * io_entity, stk::mesh::EntityRank part_type, BulkData bulk, 
        std::vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > > & entities)
    """
  return _PerceptMesh.get_entity_list(*args)

def field_data_from_ioss(*args):
  """
    field_data_from_ioss(FieldBase field, std::vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > > & entities, 
        Ioss::GroupingEntity * io_entity, std::string const & io_fld_name)
    """
  return _PerceptMesh.field_data_from_ioss(*args)

def field_data_to_ioss(*args):
  """
    field_data_to_ioss(FieldBase field, std::vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > > & entities, 
        Ioss::GroupingEntity * io_entity, std::string const & io_fld_name, 
        Ioss::Field::RoleType filter_role)
    """
  return _PerceptMesh.field_data_to_ioss(*args)

def get_distribution_factor_field(*args):
  """get_distribution_factor_field(mesh::Part const & p) -> mesh::Field< double,mesh::ElementNode,void,void,void,void,void,void > const *"""
  return _PerceptMesh.get_distribution_factor_field(*args)

def set_distribution_factor_field(*args):
  """set_distribution_factor_field(mesh::Part & p, mesh::Field< double,mesh::ElementNode,void,void,void,void,void,void > const & df_field)"""
  return _PerceptMesh.set_distribution_factor_field(*args)

def get_field_role(*args):
  """get_field_role(mesh::FieldBase const & f) -> Ioss::Field::RoleType const *"""
  return _PerceptMesh.get_field_role(*args)

def set_field_role(*args):
  """set_field_role(mesh::FieldBase & f, Ioss::Field::RoleType const & role)"""
  return _PerceptMesh.set_field_role(*args)

def is_part_io_part(*args):
  """is_part_io_part(mesh::Part & part) -> bool"""
  return _PerceptMesh.is_part_io_part(*args)

def put_io_part_attribute(*args):
  """
    put_io_part_attribute(mesh::Part & part, Ioss::GroupingEntity * entity=None)
    put_io_part_attribute(mesh::Part & part)
    """
  return _PerceptMesh.put_io_part_attribute(*args)

def remove_io_part_attribute(*args):
  """remove_io_part_attribute(mesh::Part & part)"""
  return _PerceptMesh.remove_io_part_attribute(*args)

def get_associated_ioss_entity(*args):
  """get_associated_ioss_entity(mesh::Part const & part) -> Ioss::GroupingEntity const *"""
  return _PerceptMesh.get_associated_ioss_entity(*args)

def db_api_int_size(*args):
  """db_api_int_size(Ioss::GroupingEntity const * entity) -> size_t"""
  return _PerceptMesh.db_api_int_size(*args)

def invalid_rank(*args):
  """invalid_rank(mesh::EntityRank rank) -> bool"""
  return _PerceptMesh.invalid_rank(*args)

def part_primary_entity_rank(*args):
  """part_primary_entity_rank(mesh::Part const & part) -> mesh::EntityRank"""
  return _PerceptMesh.part_primary_entity_rank(*args)

def element_rank(*args):
  """element_rank(mesh::MetaData const & meta) -> mesh::EntityRank"""
  return _PerceptMesh.element_rank(*args)

def side_rank(*args):
  """side_rank(mesh::MetaData const & meta) -> mesh::EntityRank"""
  return _PerceptMesh.side_rank(*args)

def face_rank(*args):
  """face_rank(mesh::MetaData const & meta) -> mesh::EntityRank"""
  return _PerceptMesh.face_rank(*args)

def edge_rank(*args):
  """edge_rank(mesh::MetaData const & meta) -> mesh::EntityRank"""
  return _PerceptMesh.edge_rank(*args)

def node_rank(*args):
  """node_rank(mesh::MetaData const & meta) -> mesh::EntityRank"""
  return _PerceptMesh.node_rank(*args)

def initialize_spatial_dimension(*args):
  """
    initialize_spatial_dimension(mesh::fem::FEMMetaData & fem_meta, size_t spatial_dimension, vectors entity_rank_names)
    initialize_spatial_dimension(mesh::MetaData & meta, size_t spatial_dimension, vectors entity_rank_names)
    """
  return _PerceptMesh.initialize_spatial_dimension(*args)

def get_io_field_type(*args):
  """get_io_field_type(FieldBase field, stk::mesh::FieldRestriction const & res, std::pair< std::string,Ioss::Field::BasicType > * result)"""
  return _PerceptMesh.get_io_field_type(*args)
class Gmesh_STKmesh_Fixture(_object):
    """Proxy of C++ stk::io::util::Gmesh_STKmesh_Fixture class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Gmesh_STKmesh_Fixture, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Gmesh_STKmesh_Fixture, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(stk::io::util::Gmesh_STKmesh_Fixture self, stk::ParallelMachine comm, std::string const & gmesh_spec) -> Gmesh_STKmesh_Fixture"""
        this = _PerceptMesh.new_Gmesh_STKmesh_Fixture(*args)
        try: self.this.append(this)
        except: self.this = this
    def commit(self):
        """commit(Gmesh_STKmesh_Fixture self)"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_commit(self)

    def getSurfElemCount(self, *args):
        """getSurfElemCount(Gmesh_STKmesh_Fixture self, size_t surf_id) -> size_t"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getSurfElemCount(self, *args)

    def getSurfCoordInfo(self, *args):
        """getSurfCoordInfo(Gmesh_STKmesh_Fixture self, size_t surf_id) -> std::pair< int,double >"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getSurfCoordInfo(self, *args)

    def getSideCount(self):
        """getSideCount(Gmesh_STKmesh_Fixture self) -> size_t"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getSideCount(self)

    def getElemCount(self):
        """getElemCount(Gmesh_STKmesh_Fixture self) -> size_t"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getElemCount(self)

    def getNodeCount(self):
        """getNodeCount(Gmesh_STKmesh_Fixture self) -> size_t"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getNodeCount(self)

    def getSidesetNames(self):
        """getSidesetNames(Gmesh_STKmesh_Fixture self) -> vectors"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getSidesetNames(self)

    def getSideParts(self):
        """getSideParts(Gmesh_STKmesh_Fixture self) -> vectorparts"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getSideParts(self)

    def getMetaData(self):
        """getMetaData(Gmesh_STKmesh_Fixture self) -> stk::mesh::MetaData &"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getMetaData(self)

    def getFEMMetaData(self):
        """getFEMMetaData(Gmesh_STKmesh_Fixture self) -> FEMMetaData"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getFEMMetaData(self)

    def getBulkData(self):
        """getBulkData(Gmesh_STKmesh_Fixture self) -> BulkData"""
        return _PerceptMesh.Gmesh_STKmesh_Fixture_getBulkData(self)

    __swig_destroy__ = _PerceptMesh.delete_Gmesh_STKmesh_Fixture
Gmesh_STKmesh_Fixture_swigregister = _PerceptMesh.Gmesh_STKmesh_Fixture_swigregister
Gmesh_STKmesh_Fixture_swigregister(Gmesh_STKmesh_Fixture)

class FieldBase(_object):
    """Proxy of C++ stk::mesh::FieldBase class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, FieldBase, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, FieldBase, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    def mesh_meta_data(self):
        """mesh_meta_data(FieldBase self) -> MetaData &"""
        return _PerceptMesh.FieldBase_mesh_meta_data(self)

    def mesh_meta_data_ordinal(self):
        """mesh_meta_data_ordinal(FieldBase self) -> unsigned int"""
        return _PerceptMesh.FieldBase_mesh_meta_data_ordinal(self)

    def name(self):
        """name(FieldBase self) -> std::string const &"""
        return _PerceptMesh.FieldBase_name(self)

    def data_traits(self):
        """data_traits(FieldBase self) -> DataTraits const &"""
        return _PerceptMesh.FieldBase_data_traits(self)

    def number_of_states(self):
        """number_of_states(FieldBase self) -> unsigned int"""
        return _PerceptMesh.FieldBase_number_of_states(self)

    def state(self):
        """state(FieldBase self) -> FieldState"""
        return _PerceptMesh.FieldBase_state(self)

    def rank(self):
        """rank(FieldBase self) -> unsigned int"""
        return _PerceptMesh.FieldBase_rank(self)

    def dimension_tags(self):
        """dimension_tags(FieldBase self) -> shards::ArrayDimTag const *const *"""
        return _PerceptMesh.FieldBase_dimension_tags(self)

    def max_size(self, *args):
        """max_size(FieldBase self, EntityRank entity_rank) -> unsigned int"""
        return _PerceptMesh.FieldBase_max_size(self, *args)

    def restrictions(self):
        """restrictions(FieldBase self) -> stk::mesh::FieldBase::RestrictionVector const &"""
        return _PerceptMesh.FieldBase_restrictions(self)

    def selector_restrictions(self):
        """selector_restrictions(FieldBase self) -> stk::mesh::FieldBase::RestrictionVector const &"""
        return _PerceptMesh.FieldBase_selector_restrictions(self)

    def restriction(self, *args):
        """restriction(FieldBase self, unsigned int entity_rank, Part const & part) -> stk::mesh::FieldBase::Restriction const &"""
        return _PerceptMesh.FieldBase_restriction(self, *args)

    def field_state(self, *args):
        """field_state(FieldBase self, FieldState fstate) -> FieldBase"""
        return _PerceptMesh.FieldBase_field_state(self, *args)

    def get_initial_value(self, *args):
        """
        get_initial_value(FieldBase self) -> void const
        get_initial_value(FieldBase self) -> void *
        """
        return _PerceptMesh.FieldBase_get_initial_value(self, *args)

    def get_initial_value_num_bytes(self):
        """get_initial_value_num_bytes(FieldBase self) -> unsigned int"""
        return _PerceptMesh.FieldBase_get_initial_value_num_bytes(self)

    __swig_destroy__ = _PerceptMesh.delete_FieldBase
FieldBase_swigregister = _PerceptMesh.FieldBase_swigregister
FieldBase_swigregister(FieldBase)

class Part(_object):
    """Proxy of C++ stk::mesh::Part class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Part, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Part, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    def mesh_meta_data(self):
        """mesh_meta_data(Part self) -> MetaData &"""
        return _PerceptMesh.Part_mesh_meta_data(self)

    def primary_entity_rank(self):
        """primary_entity_rank(Part self) -> unsigned int"""
        return _PerceptMesh.Part_primary_entity_rank(self)

    def name(self):
        """name(Part self) -> std::string const &"""
        return _PerceptMesh.Part_name(self)

    def mesh_meta_data_ordinal(self):
        """mesh_meta_data_ordinal(Part self) -> unsigned int"""
        return _PerceptMesh.Part_mesh_meta_data_ordinal(self)

    def supersets(self):
        """supersets(Part self) -> PartVector const &"""
        return _PerceptMesh.Part_supersets(self)

    def subsets(self):
        """subsets(Part self) -> PartVector const &"""
        return _PerceptMesh.Part_subsets(self)

    def intersection_of(self):
        """intersection_of(Part self) -> PartVector const &"""
        return _PerceptMesh.Part_intersection_of(self)

    def relations(self):
        """relations(Part self) -> std::vector< PartRelation,std::allocator< PartRelation > > const &"""
        return _PerceptMesh.Part_relations(self)

    def __eq__(self, *args):
        """__eq__(Part self, Part rhs) -> bool"""
        return _PerceptMesh.Part___eq__(self, *args)

    def __ne__(self, *args):
        """__ne__(Part self, Part rhs) -> bool"""
        return _PerceptMesh.Part___ne__(self, *args)

Part_swigregister = _PerceptMesh.Part_swigregister
Part_swigregister(Part)


def is_internal(*args):
  """
    is_internal(std::string const & part_name) -> bool
    is_internal(Part part) -> bool
    """
  return _PerceptMesh.is_internal(*args)

def convert_to_internal_name(*args):
  """convert_to_internal_name(std::string const & part_name) -> std::string"""
  return _PerceptMesh.convert_to_internal_name(*args)
class PartLess(_object):
    """Proxy of C++ stk::mesh::PartLess class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, PartLess, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, PartLess, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::mesh::PartLess self) -> PartLess"""
        this = _PerceptMesh.new_PartLess()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_PartLess
PartLess_swigregister = _PerceptMesh.PartLess_swigregister
PartLess_swigregister(PartLess)
cvar = _PerceptMesh.cvar
INTERNAL_PART_PREFIX = cvar.INTERNAL_PART_PREFIX
INTERNAL_PART_POSTFIX = cvar.INTERNAL_PART_POSTFIX


def order(*args):
  """
    order(PartVector & arg1)
    order(OrdinalVector & v)
    """
  return _PerceptMesh.order(*args)

def insert(*args):
  """insert(PartVector & arg1, Part arg2) -> bool"""
  return _PerceptMesh.insert(*args)

def insert_ordinal(*args):
  """insert_ordinal(OrdinalVector & v, unsigned int part_ordinal) -> bool"""
  return _PerceptMesh.insert_ordinal(*args)

def remove(*args):
  """remove(PartVector & arg1, Part arg2)"""
  return _PerceptMesh.remove(*args)

def find(*args):
  """find(PartVector const & arg1, std::string const & arg2) -> Part"""
  return _PerceptMesh.find(*args)

def contain(*args):
  """
    contain(PartVector const & arg1, Part arg2) -> bool
    contain(PartVector const & arg1, PartVector const & arg2) -> bool
    """
  return _PerceptMesh.contain(*args)

def intersect(*args):
  """
    intersect(PartVector const & arg1, PartVector const & arg2) -> size_t
    intersect(PartVector const & arg1, PartVector const & arg2, PartVector & arg3) -> size_t
    intersect(Part arg1, Part arg2) -> bool
    """
  return _PerceptMesh.intersect(*args)
MaximumFieldDimension = _PerceptMesh.MaximumFieldDimension
EntityLogNoChange = _PerceptMesh.EntityLogNoChange
EntityLogCreated = _PerceptMesh.EntityLogCreated
EntityLogModified = _PerceptMesh.EntityLogModified
EntityLogDeleted = _PerceptMesh.EntityLogDeleted
class PartOrdLess(_object):
    """Proxy of C++ stk::mesh::PartOrdLess class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, PartOrdLess, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, PartOrdLess, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::mesh::PartOrdLess self) -> PartOrdLess"""
        this = _PerceptMesh.new_PartOrdLess()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_PartOrdLess
PartOrdLess_swigregister = _PerceptMesh.PartOrdLess_swigregister
PartOrdLess_swigregister(PartOrdLess)
InvalidOrdinal = cvar.InvalidOrdinal
BaseEntityRank = cvar.BaseEntityRank
InvalidEntityRank = cvar.InvalidEntityRank
InvalidPartOrdinal = cvar.InvalidPartOrdinal
InvalidRelationIdentifier = cvar.InvalidRelationIdentifier

INVALID = _PerceptMesh.INVALID
COMPOUND = _PerceptMesh.COMPOUND
PART_ID = _PerceptMesh.PART_ID
class OpType(_object):
    """Proxy of C++ stk::mesh::OpType class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, OpType, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, OpType, name)
    __repr__ = _swig_repr
    __swig_setmethods__["m_part_id"] = _PerceptMesh.OpType_m_part_id_set
    __swig_getmethods__["m_part_id"] = _PerceptMesh.OpType_m_part_id_get
    if _newclass:m_part_id = _swig_property(_PerceptMesh.OpType_m_part_id_get, _PerceptMesh.OpType_m_part_id_set)
    __swig_setmethods__["m_unary"] = _PerceptMesh.OpType_m_unary_set
    __swig_getmethods__["m_unary"] = _PerceptMesh.OpType_m_unary_get
    if _newclass:m_unary = _swig_property(_PerceptMesh.OpType_m_unary_get, _PerceptMesh.OpType_m_unary_set)
    __swig_setmethods__["m_count"] = _PerceptMesh.OpType_m_count_set
    __swig_getmethods__["m_count"] = _PerceptMesh.OpType_m_count_get
    if _newclass:m_count = _swig_property(_PerceptMesh.OpType_m_count_get, _PerceptMesh.OpType_m_count_set)
    __swig_setmethods__["m_op"] = _PerceptMesh.OpType_m_op_set
    __swig_getmethods__["m_op"] = _PerceptMesh.OpType_m_op_get
    if _newclass:m_op = _swig_property(_PerceptMesh.OpType_m_op_get, _PerceptMesh.OpType_m_op_set)
    def __init__(self, *args): 
        """
        __init__(stk::mesh::OpType self) -> OpType
        __init__(stk::mesh::OpType self, unsigned int part_id, unsigned int unary, unsigned int count, stk::mesh::Op op=INVALID) -> OpType
        __init__(stk::mesh::OpType self, unsigned int part_id, unsigned int unary, unsigned int count) -> OpType
        """
        this = _PerceptMesh.new_OpType(*args)
        try: self.this.append(this)
        except: self.this = this
    def __eq__(self, *args):
        """__eq__(OpType self, OpType opType) -> bool"""
        return _PerceptMesh.OpType___eq__(self, *args)

    def __ne__(self, *args):
        """__ne__(OpType self, OpType opType) -> bool"""
        return _PerceptMesh.OpType___ne__(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_OpType
OpType_swigregister = _PerceptMesh.OpType_swigregister
OpType_swigregister(OpType)

class Selector(_object):
    """Proxy of C++ stk::mesh::Selector class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Selector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Selector, name)
    __repr__ = _swig_repr
    def __eq__(self, *args):
        """__eq__(Selector self, Selector rhs) -> bool"""
        return _PerceptMesh.Selector___eq__(self, *args)

    def __ne__(self, *args):
        """__ne__(Selector self, Selector rhs) -> bool"""
        return _PerceptMesh.Selector___ne__(self, *args)

    def __init__(self, *args): 
        """
        __init__(stk::mesh::Selector self) -> Selector
        __init__(stk::mesh::Selector self, Part part) -> Selector
        """
        this = _PerceptMesh.new_Selector(*args)
        try: self.this.append(this)
        except: self.this = this
    def __iand__(self, *args):
        """__iand__(Selector self, Selector selector) -> Selector"""
        return _PerceptMesh.Selector___iand__(self, *args)

    def __ior__(self, *args):
        """__ior__(Selector self, Selector selector) -> Selector"""
        return _PerceptMesh.Selector___ior__(self, *args)

    def complement(self):
        """complement(Selector self) -> Selector"""
        return _PerceptMesh.Selector_complement(self)

    def get_ops(self):
        """get_ops(Selector self) -> std::vector< stk::mesh::OpType,std::allocator< stk::mesh::OpType > > const &"""
        return _PerceptMesh.Selector_get_ops(self)

    def set_ops(self, *args):
        """set_ops(Selector self, std::vector< stk::mesh::OpType,std::allocator< stk::mesh::OpType > > const & ops)"""
        return _PerceptMesh.Selector_set_ops(self, *args)

    def compoundAll(self):
        """compoundAll(Selector self)"""
        return _PerceptMesh.Selector_compoundAll(self)

    __swig_destroy__ = _PerceptMesh.delete_Selector
Selector_swigregister = _PerceptMesh.Selector_swigregister
Selector_swigregister(Selector)


def __and__(*args):
  """
    __and__(Part A, Part B) -> Selector
    __and__(Part A, Selector B) -> Selector
    __and__(Selector A, Part B) -> Selector
    __and__(Selector A, Selector B) -> Selector
    """
  return _PerceptMesh.__and__(*args)

def __or__(*args):
  """
    __or__(Part A, Part B) -> Selector
    __or__(Part A, Selector B) -> Selector
    __or__(Selector A, Part B) -> Selector
    __or__(Selector A, Selector B) -> Selector
    """
  return _PerceptMesh.__or__(*args)

def selectUnion(*args):
  """selectUnion(vectorparts union_part_vector) -> Selector"""
  return _PerceptMesh.selectUnion(*args)

def selectIntersection(*args):
  """selectIntersection(vectorparts intersection_part_vector) -> Selector"""
  return _PerceptMesh.selectIntersection(*args)

def selectField(*args):
  """selectField(FieldBase field) -> Selector"""
  return _PerceptMesh.selectField(*args)
class BulkData(_object):
    """Proxy of C++ stk::mesh::BulkData class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, BulkData, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, BulkData, name)
    __repr__ = _swig_repr
    def optimize_buckets_at_modification_end(self, *args):
        """optimize_buckets_at_modification_end(BulkData self, bool b)"""
        return _PerceptMesh.BulkData_optimize_buckets_at_modification_end(self, *args)

    def get(*args):
        """
        get(Bucket bucket) -> BulkData
        get(stk::mesh::Entity const & entity) -> BulkData
        get(stk::mesh::Ghosting const & ghost) -> BulkData
        """
        return _PerceptMesh.BulkData_get(*args)

    if _newclass:get = staticmethod(get)
    __swig_getmethods__["get"] = lambda x: get
    MODIFIABLE = _PerceptMesh.BulkData_MODIFIABLE
    SYNCHRONIZED = _PerceptMesh.BulkData_SYNCHRONIZED
    __swig_destroy__ = _PerceptMesh.delete_BulkData
    def __init__(self, *args): 
        """
        __init__(stk::mesh::BulkData self, stk::mesh::MetaData & mesh_meta_data, stk::ParallelMachine parallel, unsigned int bucket_max_size=1000, 
            bool use_memory_pool=True) -> BulkData
        __init__(stk::mesh::BulkData self, stk::mesh::MetaData & mesh_meta_data, stk::ParallelMachine parallel, unsigned int bucket_max_size=1000) -> BulkData
        __init__(stk::mesh::BulkData self, stk::mesh::MetaData & mesh_meta_data, stk::ParallelMachine parallel) -> BulkData
        """
        this = _PerceptMesh.new_BulkData(*args)
        try: self.this.append(this)
        except: self.this = this
    def mesh_meta_data(self):
        """mesh_meta_data(BulkData self) -> stk::mesh::MetaData const &"""
        return _PerceptMesh.BulkData_mesh_meta_data(self)

    def parallel(self):
        """parallel(BulkData self) -> stk::ParallelMachine"""
        return _PerceptMesh.BulkData_parallel(self)

    def parallel_size(self):
        """parallel_size(BulkData self) -> unsigned int"""
        return _PerceptMesh.BulkData_parallel_size(self)

    def parallel_rank(self):
        """parallel_rank(BulkData self) -> unsigned int"""
        return _PerceptMesh.BulkData_parallel_rank(self)

    def synchronized_state(self):
        """synchronized_state(BulkData self) -> stk::mesh::BulkData::BulkDataSyncState"""
        return _PerceptMesh.BulkData_synchronized_state(self)

    def synchronized_count(self):
        """synchronized_count(BulkData self) -> size_t"""
        return _PerceptMesh.BulkData_synchronized_count(self)

    def modification_begin(self):
        """modification_begin(BulkData self) -> bool"""
        return _PerceptMesh.BulkData_modification_begin(self)

    def modification_end(self):
        """modification_end(BulkData self) -> bool"""
        return _PerceptMesh.BulkData_modification_end(self)

    def final_modification_end(self):
        """final_modification_end(BulkData self) -> bool"""
        return _PerceptMesh.BulkData_final_modification_end(self)

    def change_entity_owner(self, *args):
        """change_entity_owner(BulkData self, std::vector< stk::mesh::EntityProc,std::allocator< stk::mesh::EntityProc > > const & arg_change)"""
        return _PerceptMesh.BulkData_change_entity_owner(self, *args)

    def update_field_data_states(self):
        """update_field_data_states(BulkData self)"""
        return _PerceptMesh.BulkData_update_field_data_states(self)

    def copy_entity_fields(self, *args):
        """copy_entity_fields(BulkData self, stk::mesh::Entity const & src, stk::mesh::Entity & dest)"""
        return _PerceptMesh.BulkData_copy_entity_fields(self, *args)

    def buckets(self, *args):
        """buckets(BulkData self, stk::mesh::EntityRank rank) -> std::vector< stk::mesh::Bucket *,std::allocator< stk::mesh::Bucket * > > const &"""
        return _PerceptMesh.BulkData_buckets(self, *args)

    def get_entity(self, *args):
        """
        get_entity(BulkData self, stk::mesh::EntityRank entity_rank, stk::mesh::EntityId entity_id) -> stk::mesh::Entity
        get_entity(BulkData self, EntityKey const key) -> stk::mesh::Entity *
        """
        return _PerceptMesh.BulkData_get_entity(self, *args)

    def declare_entity(self, *args):
        """declare_entity(BulkData self, stk::mesh::EntityRank ent_rank, stk::mesh::EntityId ent_id, vectorparts parts) -> stk::mesh::Entity &"""
        return _PerceptMesh.BulkData_declare_entity(self, *args)

    def change_entity_id(self, *args):
        """change_entity_id(BulkData self, stk::mesh::EntityId id, stk::mesh::Entity & entity)"""
        return _PerceptMesh.BulkData_change_entity_id(self, *args)

    def change_entity_parts(self, *args):
        """
        change_entity_parts(BulkData self, stk::mesh::Entity & entity, vectorparts add_parts, vectorparts remove_parts=stk::mesh::PartVector())
        change_entity_parts(BulkData self, stk::mesh::Entity & entity, vectorparts add_parts)
        """
        return _PerceptMesh.BulkData_change_entity_parts(self, *args)

    def destroy_entity(self, *args):
        """destroy_entity(BulkData self, stk::mesh::Entity *& entity) -> bool"""
        return _PerceptMesh.BulkData_destroy_entity(self, *args)

    def generate_new_entities(self, *args):
        """generate_new_entities(BulkData self, std::vector< size_t,std::allocator< size_t > > const & requests, std::vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > > & requested_entities)"""
        return _PerceptMesh.BulkData_generate_new_entities(self, *args)

    def declare_relation(self, *args):
        """
        declare_relation(BulkData self, stk::mesh::Entity & e_from, stk::mesh::Entity & e_to, stk::mesh::RelationIdentifier const local_id)
        declare_relation(BulkData self, stk::mesh::Entity & entity, std::vector< stk::mesh::Relation,std::allocator< stk::mesh::Relation > > const & rel)
        """
        return _PerceptMesh.BulkData_declare_relation(self, *args)

    def destroy_relation(self, *args):
        """destroy_relation(BulkData self, stk::mesh::Entity & e_from, stk::mesh::Entity & e_to, stk::mesh::RelationIdentifier const local_id) -> bool"""
        return _PerceptMesh.BulkData_destroy_relation(self, *args)

    def shared_aura(self):
        """shared_aura(BulkData self) -> stk::mesh::Ghosting &"""
        return _PerceptMesh.BulkData_shared_aura(self)

    def create_ghosting(self, *args):
        """create_ghosting(BulkData self, std::string const & name) -> stk::mesh::Ghosting &"""
        return _PerceptMesh.BulkData_create_ghosting(self, *args)

    def change_ghosting(self, *args):
        """
        change_ghosting(BulkData self, stk::mesh::Ghosting & ghosts, std::vector< stk::mesh::EntityProc,std::allocator< stk::mesh::EntityProc > > const & add_send, 
            std::vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > > const & remove_receive)
        """
        return _PerceptMesh.BulkData_change_ghosting(self, *args)

    def destroy_all_ghosting(self):
        """destroy_all_ghosting(BulkData self)"""
        return _PerceptMesh.BulkData_destroy_all_ghosting(self)

    def ghostings(self):
        """ghostings(BulkData self) -> std::vector< stk::mesh::Ghosting *,std::allocator< stk::mesh::Ghosting * > > const &"""
        return _PerceptMesh.BulkData_ghostings(self)

    def entity_comm_sharing(self, *args):
        """entity_comm_sharing(BulkData self, EntityKey const & key) -> PairIterEntityComm"""
        return _PerceptMesh.BulkData_entity_comm_sharing(self, *args)

    def entity_comm(self, *args):
        """
        entity_comm(BulkData self) -> std::vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > > const
        entity_comm(BulkData self, EntityKey const & key) -> PairIterEntityComm
        entity_comm(BulkData self, EntityKey const & key, stk::mesh::Ghosting const & sub) -> PairIterEntityComm
        """
        return _PerceptMesh.BulkData_entity_comm(self, *args)

    def entity_comm_insert(self, *args):
        """entity_comm_insert(BulkData self, EntityKey const & key, EntityCommInfo const & val) -> bool"""
        return _PerceptMesh.BulkData_entity_comm_insert(self, *args)

    def entity_comm_erase(self, *args):
        """
        entity_comm_erase(BulkData self, EntityKey const & key, EntityCommInfo const & val) -> bool
        entity_comm_erase(BulkData self, EntityKey const & key, stk::mesh::Ghosting const & ghost) -> bool
        """
        return _PerceptMesh.BulkData_entity_comm_erase(self, *args)

    def entity_comm_clear_ghosting(self, *args):
        """entity_comm_clear_ghosting(BulkData self, EntityKey const & key)"""
        return _PerceptMesh.BulkData_entity_comm_clear_ghosting(self, *args)

    def entity_comm_clear(self, *args):
        """entity_comm_clear(BulkData self, EntityKey const & key)"""
        return _PerceptMesh.BulkData_entity_comm_clear(self, *args)

    def entity_comm_swap(self, *args):
        """entity_comm_swap(BulkData self, EntityKey const & key1, EntityKey const & key2)"""
        return _PerceptMesh.BulkData_entity_comm_swap(self, *args)

BulkData_swigregister = _PerceptMesh.BulkData_swigregister
BulkData_swigregister(BulkData)

def BulkData_get(*args):
  """
    get(Bucket bucket) -> BulkData
    get(stk::mesh::Entity const & entity) -> BulkData
    BulkData_get(stk::mesh::Ghosting const & ghost) -> BulkData
    """
  return _PerceptMesh.BulkData_get(*args)


def set_field_relations(*args):
  """set_field_relations(stk::mesh::Entity & e_from, stk::mesh::Entity & e_to, unsigned int const ident)"""
  return _PerceptMesh.set_field_relations(*args)

def bucket_part_equal(*args):
  """bucket_part_equal(unsigned int const * lhs, unsigned int const * rhs) -> bool"""
  return _PerceptMesh.bucket_part_equal(*args)

def has_superset(*args):
  """
    has_superset(Bucket arg1, Part p) -> bool
    has_superset(Bucket arg1, unsigned int const & ordinal) -> bool
    has_superset(Bucket arg1, vectorparts arg2) -> bool
    """
  return _PerceptMesh.has_superset(*args)
class Bucket(_object):
    """Proxy of C++ stk::mesh::Bucket class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Bucket, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Bucket, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    def begin(self):
        """begin(Bucket self) -> stk::mesh::Bucket::iterator"""
        return _PerceptMesh.Bucket_begin(self)

    def end(self):
        """end(Bucket self) -> stk::mesh::Bucket::iterator"""
        return _PerceptMesh.Bucket_end(self)

    def size(self):
        """size(Bucket self) -> size_t"""
        return _PerceptMesh.Bucket_size(self)

    def capacity(self):
        """capacity(Bucket self) -> size_t"""
        return _PerceptMesh.Bucket_capacity(self)

    def field_data_size(self, *args):
        """field_data_size(Bucket self, FieldBase field) -> unsigned int"""
        return _PerceptMesh.Bucket_field_data_size(self, *args)

    def field_data_stride(self, *args):
        """field_data_stride(Bucket self, FieldBase field) -> stk::mesh::FieldBase::Restriction::size_type const *"""
        return _PerceptMesh.Bucket_field_data_stride(self, *args)

    def fast_field_data_location(self, *args):
        """fast_field_data_location(Bucket self, FieldBase field, unsigned int ordinal) -> unsigned char *"""
        return _PerceptMesh.Bucket_fast_field_data_location(self, *args)

    def field_data_location(self, *args):
        """
        field_data_location(Bucket self, FieldBase field, stk::mesh::Entity const & entity) -> unsigned char
        field_data_location(Bucket self, FieldBase field, unsigned int ordinal) -> unsigned char
        field_data_location(Bucket self, FieldBase field) -> unsigned char *
        """
        return _PerceptMesh.Bucket_field_data_location(self, *args)

    def mesh(self):
        """mesh(Bucket self) -> BulkData"""
        return _PerceptMesh.Bucket_mesh(self)

    def entity_rank(self):
        """entity_rank(Bucket self) -> unsigned int"""
        return _PerceptMesh.Bucket_entity_rank(self)

    def supersets(self, *args):
        """
        supersets(Bucket self, vectorparts arg2)
        supersets(Bucket self, stk::mesh::OrdinalVector & arg2)
        """
        return _PerceptMesh.Bucket_supersets(self, *args)

    def member(self, *args):
        """member(Bucket self, Part arg2) -> bool"""
        return _PerceptMesh.Bucket_member(self, *args)

    def member_all(self, *args):
        """
        member_all(Bucket self, vectorparts arg2) -> bool
        member_all(Bucket self, stk::mesh::OrdinalVector const & arg2) -> bool
        """
        return _PerceptMesh.Bucket_member_all(self, *args)

    def member_any(self, *args):
        """
        member_any(Bucket self, vectorparts arg2) -> bool
        member_any(Bucket self, stk::mesh::OrdinalVector const & arg2) -> bool
        """
        return _PerceptMesh.Bucket_member_any(self, *args)

    def superset_part_ordinals(self):
        """superset_part_ordinals(Bucket self) -> std::pair< unsigned int const *,unsigned int const * >"""
        return _PerceptMesh.Bucket_superset_part_ordinals(self)

    def equivalent(self, *args):
        """equivalent(Bucket self, Bucket b) -> bool"""
        return _PerceptMesh.Bucket_equivalent(self, *args)

    def key(self):
        """key(Bucket self) -> unsigned int const *"""
        return _PerceptMesh.Bucket_key(self)

    def allocation_size(self):
        """allocation_size(Bucket self) -> unsigned int"""
        return _PerceptMesh.Bucket_allocation_size(self)

    def assert_correct(self):
        """assert_correct(Bucket self) -> bool"""
        return _PerceptMesh.Bucket_assert_correct(self)

Bucket_swigregister = _PerceptMesh.Bucket_swigregister
Bucket_swigregister(Bucket)

class BucketLess(_object):
    """Proxy of C++ stk::mesh::BucketLess class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, BucketLess, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, BucketLess, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::mesh::BucketLess self) -> BucketLess"""
        this = _PerceptMesh.new_BucketLess()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_BucketLess
BucketLess_swigregister = _PerceptMesh.BucketLess_swigregister
BucketLess_swigregister(BucketLess)


def lower_bound(*args):
  """lower_bound(std::vector< stk::mesh::Bucket *,std::allocator< stk::mesh::Bucket * > > & v, unsigned int const * key) -> std::vector< stk::mesh::Bucket *,std::allocator< stk::mesh::Bucket * > >::iterator"""
  return _PerceptMesh.lower_bound(*args)
class To_Ptr(_object):
    """Proxy of C++ stk::mesh::To_Ptr class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, To_Ptr, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, To_Ptr, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::mesh::To_Ptr self) -> To_Ptr"""
        this = _PerceptMesh.new_To_Ptr()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_To_Ptr
To_Ptr_swigregister = _PerceptMesh.To_Ptr_swigregister
To_Ptr_swigregister(To_Ptr)

class FieldRepository(_object):
    """Proxy of C++ stk::mesh::impl::FieldRepository class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, FieldRepository, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, FieldRepository, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::mesh::impl::FieldRepository self) -> FieldRepository"""
        this = _PerceptMesh.new_FieldRepository()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_FieldRepository
    def get_field(self, *args):
        """
        get_field(FieldRepository self, char const * arg_method, std::string const & arg_name, DataTraits const & arg_traits, 
            unsigned int arg_rank, shards::ArrayDimTag const *const * arg_dim_tags, 
            unsigned int arg_num_states) -> FieldBase
        """
        return _PerceptMesh.FieldRepository_get_field(self, *args)

    def declare_field(self, *args):
        """
        declare_field(FieldRepository self, std::string const & arg_name, DataTraits const & arg_traits, unsigned int arg_rank, 
            shards::ArrayDimTag const *const * arg_dim_tags, unsigned int arg_num_states, 
            stk::mesh::MetaData * arg_meta_data) -> FieldBase
        """
        return _PerceptMesh.FieldRepository_declare_field(self, *args)

    def verify_and_clean_restrictions(self, *args):
        """verify_and_clean_restrictions(FieldRepository self, char const * arg_method, Part superset, Part subset, vectorparts arg_all_parts)"""
        return _PerceptMesh.FieldRepository_verify_and_clean_restrictions(self, *args)

    def get_fields(self):
        """get_fields(FieldRepository self) -> vectorfieldbase"""
        return _PerceptMesh.FieldRepository_get_fields(self)

    def declare_field_restriction(self, *args):
        """
        declare_field_restriction(FieldRepository self, char const * arg_method, FieldBase arg_field, unsigned int arg_entity_rank, Part arg_part, 
            vectorparts arg_all_parts, unsigned int const * arg_stride, void const * arg_init_value=None)
        declare_field_restriction(FieldRepository self, char const * arg_method, FieldBase arg_field, unsigned int arg_entity_rank, Part arg_part, 
            vectorparts arg_all_parts, unsigned int const * arg_stride)
        declare_field_restriction(FieldRepository self, char const * arg_method, FieldBase arg_field, unsigned int arg_entity_rank, Selector arg_selector, 
            vectorparts arg_all_parts, unsigned int const * arg_stride, 
            void const * arg_init_value=None)
        declare_field_restriction(FieldRepository self, char const * arg_method, FieldBase arg_field, unsigned int arg_entity_rank, Selector arg_selector, 
            vectorparts arg_all_parts, unsigned int const * arg_stride)
        """
        return _PerceptMesh.FieldRepository_declare_field_restriction(self, *args)

FieldRepository_swigregister = _PerceptMesh.FieldRepository_swigregister
FieldRepository_swigregister(FieldRepository)

class FEMMetaData(_object):
    """Proxy of C++ stk::mesh::fem::FEMMetaData class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, FEMMetaData, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, FEMMetaData, name)
    __repr__ = _swig_repr
    INVALID_RANK = _PerceptMesh.FEMMetaData_INVALID_RANK
    NODE_RANK = _PerceptMesh.FEMMetaData_NODE_RANK
    EDGE_RANK = _PerceptMesh.FEMMetaData_EDGE_RANK
    FACE_RANK = _PerceptMesh.FEMMetaData_FACE_RANK
    VOLUME_RANK = _PerceptMesh.FEMMetaData_VOLUME_RANK
    __swig_destroy__ = _PerceptMesh.delete_FEMMetaData
    def __init__(self, *args): 
        """
        __init__(stk::mesh::fem::FEMMetaData self) -> FEMMetaData
        __init__(stk::mesh::fem::FEMMetaData self, size_t spatial_dimension, vectors in_entity_rank_names=std::vector< std::string >()) -> FEMMetaData
        __init__(stk::mesh::fem::FEMMetaData self, size_t spatial_dimension) -> FEMMetaData
        """
        this = _PerceptMesh.new_FEMMetaData(*args)
        try: self.this.append(this)
        except: self.this = this
    def FEM_initialize(self, *args):
        """
        FEM_initialize(FEMMetaData self, size_t spatial_dimension, vectors in_entity_rank_names=std::vector< std::string >())
        FEM_initialize(FEMMetaData self, size_t spatial_dimension)
        """
        return _PerceptMesh.FEMMetaData_FEM_initialize(self, *args)

    def is_FEM_initialized(self):
        """is_FEM_initialized(FEMMetaData self) -> bool"""
        return _PerceptMesh.FEMMetaData_is_FEM_initialized(self)

    def get_meta_data(*args):
        """get_meta_data(FEMMetaData fem_meta) -> stk::mesh::MetaData &"""
        return _PerceptMesh.FEMMetaData_get_meta_data(*args)

    if _newclass:get_meta_data = staticmethod(get_meta_data)
    __swig_getmethods__["get_meta_data"] = lambda x: get_meta_data
    def spatial_dimension(self):
        """spatial_dimension(FEMMetaData self) -> size_t"""
        return _PerceptMesh.FEMMetaData_spatial_dimension(self)

    def node_rank(self):
        """node_rank(FEMMetaData self) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_node_rank(self)

    def edge_rank(self):
        """edge_rank(FEMMetaData self) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_edge_rank(self)

    def face_rank(self):
        """face_rank(FEMMetaData self) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_face_rank(self)

    def volume_rank(self):
        """volume_rank(FEMMetaData self) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_volume_rank(self)

    def side_rank(self):
        """side_rank(FEMMetaData self) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_side_rank(self)

    def element_rank(self):
        """element_rank(FEMMetaData self) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_element_rank(self)

    def check_rank(self, *args):
        """check_rank(FEMMetaData self, stk::mesh::EntityRank rank) -> bool"""
        return _PerceptMesh.FEMMetaData_check_rank(self, *args)

    def register_cell_topology(self, *args):
        """register_cell_topology(FEMMetaData self, stk::mesh::fem::CellTopology const cell_topology, stk::mesh::EntityRank in_entity_rank)"""
        return _PerceptMesh.FEMMetaData_register_cell_topology(self, *args)

    def get_cell_topology_root_part(self, *args):
        """get_cell_topology_root_part(FEMMetaData self, stk::mesh::fem::CellTopology const cell_topology) -> Part"""
        return _PerceptMesh.FEMMetaData_get_cell_topology_root_part(self, *args)

    def get_cell_topology(self, *args):
        """
        get_cell_topology(FEMMetaData self, Part part) -> stk::mesh::fem::CellTopology
        get_cell_topology(FEMMetaData self, std::string const & topology_name) -> stk::mesh::fem::CellTopology
        """
        return _PerceptMesh.FEMMetaData_get_cell_topology(self, *args)

    def get_entity_rank(self, *args):
        """get_entity_rank(FEMMetaData self, stk::mesh::fem::CellTopology const cell_topology) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_get_entity_rank(self, *args)

    def get(*args):
        """
        get(stk::mesh::MetaData const & meta) -> FEMMetaData
        get(Part part) -> FEMMetaData
        get(FieldBase field) -> FEMMetaData
        get(stk::mesh::PropertyBase const & property) -> FEMMetaData
        get(BulkData bulk_data) -> FEMMetaData
        get(Bucket bucket) -> FEMMetaData
        get(stk::mesh::Entity const & entity) -> FEMMetaData
        get(stk::mesh::Ghosting const & ghost) -> FEMMetaData
        """
        return _PerceptMesh.FEMMetaData_get(*args)

    if _newclass:get = staticmethod(get)
    __swig_getmethods__["get"] = lambda x: get
    def universal_part(self):
        """universal_part(FEMMetaData self) -> Part"""
        return _PerceptMesh.FEMMetaData_universal_part(self)

    def locally_owned_part(self):
        """locally_owned_part(FEMMetaData self) -> Part"""
        return _PerceptMesh.FEMMetaData_locally_owned_part(self)

    def globally_shared_part(self):
        """globally_shared_part(FEMMetaData self) -> Part"""
        return _PerceptMesh.FEMMetaData_globally_shared_part(self)

    def get_part(self, *args):
        """
        get_part(FEMMetaData self, std::string const & p_name, char const * required_by=None) -> Part
        get_part(FEMMetaData self, std::string const & p_name) -> Part
        get_part(FEMMetaData self, unsigned int ord) -> Part
        """
        return _PerceptMesh.FEMMetaData_get_part(self, *args)

    def get_parts(self):
        """get_parts(FEMMetaData self) -> vectorparts"""
        return _PerceptMesh.FEMMetaData_get_parts(self)

    def declare_part(self, *args):
        """
        declare_part(FEMMetaData self, std::string const & name, stk::mesh::fem::CellTopology cell_topology) -> Part
        declare_part(FEMMetaData self, std::string const & p_name, stk::mesh::EntityRank rank) -> Part
        declare_part(FEMMetaData self, std::string const & p_name) -> Part
        """
        return _PerceptMesh.FEMMetaData_declare_part(self, *args)

    def declare_part_subset(self, *args):
        """declare_part_subset(FEMMetaData self, Part superset, Part subset)"""
        return _PerceptMesh.FEMMetaData_declare_part_subset(self, *args)

    def declare_part_relation(self, *args):
        """declare_part_relation(FEMMetaData self, Part root_part, stk::mesh::relation_stencil_ptr stencil, Part target_part)"""
        return _PerceptMesh.FEMMetaData_declare_part_relation(self, *args)

    def set_entity_rank_names(self, *args):
        """set_entity_rank_names(FEMMetaData self, vectors in_entity_rank_names)"""
        return _PerceptMesh.FEMMetaData_set_entity_rank_names(self, *args)

    def entity_rank(self, *args):
        """entity_rank(FEMMetaData self, std::string const & name) -> stk::mesh::EntityRank"""
        return _PerceptMesh.FEMMetaData_entity_rank(self, *args)

    def entity_rank_names(self):
        """entity_rank_names(FEMMetaData self) -> vectors"""
        return _PerceptMesh.FEMMetaData_entity_rank_names(self)

    def entity_rank_count(self):
        """entity_rank_count(FEMMetaData self) -> std::vector< std::string,std::allocator< std::string > >::size_type"""
        return _PerceptMesh.FEMMetaData_entity_rank_count(self)

    def entity_rank_name(self, *args):
        """entity_rank_name(FEMMetaData self, stk::mesh::EntityRank in_entity_rank) -> std::string const &"""
        return _PerceptMesh.FEMMetaData_entity_rank_name(self, *args)

    def is_valid_entity_rank(self, *args):
        """is_valid_entity_rank(FEMMetaData self, stk::mesh::EntityRank rank) -> bool"""
        return _PerceptMesh.FEMMetaData_is_valid_entity_rank(self, *args)

    def get_fields(self):
        """get_fields(FEMMetaData self) -> vectorfieldbase"""
        return _PerceptMesh.FEMMetaData_get_fields(self)

    def get_field_relations(self):
        """get_field_relations(FEMMetaData self) -> std::vector< FieldRelation,std::allocator< FieldRelation > > const &"""
        return _PerceptMesh.FEMMetaData_get_field_relations(self)

    def commit(self):
        """commit(FEMMetaData self)"""
        return _PerceptMesh.FEMMetaData_commit(self)

    def is_commit(self):
        """is_commit(FEMMetaData self) -> bool"""
        return _PerceptMesh.FEMMetaData_is_commit(self)

    def declare_field_base(self, *args):
        """
        declare_field_base(FEMMetaData self, std::string const & arg_name, DataTraits const & arg_traits, unsigned int arg_rank, 
            shards::ArrayDimTag const *const * arg_dim_tags, unsigned int arg_num_states) -> FieldBase
        """
        return _PerceptMesh.FEMMetaData_declare_field_base(self, *args)

    def declare_field_restriction(self, *args):
        """
        declare_field_restriction(FEMMetaData self, FieldBase arg_field, stk::mesh::EntityRank arg_entity_rank, Part arg_part, unsigned int const * arg_stride, 
            void const * arg_init_value=None)
        declare_field_restriction(FEMMetaData self, FieldBase arg_field, stk::mesh::EntityRank arg_entity_rank, Part arg_part, unsigned int const * arg_stride)
        """
        return _PerceptMesh.FEMMetaData_declare_field_restriction(self, *args)

    def get_field(self, *args):
        """get_field(FEMMetaData self, std::string const & name) -> FieldBase"""
        return _PerceptMesh.FEMMetaData_get_field(self, *args)

FEMMetaData_swigregister = _PerceptMesh.FEMMetaData_swigregister
FEMMetaData_swigregister(FEMMetaData)

def FEMMetaData_get_meta_data(*args):
  """FEMMetaData_get_meta_data(FEMMetaData fem_meta) -> stk::mesh::MetaData &"""
  return _PerceptMesh.FEMMetaData_get_meta_data(*args)

def FEMMetaData_get(*args):
  """
    get(stk::mesh::MetaData const & meta) -> FEMMetaData
    get(Part part) -> FEMMetaData
    get(FieldBase field) -> FEMMetaData
    get(stk::mesh::PropertyBase const & property) -> FEMMetaData
    get(BulkData bulk_data) -> FEMMetaData
    get(Bucket bucket) -> FEMMetaData
    get(stk::mesh::Entity const & entity) -> FEMMetaData
    FEMMetaData_get(stk::mesh::Ghosting const & ghost) -> FEMMetaData
    """
  return _PerceptMesh.FEMMetaData_get(*args)


def is_cell_topology_root_part(*args):
  """is_cell_topology_root_part(Part part) -> bool"""
  return _PerceptMesh.is_cell_topology_root_part(*args)

def set_cell_topology(*args):
  """set_cell_topology(Part part, stk::mesh::fem::CellTopology const cell_topology)"""
  return _PerceptMesh.set_cell_topology(*args)

def get_cell_topology(*args):
  """
    get_cell_topology(Bucket bucket) -> stk::mesh::fem::CellTopology
    get_cell_topology(stk::mesh::Entity const & entity) -> stk::mesh::fem::CellTopology
    """
  return _PerceptMesh.get_cell_topology(*args)

def entity_rank_names(*args):
  """entity_rank_names(size_t spatial_dimension) -> vectors"""
  return _PerceptMesh.entity_rank_names(*args)
class MDArrayString(_object):
    """Proxy of C++ stk::percept::MDArrayString class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, MDArrayString, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, MDArrayString, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::MDArrayString self) -> MDArrayString
        __init__(stk::percept::MDArrayString self, int dim) -> MDArrayString
        __init__(stk::percept::MDArrayString self, int dim0, int dim1) -> MDArrayString
        __init__(stk::percept::MDArrayString self, MDArrayString mda) -> MDArrayString

        stk::percept::MDArrayString::MDArrayString(const MDArrayString &mda)

        """
        this = _PerceptMesh.new_MDArrayString(*args)
        try: self.this.append(this)
        except: self.this = this
    def resize(self, *args):
        """
        resize(MDArrayString self, int dim)
        resize(MDArrayString self, int dim0, int dim1)

        void
        stk::percept::MDArrayString::resize(int dim0, int dim1) 
        """
        return _PerceptMesh.MDArrayString_resize(self, *args)

    def rank(self):
        """
        rank(MDArrayString self) -> int const

        const int
        stk::percept::MDArrayString::rank() const 
        """
        return _PerceptMesh.MDArrayString_rank(self)

    def setValues(self, *args):
        """
        setValues(MDArrayString self, std::string * data)

        void
        stk::percept::MDArrayString::setValues(std::string *data) 
        """
        return _PerceptMesh.MDArrayString_setValues(self, *args)

    def dimension(self, *args):
        """
        dimension(MDArrayString self, int i1) -> int

        int
        stk::percept::MDArrayString::dimension(int i1) const 
        """
        return _PerceptMesh.MDArrayString_dimension(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_MDArrayString
MDArrayString_swigregister = _PerceptMesh.MDArrayString_swigregister
MDArrayString_swigregister(MDArrayString)

class MDArray(_object):
    """Proxy of C++ Intrepid::FieldContainer<(double)> class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, MDArray, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, MDArray, name)
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_MDArray
    def __init__(self, *args): 
        """
        __init__(Intrepid::FieldContainer<(double)> self) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, MDArray right) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, int const dim0) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, int const dim0, int const dim1) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, int const dim0, int const dim1, int const dim2) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, int const dim0, int const dim1, int const dim2, int const dim3) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, int const dim0, int const dim1, int const dim2, int const dim3, int const dim4) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, Teuchos::Array< int > const & dimensions) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, Teuchos::Array< int > const & dimensions, Teuchos::ArrayView< double > const & data) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, Teuchos::Array< int > const & dimensions, Teuchos::ArrayRCP< double > const & data) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, Teuchos::Array< int > const & dimensions, double * data, bool const deep_copy=False, 
            bool const owns_mem=False) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, Teuchos::Array< int > const & dimensions, double * data, bool const deep_copy=False) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, Teuchos::Array< int > const & dimensions, double * data) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, shards::Array< double,shards::NaturalOrder > const & data, bool const deep_copy=False, 
            bool const owns_mem=False) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, shards::Array< double,shards::NaturalOrder > const & data, bool const deep_copy=False) -> MDArray
        __init__(Intrepid::FieldContainer<(double)> self, shards::Array< double,shards::NaturalOrder > const & data) -> MDArray
        """
        this = _PerceptMesh.new_MDArray(*args)
        try: self.this.append(this)
        except: self.this = this
    def rank(self):
        """rank(MDArray self) -> int"""
        return _PerceptMesh.MDArray_rank(self)

    def size(self):
        """size(MDArray self) -> int"""
        return _PerceptMesh.MDArray_size(self)

    def dimension(self, *args):
        """dimension(MDArray self, int const whichDim) -> int"""
        return _PerceptMesh.MDArray_dimension(self, *args)

    def getEnumeration(self, *args):
        """
        getEnumeration(MDArray self, int const i0) -> int
        getEnumeration(MDArray self, int const i0, int const i1) -> int
        getEnumeration(MDArray self, int const i0, int const i1, int const i2) -> int
        getEnumeration(MDArray self, int const i0, int const i1, int const i2, int const i3) -> int
        getEnumeration(MDArray self, int const i0, int const i1, int const i2, int const i3, int const i4) -> int
        getEnumeration(MDArray self, Teuchos::Array< int > const & multiIndex) -> int
        """
        return _PerceptMesh.MDArray_getEnumeration(self, *args)

    def getMultiIndex(self, *args):
        """
        getMultiIndex(MDArray self, int & i0, int const valueEnum)
        getMultiIndex(MDArray self, int & i0, int & i1, int const valueEnum)
        getMultiIndex(MDArray self, int & i0, int & i1, int & i2, int const valueEnum)
        getMultiIndex(MDArray self, int & i0, int & i1, int & i2, int & i3, int const valueEnum)
        getMultiIndex(MDArray self, int & i0, int & i1, int & i2, int & i3, int & i4, int const valueEnum)
        """
        return _PerceptMesh.MDArray_getMultiIndex(self, *args)

    def clear(self):
        """clear(MDArray self)"""
        return _PerceptMesh.MDArray_clear(self)

    def resize(self, *args):
        """
        resize(MDArray self, int const dim0)
        resize(MDArray self, int const dim0, int const dim1)
        resize(MDArray self, int const dim0, int const dim1, int const dim2)
        resize(MDArray self, int const dim0, int const dim1, int const dim2, int const dim3)
        resize(MDArray self, int const dim0, int const dim1, int const dim2, int const dim3, int const dim4)
        resize(MDArray self, Teuchos::Array< int > const & newDimensions)
        resize(MDArray self, MDArray anotherContainer)
        resize(MDArray self, int const numPoints, int const numFields, EFunctionSpace const spaceType, EOperator const operatorType, 
            int const spaceDim)
        """
        return _PerceptMesh.MDArray_resize(self, *args)

    def initialize(self, value=0):
        """
        initialize(MDArray self, double const value=0)
        initialize(MDArray self)
        """
        return _PerceptMesh.MDArray_initialize(self, value)

    def getValue(self, *args):
        """getValue(MDArray self, Teuchos::Array< int > const & multiIndex) -> double"""
        return _PerceptMesh.MDArray_getValue(self, *args)

    def setValue(self, *args):
        """
        setValue(MDArray self, double const dataValue, Teuchos::Array< int > const & multiIndex)
        setValue(MDArray self, double const dataValue, int const order)
        """
        return _PerceptMesh.MDArray_setValue(self, *args)

    def setValues(self, *args):
        """
        setValues(MDArray self, Teuchos::ArrayView< double > const & dataArray)
        setValues(MDArray self, double const * dataPtr, int const numData)
        """
        return _PerceptMesh.MDArray_setValues(self, *args)

    def getData(self):
        """getData(MDArray self) -> Teuchos::ArrayRCP< double >"""
        return _PerceptMesh.MDArray_getData(self)

MDArray_swigregister = _PerceptMesh.MDArray_swigregister
MDArray_swigregister(MDArray)

class Name(_object):
    """
    Useful in other places where two strings are passed into a function or
    constructor.

    this is to avoid a common bug where the name of the String Function is
    given instead of function_string, ie., first two args are accidentally
    reversed - this is essentially a model of a "named argument", as
    opposed to a positional one; of course, it is still only a hint
    (though a strong one) to the user

    C++ includes: Name.hpp 
    """
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Name, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Name, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::Name self, std::string const name) -> Name

        stk::percept::Name::Name(const std::string name) 
        """
        this = _PerceptMesh.new_Name(*args)
        try: self.this.append(this)
        except: self.this = this
    def getName(self):
        """
        getName(Name self) -> std::string const &

        const
        std::string& stk::percept::Name::getName() const 
        """
        return _PerceptMesh.Name_getName(self)

    __swig_destroy__ = _PerceptMesh.delete_Name
Name_swigregister = _PerceptMesh.Name_swigregister
Name_swigregister(Name)

PERCEPT_USE_FAMILY_TREE = _PerceptMesh.PERCEPT_USE_FAMILY_TREE
PERCEPT_USE_PSEUDO_ELEMENTS = _PerceptMesh.PERCEPT_USE_PSEUDO_ELEMENTS
FAMILY_TREE_LEVEL_0 = _PerceptMesh.FAMILY_TREE_LEVEL_0
FAMILY_TREE_LEVEL_1 = _PerceptMesh.FAMILY_TREE_LEVEL_1
FAMILY_TREE_PARENT = _PerceptMesh.FAMILY_TREE_PARENT
FAMILY_TREE_CHILD_START_INDEX = _PerceptMesh.FAMILY_TREE_CHILD_START_INDEX
class GMeshSpec(Name):
    """Proxy of C++ stk::percept::GMeshSpec class"""
    __swig_setmethods__ = {}
    for _s in [Name]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, GMeshSpec, name, value)
    __swig_getmethods__ = {}
    for _s in [Name]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, GMeshSpec, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::GMeshSpec self, std::string const & name) -> GMeshSpec

        stk::percept::GMeshSpec::GMeshSpec(const std::string &name) 
        """
        this = _PerceptMesh.new_GMeshSpec(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_GMeshSpec
GMeshSpec_swigregister = _PerceptMesh.GMeshSpec_swigregister
GMeshSpec_swigregister(GMeshSpec)
EntityRankEnd = cvar.EntityRankEnd

class FieldCreateOrder(_object):
    """Proxy of C++ stk::percept::FieldCreateOrder class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, FieldCreateOrder, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, FieldCreateOrder, name)
    __repr__ = _swig_repr
    __swig_getmethods__["m_name"] = _PerceptMesh.FieldCreateOrder_m_name_get
    if _newclass:m_name = _swig_property(_PerceptMesh.FieldCreateOrder_m_name_get)
    __swig_getmethods__["m_entity_rank"] = _PerceptMesh.FieldCreateOrder_m_entity_rank_get
    if _newclass:m_entity_rank = _swig_property(_PerceptMesh.FieldCreateOrder_m_entity_rank_get)
    __swig_getmethods__["m_dimensions"] = _PerceptMesh.FieldCreateOrder_m_dimensions_get
    if _newclass:m_dimensions = _swig_property(_PerceptMesh.FieldCreateOrder_m_dimensions_get)
    __swig_setmethods__["m_part"] = _PerceptMesh.FieldCreateOrder_m_part_set
    __swig_getmethods__["m_part"] = _PerceptMesh.FieldCreateOrder_m_part_get
    if _newclass:m_part = _swig_property(_PerceptMesh.FieldCreateOrder_m_part_get, _PerceptMesh.FieldCreateOrder_m_part_set)
    def __init__(self, *args): 
        """
        __init__(stk::percept::FieldCreateOrder self) -> FieldCreateOrder
        __init__(stk::percept::FieldCreateOrder self, std::string const name, unsigned int const entity_rank, vectori dimensions, Part part) -> FieldCreateOrder

        stk::percept::FieldCreateOrder::FieldCreateOrder(const std::string
        name, const unsigned entity_rank, const std::vector< int > dimensions,
        const mesh::Part *part) 
        """
        this = _PerceptMesh.new_FieldCreateOrder(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_FieldCreateOrder
FieldCreateOrder_swigregister = _PerceptMesh.FieldCreateOrder_swigregister
FieldCreateOrder_swigregister(FieldCreateOrder)

class PerceptMesh(_object):
    """Proxy of C++ stk::percept::PerceptMesh class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, PerceptMesh, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, PerceptMesh, name)
    __repr__ = _swig_repr
    __swig_setmethods__["s_omit_part"] = _PerceptMesh.PerceptMesh_s_omit_part_set
    __swig_getmethods__["s_omit_part"] = _PerceptMesh.PerceptMesh_s_omit_part_get
    if _newclass:s_omit_part = _swig_property(_PerceptMesh.PerceptMesh_s_omit_part_get, _PerceptMesh.PerceptMesh_s_omit_part_set)
    def __init__(self, *args): 
        """
        __init__(stk::percept::PerceptMesh self, size_t spatialDimension=3u, stk::ParallelMachine comm=MPI_COMM_WORLD) -> PerceptMesh
        __init__(stk::percept::PerceptMesh self, size_t spatialDimension=3u) -> PerceptMesh
        __init__(stk::percept::PerceptMesh self) -> PerceptMesh
        __init__(stk::percept::PerceptMesh self, FEMMetaData metaData, BulkData bulkData, bool isCommitted=True) -> PerceptMesh
        __init__(stk::percept::PerceptMesh self, FEMMetaData metaData, BulkData bulkData) -> PerceptMesh

        stk::percept::PerceptMesh::PerceptMesh(const
        stk::mesh::fem::FEMMetaData *metaData, stk::mesh::BulkData *bulkData,
        bool isCommitted=true)

        Create a Mesh object that doesn't own its constituent FEMMetaData and
        BulkData, pointers to which are adopted by this constructor. 
        """
        this = _PerceptMesh.new_PerceptMesh(*args)
        try: self.this.append(this)
        except: self.this = this
    def open_read_only(self, *args):
        """
        open_read_only(PerceptMesh self, std::string const & in_filename)

        void stk::percept::PerceptMesh::open_read_only(const std::string
        &in_filename)

        reads and commits mesh, editing disabled 
        """
        return _PerceptMesh.PerceptMesh_open_read_only(self, *args)

    def open(self, *args):
        """
        open(PerceptMesh self, std::string const & in_filename)

        void
        stk::percept::PerceptMesh::open(const std::string &in_filename)

        reads but doesn't commit mesh, enabling edit 
        """
        return _PerceptMesh.PerceptMesh_open(self, *args)

    def new_mesh_read_only(self, *args):
        """
        new_mesh_read_only(PerceptMesh self, GMeshSpec gmesh_spec)

        void stk::percept::PerceptMesh::new_mesh_read_only(const GMeshSpec
        gmesh_spec)

        creates a new mesh using the GeneratedMesh fixture with spec

        Parameters:
        -----------

        gmesh_spec:  Read:  Only mode, no edits allowed 
        """
        return _PerceptMesh.PerceptMesh_new_mesh_read_only(self, *args)

    def new_mesh(self, *args):
        """
        new_mesh(PerceptMesh self, GMeshSpec gmesh_spec)

        void
        stk::percept::PerceptMesh::new_mesh(const GMeshSpec gmesh_spec)

        creates a new mesh using the GeneratedMesh fixture with spec

        Parameters:
        -----------

        gmesh_spec:  
        """
        return _PerceptMesh.PerceptMesh_new_mesh(self, *args)

    def add_field(self, *args):
        """
        add_field(PerceptMesh self, std::string const & name, unsigned int const entity_rank, int vectorDimension=0, 
            std::string const part_name="universal_part") -> FieldBase
        add_field(PerceptMesh self, std::string const & name, unsigned int const entity_rank, int vectorDimension=0) -> FieldBase
        add_field(PerceptMesh self, std::string const & name, unsigned int const entity_rank) -> FieldBase

        stk::mesh::FieldBase * stk::percept::PerceptMesh::add_field(const
        std::string &name, const unsigned entity_rank, int vectorDimension=0,
        const std::string part_name="universal_part")

        add a field to the mesh 
        """
        return _PerceptMesh.PerceptMesh_add_field(self, *args)

    def get_field(self, *args):
        """
        get_field(PerceptMesh self, std::string const & name) -> FieldBase

        stk::mesh::FieldBase * stk::percept::PerceptMesh::get_field(const
        std::string &name) 
        """
        return _PerceptMesh.PerceptMesh_get_field(self, *args)

    def commit(self):
        """
        commit(PerceptMesh self)

        void
        stk::percept::PerceptMesh::commit()

        commits mesh - any operations done on a non-committed mesh, except to
        add fields will throw an exception 
        """
        return _PerceptMesh.PerceptMesh_commit(self)

    def reopen(self, temp_file_name="percept_tmp.e"):
        """
        reopen(PerceptMesh self, std::string const temp_file_name="percept_tmp.e")
        reopen(PerceptMesh self)

        void
        stk::percept::PerceptMesh::reopen(const std::string
        temp_file_name="percept_tmp.e")

        reopens the mesh for editing - warning, this operation writes the mesh
        to a temp file then re-reads it and thus recreates the internal
        FEMMetaData and BulkData 
        """
        return _PerceptMesh.PerceptMesh_reopen(self, temp_file_name)

    def save_as(self, *args):
        """
        save_as(PerceptMesh self, std::string const & out_filename)

        void
        stk::percept::PerceptMesh::save_as(const std::string &out_filename)

        commits mesh if not committed and saves it in new file 
        """
        return _PerceptMesh.PerceptMesh_save_as(self, *args)

    def close(self):
        """
        close(PerceptMesh self)

        void
        stk::percept::PerceptMesh::close()

        closes this mesh, deleting its data

        closes this mesh to further changes 
        """
        return _PerceptMesh.PerceptMesh_close(self)

    def print_info(self, *args):
        """
        print_info(PerceptMesh self, std::ostream & stream, std::string header="", int print_level=0, bool do_endl=True)
        print_info(PerceptMesh self, std::ostream & stream, std::string header="", int print_level=0)
        print_info(PerceptMesh self, std::ostream & stream, std::string header="")
        print_info(PerceptMesh self, std::ostream & stream)
        print_info(PerceptMesh self, std::string header="", int print_level=0, bool do_endl=True)
        print_info(PerceptMesh self, std::string header="", int print_level=0)
        print_info(PerceptMesh self, std::string header="")
        print_info(PerceptMesh self)

        void
        stk::percept::PerceptMesh::print_info(std::string header="", int
        print_level=0, bool do_endl=true)

        print number of parts and fields, and info on each 
        """
        return _PerceptMesh.PerceptMesh_print_info(self, *args)

    def print_fields(self, header=""):
        """
        print_fields(PerceptMesh self, std::string header="")
        print_fields(PerceptMesh self)

        void
        stk::percept::PerceptMesh::print_fields(std::string header="")

        print the fields defined on the mesh 
        """
        return _PerceptMesh.PerceptMesh_print_fields(self, header)

    def get_spatial_dim(self):
        """
        get_spatial_dim(PerceptMesh self) -> int

        int stk::percept::PerceptMesh::get_spatial_dim() 
        """
        return _PerceptMesh.PerceptMesh_get_spatial_dim(self)

    def get_number_elements(self):
        """
        get_number_elements(PerceptMesh self) -> int

        int stk::percept::PerceptMesh::get_number_elements() 
        """
        return _PerceptMesh.PerceptMesh_get_number_elements(self)

    def get_number_nodes(self):
        """
        get_number_nodes(PerceptMesh self) -> int

        int stk::percept::PerceptMesh::get_number_nodes() 
        """
        return _PerceptMesh.PerceptMesh_get_number_nodes(self)

    def get_number_edges(self):
        """
        get_number_edges(PerceptMesh self) -> int

        int stk::percept::PerceptMesh::get_number_edges() 
        """
        return _PerceptMesh.PerceptMesh_get_number_edges(self)

    def get_number_elements_locally_owned(self):
        """
        get_number_elements_locally_owned(PerceptMesh self) -> int

        int
        stk::percept::PerceptMesh::get_number_elements_locally_owned() 
        """
        return _PerceptMesh.PerceptMesh_get_number_elements_locally_owned(self)

    def get_rank(self):
        """
        get_rank(PerceptMesh self) -> unsigned int

        unsigned
        stk::percept::PerceptMesh::get_rank()

        parallel rank 
        """
        return _PerceptMesh.PerceptMesh_get_rank(self)

    def get_parallel_rank(self):
        """
        get_parallel_rank(PerceptMesh self) -> unsigned int

        unsigned stk::percept::PerceptMesh::get_parallel_rank() 
        """
        return _PerceptMesh.PerceptMesh_get_parallel_rank(self)

    def get_parallel_size(self):
        """
        get_parallel_size(PerceptMesh self) -> unsigned int

        unsigned stk::percept::PerceptMesh::get_parallel_size() 
        """
        return _PerceptMesh.PerceptMesh_get_parallel_size(self)

    def print_entity(self, *args):
        """
        print_entity(PerceptMesh self, stk::mesh::Entity const & entity, FieldBase field=None)
        print_entity(PerceptMesh self, stk::mesh::Entity const & entity)

        void
        stk::percept::PerceptMesh::print_entity(std::ostream &out, const
        stk::mesh::Entity &entity, stk::mesh::FieldBase *field=0) 
        """
        return _PerceptMesh.PerceptMesh_print_entity(self, *args)

    def print_entity_compact(self, *args):
        """
        print_entity_compact(PerceptMesh self, stk::mesh::Entity const & entity, FieldBase field=None) -> std::string
        print_entity_compact(PerceptMesh self, stk::mesh::Entity const & entity) -> std::string

        std::string stk::percept::PerceptMesh::print_entity_compact(const
        stk::mesh::Entity &entity, stk::mesh::FieldBase *field=0)

        shorter output for print_entity 
        """
        return _PerceptMesh.PerceptMesh_print_entity_compact(self, *args)

    def dump_elements(self, partName=""):
        """
        dump_elements(PerceptMesh self, std::string const & partName="")
        dump_elements(PerceptMesh self)

        void
        stk::percept::PerceptMesh::dump_elements(const std::string
        &partName="")

        print elements on the given part 
        """
        return _PerceptMesh.PerceptMesh_dump_elements(self, partName)

    def dump_elements_compact(self, partName=""):
        """
        dump_elements_compact(PerceptMesh self, std::string const & partName="")
        dump_elements_compact(PerceptMesh self)

        void
        stk::percept::PerceptMesh::dump_elements_compact(const std::string
        &partName="")

        compact print of elements on the given part 
        """
        return _PerceptMesh.PerceptMesh_dump_elements_compact(self, partName)

    def get_bulk_data(self):
        """
        get_bulk_data(PerceptMesh self) -> BulkData

        stk::mesh::BulkData * stk::percept::PerceptMesh::get_bulk_data()

        get the low-level bulk data pointer from stk_mesh 
        """
        return _PerceptMesh.PerceptMesh_get_bulk_data(self)

    def get_fem_meta_data(self):
        """
        get_fem_meta_data(PerceptMesh self) -> FEMMetaData

        stk::mesh::fem::FEMMetaData *
        stk::percept::PerceptMesh::get_fem_meta_data()

        get the low-level meta data pointer from stk_mesh 
        """
        return _PerceptMesh.PerceptMesh_get_fem_meta_data(self)

    def get_part(self, *args):
        """
        get_part(PerceptMesh self, std::string const & part_name) -> Part

        mesh::Part* stk::percept::PerceptMesh::get_part(const std::string
        &part_name)

        get a pointer to a stk_mesh Part with the given name 
        """
        return _PerceptMesh.PerceptMesh_get_part(self, *args)

    def get_field_data(self, *args):
        """
        get_field_data(PerceptMesh self, FieldBase field, stk::mesh::Entity const * entity, unsigned int ordinal=0) -> double
        get_field_data(PerceptMesh self, FieldBase field, stk::mesh::Entity const * entity) -> double

        double stk::percept::PerceptMesh::get_field_data(const
        stk::mesh::FieldBase *field, const mesh::Entity *entity, unsigned
        ordinal=0)

        get the value of a field on the given entity; if a vector field, pass
        in the index of the vector required (ordinal) 
        """
        return _PerceptMesh.PerceptMesh_get_field_data(self, *args)

    def set_field_data(self, *args):
        """
        set_field_data(PerceptMesh self, double value, FieldBase field, stk::mesh::Entity const * entity, unsigned int ordinal=0)
        set_field_data(PerceptMesh self, double value, FieldBase field, stk::mesh::Entity const * entity)

        void stk::percept::PerceptMesh::set_field_data(double value, const
        stk::mesh::FieldBase *field, const mesh::Entity *entity, unsigned
        ordinal=0)

        set the value of a field on the given entity; if a vector field, pass
        in the index of the vector required (ordinal) 
        """
        return _PerceptMesh.PerceptMesh_set_field_data(self, *args)

    def get_node_field_data(self, *args):
        """
        get_node_field_data(PerceptMesh self, FieldBase field, stk::mesh::EntityId const node_id, unsigned int ordinal=0) -> double
        get_node_field_data(PerceptMesh self, FieldBase field, stk::mesh::EntityId const node_id) -> double

        double
        stk::percept::PerceptMesh::get_node_field_data(stk::mesh::FieldBase
        *field, const mesh::EntityId node_id, unsigned ordinal=0)

        get the value of a field on the given node; if a vector field, pass in
        the index of the vector required (ordinal) 
        """
        return _PerceptMesh.PerceptMesh_get_node_field_data(self, *args)

    def set_node_field_data(self, *args):
        """
        set_node_field_data(PerceptMesh self, double value, FieldBase field, stk::mesh::EntityId const node_id, unsigned int ordinal=0)
        set_node_field_data(PerceptMesh self, double value, FieldBase field, stk::mesh::EntityId const node_id)

        void stk::percept::PerceptMesh::set_node_field_data(double value,
        stk::mesh::FieldBase *field, const mesh::EntityId node_id, unsigned
        ordinal=0)

        set the value of a field on the given node; if a vector field, pass in
        the index of the vector required (ordinal) 
        """
        return _PerceptMesh.PerceptMesh_set_node_field_data(self, *args)

    def get_entity(self, *args):
        """
        get_entity(PerceptMesh self, stk::mesh::EntityRank rank, stk::mesh::EntityId const id) -> stk::mesh::Entity *

        stk::mesh::Entity*
        stk::percept::PerceptMesh::get_entity(mesh::EntityRank rank, const
        mesh::EntityId id)

        get a pointer to an entity with given id 
        """
        return _PerceptMesh.PerceptMesh_get_entity(self, *args)

    def get_node(self, *args):
        """
        get_node(PerceptMesh self, stk::mesh::EntityId const node_id) -> stk::mesh::Entity
        get_node(PerceptMesh self, double x, double y, double z=0, double t=0) -> stk::mesh::Entity
        get_node(PerceptMesh self, double x, double y, double z=0) -> stk::mesh::Entity
        get_node(PerceptMesh self, double x, double y) -> stk::mesh::Entity *

        stk::mesh::Entity * stk::percept::PerceptMesh::get_node(double x,
        double y, double z=0, double t=0)

        find and return pointer to node closest to given point - in parallel,
        check return for null (if null, closest node is on another proc) 
        """
        return _PerceptMesh.PerceptMesh_get_node(self, *args)

    def get_element(self, *args):
        """
        get_element(PerceptMesh self, stk::mesh::EntityId const element_id) -> stk::mesh::Entity
        get_element(PerceptMesh self, double x, double y, double z=0, double t=0) -> stk::mesh::Entity
        get_element(PerceptMesh self, double x, double y, double z=0) -> stk::mesh::Entity
        get_element(PerceptMesh self, double x, double y) -> stk::mesh::Entity *

        stk::mesh::Entity * stk::percept::PerceptMesh::get_element(double x,
        double y, double z=0, double t=0)

        find and return pointer to element that contains given point - in
        parallel, check return for null (if null, element containing point is
        on another proc)

        find element that contains or is closest to given point 
        """
        return _PerceptMesh.PerceptMesh_get_element(self, *args)

    def mesh_difference(*args):
        """
        mesh_difference(PerceptMesh mesh1, PerceptMesh mesh2, std::string msg, bool _print=True, bool print_all_field_diffs=False) -> bool
        mesh_difference(PerceptMesh mesh1, PerceptMesh mesh2, std::string msg, bool _print=True) -> bool
        mesh_difference(PerceptMesh mesh1, PerceptMesh mesh2, std::string msg) -> bool
        """
        return _PerceptMesh.PerceptMesh_mesh_difference(*args)

    if _newclass:mesh_difference = staticmethod(mesh_difference)
    __swig_getmethods__["mesh_difference"] = lambda x: mesh_difference
    def get_coordinates_field(self):
        """
        get_coordinates_field(PerceptMesh self) -> stk::percept::VectorFieldType *

        VectorFieldType*
        stk::percept::PerceptMesh::get_coordinates_field()

        return a pointer to the field containing node coordinates 
        """
        return _PerceptMesh.PerceptMesh_get_coordinates_field(self)

    def node_rank(self):
        """
        node_rank(PerceptMesh self) -> stk::mesh::EntityRank

        stk::mesh::EntityRank stk::percept::PerceptMesh::node_rank() const

        return the rank of a node 
        """
        return _PerceptMesh.PerceptMesh_node_rank(self)

    def edge_rank(self):
        """
        edge_rank(PerceptMesh self) -> stk::mesh::EntityRank

        stk::mesh::EntityRank stk::percept::PerceptMesh::edge_rank() const

        Returns the edge rank which changes depending on spatial dimension. 
        """
        return _PerceptMesh.PerceptMesh_edge_rank(self)

    def face_rank(self):
        """
        face_rank(PerceptMesh self) -> stk::mesh::EntityRank

        stk::mesh::EntityRank stk::percept::PerceptMesh::face_rank() const

        Returns the face rank which changes depending on spatial dimension. 
        """
        return _PerceptMesh.PerceptMesh_face_rank(self)

    def side_rank(self):
        """
        side_rank(PerceptMesh self) -> stk::mesh::EntityRank

        stk::mesh::EntityRank stk::percept::PerceptMesh::side_rank() const

        Returns the side rank which changes depending on spatial dimension. 
        """
        return _PerceptMesh.PerceptMesh_side_rank(self)

    def element_rank(self):
        """
        element_rank(PerceptMesh self) -> stk::mesh::EntityRank

        stk::mesh::EntityRank stk::percept::PerceptMesh::element_rank() const

        Returns the element rank which is always equal to spatial dimension.

        """
        return _PerceptMesh.PerceptMesh_element_rank(self)

    def read_database_at_step(self, *args):
        """
        read_database_at_step(PerceptMesh self, int step)

        void
        stk::percept::PerceptMesh::read_database_at_step(int step)

        set the current data in fields to the given Exodus step by reading
        from the database 
        """
        return _PerceptMesh.PerceptMesh_read_database_at_step(self, *args)

    def read_database_at_time(self, *args):
        """
        read_database_at_time(PerceptMesh self, double time)

        void
        stk::percept::PerceptMesh::read_database_at_time(double time)

        set the current data in fields to the given Exodus time by reading
        from the database (finds the closest step to the given time (no
        interpolation yet)) 
        """
        return _PerceptMesh.PerceptMesh_read_database_at_time(self, *args)

    def get_current_database_step(self):
        """
        get_current_database_step(PerceptMesh self) -> int

        int
        stk::percept::PerceptMesh::get_current_database_step()

        return the current state of the Exodus database, 0 if not loaded yet
        (steps are 1-based in Exodus) 
        """
        return _PerceptMesh.PerceptMesh_get_current_database_step(self)

    def get_current_database_time(self):
        """
        get_current_database_time(PerceptMesh self) -> double

        double
        stk::percept::PerceptMesh::get_current_database_time()

        return the current state of the Exodus database (time associated with
        current step) 
        """
        return _PerceptMesh.PerceptMesh_get_current_database_time(self)

    def get_database_step_at_time(self, *args):
        """
        get_database_step_at_time(PerceptMesh self, double time) -> int

        int
        stk::percept::PerceptMesh::get_database_step_at_time(double time)

        return the step number closest to specified time, thus
        read_database_at_time(time) is equivalent to
        read_database_at_step(get_database_step_at_time(time)) 
        """
        return _PerceptMesh.PerceptMesh_get_database_step_at_time(self, *args)

    def get_database_time_at_step(self, *args):
        """
        get_database_time_at_step(PerceptMesh self, int step) -> double

        double
        stk::percept::PerceptMesh::get_database_time_at_step(int step)

        return the state time associated with given step 
        """
        return _PerceptMesh.PerceptMesh_get_database_time_at_step(self, *args)

    def get_database_time_step_count(self):
        """
        get_database_time_step_count(PerceptMesh self) -> int

        int
        stk::percept::PerceptMesh::get_database_time_step_count()

        return the number of steps in the database 
        """
        return _PerceptMesh.PerceptMesh_get_database_time_step_count(self)

    def transform_mesh(self, *args):
        """
        transform_mesh(PerceptMesh self, MDArray matrix)

        void stk::percept::PerceptMesh::transform_mesh(Math::Matrix &matrix)

        transform mesh by a given 3x3 matrix 
        """
        return _PerceptMesh.PerceptMesh_transform_mesh(self, *args)

    def add_coordinate_state_fields(self):
        """
        add_coordinate_state_fields(PerceptMesh self)

        void
        stk::percept::PerceptMesh::add_coordinate_state_fields()

        add coordinate-like fields needed, for example, to use smoothing of
        geometry-projected refined meshes Must be called before commit() 
        """
        return _PerceptMesh.PerceptMesh_add_coordinate_state_fields(self)

    def add_spacing_fields(self):
        """
        add_spacing_fields(PerceptMesh self)

        void stk::percept::PerceptMesh::add_spacing_fields()

        add spacing fields for having refinement obey the spacing (i.e.
        putting new nodes not at midpoint) 
        """
        return _PerceptMesh.PerceptMesh_add_spacing_fields(self)

    def set_proc_rank_field(self, proc_rank_field=None):
        """
        set_proc_rank_field(PerceptMesh self, FieldBase proc_rank_field=None)
        set_proc_rank_field(PerceptMesh self)

        void
        stk::percept::PerceptMesh::set_proc_rank_field(stk::mesh::FieldBase
        *proc_rank_field=0)

        set proc_rank on each element 
        """
        return _PerceptMesh.PerceptMesh_set_proc_rank_field(self, proc_rank_field)

    def has_coordinate_state_fields(self):
        """
        has_coordinate_state_fields(PerceptMesh self) -> bool

        bool
        stk::percept::PerceptMesh::has_coordinate_state_fields()

        get number of coordinate field states needed 
        """
        return _PerceptMesh.PerceptMesh_has_coordinate_state_fields(self)

    def copy_field_state(self, *args):
        """
        copy_field_state(PerceptMesh self, FieldBase field, unsigned int dest_state, unsigned int src_state)

        void stk::percept::PerceptMesh::copy_field_state(stk::mesh::FieldBase
        *field, unsigned dest_state, unsigned src_state)

        copy field state data from one state (src_state) to another
        (dest_state) 
        """
        return _PerceptMesh.PerceptMesh_copy_field_state(self, *args)

    def copy_field(self, *args):
        """
        copy_field(PerceptMesh self, FieldBase field_dest, FieldBase field_src)

        void
        stk::percept::PerceptMesh::copy_field(stk::mesh::FieldBase
        *field_dest, stk::mesh::FieldBase *field_src)

        copy field data from one field (field_src) to another (field_dest) 
        """
        return _PerceptMesh.PerceptMesh_copy_field(self, *args)

    def nodal_field_state_axpby(self, *args):
        """
        nodal_field_state_axpby(PerceptMesh self, FieldBase field, double alpha, unsigned int x_state, double beta, unsigned int y_state)

        void
        stk::percept::PerceptMesh::nodal_field_state_axpby(stk::mesh::FieldBase
        *field, double alpha, unsigned x_state, double beta, unsigned y_state)

        axpby calculates: y = alpha*x + beta*y 
        """
        return _PerceptMesh.PerceptMesh_nodal_field_state_axpby(self, *args)

    def nodal_field_axpby(self, *args):
        """
        nodal_field_axpby(PerceptMesh self, double alpha, FieldBase field_x, double beta, FieldBase field_y)

        void stk::percept::PerceptMesh::nodal_field_axpby(double alpha,
        stk::mesh::FieldBase *field_x, double beta, stk::mesh::FieldBase
        *field_y)

        axpby calculates: y = alpha*x + beta*y 
        """
        return _PerceptMesh.PerceptMesh_nodal_field_axpby(self, *args)

    def nodal_field_state_axpbypgz(self, *args):
        """
        nodal_field_state_axpbypgz(PerceptMesh self, FieldBase field, double alpha, unsigned int x_state, double beta, unsigned int y_state, 
            double gamma, unsigned int z_state)

        void
        stk::percept::PerceptMesh::nodal_field_state_axpbypgz(stk::mesh::FieldBase
        *field, double alpha, unsigned x_state, double beta, unsigned y_state,
        double gamma, unsigned z_state)

        axpbypgz calculates: z = alpha*x + beta*y + gamma*z 
        """
        return _PerceptMesh.PerceptMesh_nodal_field_state_axpbypgz(self, *args)

    def nodal_field_axpbypgz(self, *args):
        """
        nodal_field_axpbypgz(PerceptMesh self, double alpha, FieldBase field_x, double beta, FieldBase field_y, double gamma, FieldBase field_z)

        void stk::percept::PerceptMesh::nodal_field_axpbypgz(double alpha,
        stk::mesh::FieldBase *field_x, double beta, stk::mesh::FieldBase
        *field_y, double gamma, stk::mesh::FieldBase *field_z)

        axpbypgz calculates: z = alpha*x + beta*y + gamma*z 
        """
        return _PerceptMesh.PerceptMesh_nodal_field_axpbypgz(self, *args)

    def nodal_field_dot(self, *args):
        """
        nodal_field_dot(PerceptMesh self, FieldBase field_x, FieldBase field_y) -> double

        double stk::percept::PerceptMesh::nodal_field_dot(stk::mesh::FieldBase
        *field_x, stk::mesh::FieldBase *field_y)

        dot calculates: x.y 
        """
        return _PerceptMesh.PerceptMesh_nodal_field_dot(self, *args)

    def nodal_field_set_value(self, *args):
        """
        nodal_field_set_value(PerceptMesh self, FieldBase field_x, double value=0.0)
        nodal_field_set_value(PerceptMesh self, FieldBase field_x)

        void
        stk::percept::PerceptMesh::nodal_field_set_value(stk::mesh::FieldBase
        *field_x, double value=0.0)

        set field to constant value 
        """
        return _PerceptMesh.PerceptMesh_nodal_field_set_value(self, *args)

    def remove_geometry_blocks_on_output(self, *args):
        """
        remove_geometry_blocks_on_output(PerceptMesh self, std::string geometry_file_name)

        void
        stk::percept::PerceptMesh::remove_geometry_blocks_on_output(std::string
        geometry_file_name)

        remove blocks in the mesh used solely for geometry association, during
        output of the mesh to Exodus.

        Parameters:
        -----------

        geometry_file_name:  = name of the OpenNURBS file (*.3dm) containing
        the geometry info

        Only available when Percept is configured with
        STK_PERCEPT_HAS_GEOMETRY 
        """
        return _PerceptMesh.PerceptMesh_remove_geometry_blocks_on_output(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_PerceptMesh
PerceptMesh_swigregister = _PerceptMesh.PerceptMesh_swigregister
PerceptMesh_swigregister(PerceptMesh)

def PerceptMesh_mesh_difference(*args):
  """
    mesh_difference(PerceptMesh mesh1, PerceptMesh mesh2, std::string msg, bool _print=True, bool print_all_field_diffs=False) -> bool
    mesh_difference(PerceptMesh mesh1, PerceptMesh mesh2, std::string msg, bool _print=True) -> bool
    PerceptMesh_mesh_difference(PerceptMesh mesh1, PerceptMesh mesh2, std::string msg) -> bool
    """
  return _PerceptMesh.PerceptMesh_mesh_difference(*args)

class MeshTransformer(_object):
    """Proxy of C++ stk::percept::MeshTransformer class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, MeshTransformer, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, MeshTransformer, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::MeshTransformer self) -> MeshTransformer
        __init__(stk::percept::MeshTransformer self, Math::Matrix & m) -> MeshTransformer

        stk::percept::MeshTransformer::MeshTransformer(Math::Matrix &m) 
        """
        this = _PerceptMesh.new_MeshTransformer(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_MeshTransformer
MeshTransformer_swigregister = _PerceptMesh.MeshTransformer_swigregister
MeshTransformer_swigregister(MeshTransformer)

class Dimensions(vectori):
    """Proxy of C++ stk::percept::Dimensions class"""
    __swig_setmethods__ = {}
    for _s in [vectori]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, Dimensions, name, value)
    __swig_getmethods__ = {}
    for _s in [vectori]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, Dimensions, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::Dimensions self) -> Dimensions
        __init__(stk::percept::Dimensions self, int i0) -> Dimensions
        __init__(stk::percept::Dimensions self, int i0, int i1) -> Dimensions
        __init__(stk::percept::Dimensions self, int i0, int i1, int i2) -> Dimensions

        stk::percept::Dimensions::Dimensions(int i0, int i1, int i2) 
        """
        this = _PerceptMesh.new_Dimensions(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_Dimensions
Dimensions_swigregister = _PerceptMesh.Dimensions_swigregister
Dimensions_swigregister(Dimensions)

PERCEPT_QF_USE_COORD_GATHER_FIELD = _PerceptMesh.PERCEPT_QF_USE_COORD_GATHER_FIELD
class QuadFixture_4(_object):
    """This class creates a quad mesh."""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, QuadFixture_4, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, QuadFixture_4, name)
    __repr__ = _swig_repr
    NodesPerElem = _PerceptMesh.QuadFixture_4_NodesPerElem
    def get_entity_rank_names(*args):
        """get_entity_rank_names(unsigned int dim) -> vectors"""
        return _PerceptMesh.QuadFixture_4_get_entity_rank_names(*args)

    if _newclass:get_entity_rank_names = staticmethod(get_entity_rank_names)
    __swig_getmethods__["get_entity_rank_names"] = lambda x: get_entity_rank_names
    __swig_destroy__ = _PerceptMesh.delete_QuadFixture_4
    def __init__(self, *args): 
        """
        __init__(stk::percept::QuadFixture<(double,shards::Quadrilateral<(4)>)> self, stk::ParallelMachine pm, unsigned int nx, unsigned int ny, bool generate_sidesets_in, 
            bool debug_geom_side_sets_as_blocks_in=False) -> QuadFixture_4
        __init__(stk::percept::QuadFixture<(double,shards::Quadrilateral<(4)>)> self, stk::ParallelMachine pm, unsigned int nx, unsigned int ny, bool generate_sidesets_in) -> QuadFixture_4

        stk::percept::QuadFixture< Scalar, Topology
        >::QuadFixture(stk::ParallelMachine pm, unsigned nx, unsigned ny, bool
        generate_sidesets_in, bool debug_geom_side_sets_as_blocks_in=false) 
        """
        this = _PerceptMesh.new_QuadFixture_4(*args)
        try: self.this.append(this)
        except: self.this = this
    def set_bounding_box(self, *args):
        """
        set_bounding_box(QuadFixture_4 self, double xmin, double xmax, double ymin, double ymax)

        void stk::percept::QuadFixture< Scalar, Topology
        >::set_bounding_box(double xmin, double xmax, double ymin, double
        ymax) 
        """
        return _PerceptMesh.QuadFixture_4_set_bounding_box(self, *args)

    def generate_mesh(self, *args):
        """
        generate_mesh(QuadFixture_4 self)
        generate_mesh(QuadFixture_4 self, std::vector< stk::mesh::EntityId,std::allocator< stk::mesh::EntityId > > & element_ids_on_this_processor)

        void
        stk::percept::QuadFixture< Scalar, Topology
        >::generate_mesh(std::vector< stk::mesh::EntityId >
        &element_ids_on_this_processor) 
        """
        return _PerceptMesh.QuadFixture_4_generate_mesh(self, *args)

    __swig_getmethods__["meta_data"] = _PerceptMesh.QuadFixture_4_meta_data_get
    if _newclass:meta_data = _swig_property(_PerceptMesh.QuadFixture_4_meta_data_get)
    __swig_getmethods__["bulk_data"] = _PerceptMesh.QuadFixture_4_bulk_data_get
    if _newclass:bulk_data = _swig_property(_PerceptMesh.QuadFixture_4_bulk_data_get)
    __swig_getmethods__["quad_part"] = _PerceptMesh.QuadFixture_4_quad_part_get
    if _newclass:quad_part = _swig_property(_PerceptMesh.QuadFixture_4_quad_part_get)
    __swig_getmethods__["coord_field"] = _PerceptMesh.QuadFixture_4_coord_field_get
    if _newclass:coord_field = _swig_property(_PerceptMesh.QuadFixture_4_coord_field_get)
    __swig_getmethods__["coord_gather_field"] = _PerceptMesh.QuadFixture_4_coord_gather_field_get
    if _newclass:coord_gather_field = _swig_property(_PerceptMesh.QuadFixture_4_coord_gather_field_get)
    __swig_getmethods__["NX"] = _PerceptMesh.QuadFixture_4_NX_get
    if _newclass:NX = _swig_property(_PerceptMesh.QuadFixture_4_NX_get)
    __swig_getmethods__["NY"] = _PerceptMesh.QuadFixture_4_NY_get
    if _newclass:NY = _swig_property(_PerceptMesh.QuadFixture_4_NY_get)
    __swig_setmethods__["side_parts"] = _PerceptMesh.QuadFixture_4_side_parts_set
    __swig_getmethods__["side_parts"] = _PerceptMesh.QuadFixture_4_side_parts_get
    if _newclass:side_parts = _swig_property(_PerceptMesh.QuadFixture_4_side_parts_get, _PerceptMesh.QuadFixture_4_side_parts_set)
    __swig_setmethods__["generate_sidesets"] = _PerceptMesh.QuadFixture_4_generate_sidesets_set
    __swig_getmethods__["generate_sidesets"] = _PerceptMesh.QuadFixture_4_generate_sidesets_get
    if _newclass:generate_sidesets = _swig_property(_PerceptMesh.QuadFixture_4_generate_sidesets_get, _PerceptMesh.QuadFixture_4_generate_sidesets_set)
    __swig_setmethods__["debug_geom_side_sets_as_blocks"] = _PerceptMesh.QuadFixture_4_debug_geom_side_sets_as_blocks_set
    __swig_getmethods__["debug_geom_side_sets_as_blocks"] = _PerceptMesh.QuadFixture_4_debug_geom_side_sets_as_blocks_get
    if _newclass:debug_geom_side_sets_as_blocks = _swig_property(_PerceptMesh.QuadFixture_4_debug_geom_side_sets_as_blocks_get, _PerceptMesh.QuadFixture_4_debug_geom_side_sets_as_blocks_set)
    def node_id(self, *args):
        """
        node_id(QuadFixture_4 self, unsigned int ix, unsigned int iy) -> stk::mesh::EntityId

        stk::mesh::EntityId stk::percept::QuadFixture< Scalar, Topology
        >::node_id(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_4_node_id(self, *args)

    def elem_id(self, *args):
        """
        elem_id(QuadFixture_4 self, unsigned int ix, unsigned int iy) -> stk::mesh::EntityId

        stk::mesh::EntityId stk::percept::QuadFixture< Scalar, Topology
        >::elem_id(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_4_elem_id(self, *args)

    def node(self, *args):
        """
        node(QuadFixture_4 self, unsigned int ix, unsigned int iy) -> stk::mesh::Entity *

        stk::mesh::Entity* stk::percept::QuadFixture< Scalar, Topology
        >::node(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_4_node(self, *args)

    def node_ix_iy(self, *args):
        """
        node_ix_iy(QuadFixture_4 self, stk::mesh::EntityId entity_id, unsigned int & ix, unsigned int & iy)

        void
        stk::percept::QuadFixture< Scalar, Topology
        >::node_ix_iy(stk::mesh::EntityId entity_id, unsigned &ix, unsigned
        &iy) const 
        """
        return _PerceptMesh.QuadFixture_4_node_ix_iy(self, *args)

    def elem_ix_iy(self, *args):
        """
        elem_ix_iy(QuadFixture_4 self, stk::mesh::EntityId entity_id, unsigned int & ix, unsigned int & iy)

        void
        stk::percept::QuadFixture< Scalar, Topology
        >::elem_ix_iy(stk::mesh::EntityId entity_id, unsigned &ix, unsigned
        &iy) const 
        """
        return _PerceptMesh.QuadFixture_4_elem_ix_iy(self, *args)

    def elem(self, *args):
        """
        elem(QuadFixture_4 self, unsigned int ix, unsigned int iy) -> stk::mesh::Entity *

        stk::mesh::Entity* stk::percept::QuadFixture< Scalar, Topology
        >::elem(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_4_elem(self, *args)

    def generate_sides_meta(self):
        """
        generate_sides_meta(QuadFixture_4 self)

        void stk::percept::QuadFixture< Scalar, Topology
        >::generate_sides_meta() 
        """
        return _PerceptMesh.QuadFixture_4_generate_sides_meta(self)

    def generate_sides_bulk(self, *args):
        """
        generate_sides_bulk(QuadFixture_4 self, std::vector< stk::mesh::EntityId,std::allocator< stk::mesh::EntityId > > & element_ids_on_this_processor)

        void stk::percept::QuadFixture< Scalar, Topology
        >::generate_sides_bulk(std::vector< stk::mesh::EntityId >
        &element_ids_on_this_processor) 
        """
        return _PerceptMesh.QuadFixture_4_generate_sides_bulk(self, *args)

QuadFixture_4_swigregister = _PerceptMesh.QuadFixture_4_swigregister
QuadFixture_4_swigregister(QuadFixture_4)

def QuadFixture_4_get_entity_rank_names(*args):
  """QuadFixture_4_get_entity_rank_names(unsigned int dim) -> vectors"""
  return _PerceptMesh.QuadFixture_4_get_entity_rank_names(*args)

class QuadFixture_3(_object):
    """This class creates a tri mesh."""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, QuadFixture_3, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, QuadFixture_3, name)
    __repr__ = _swig_repr
    NodesPerElem = _PerceptMesh.QuadFixture_3_NodesPerElem
    def get_entity_rank_names(*args):
        """get_entity_rank_names(unsigned int dim) -> vectors"""
        return _PerceptMesh.QuadFixture_3_get_entity_rank_names(*args)

    if _newclass:get_entity_rank_names = staticmethod(get_entity_rank_names)
    __swig_getmethods__["get_entity_rank_names"] = lambda x: get_entity_rank_names
    __swig_destroy__ = _PerceptMesh.delete_QuadFixture_3
    def __init__(self, *args): 
        """
        __init__(stk::percept::QuadFixture<(double,shards::Triangle<(3)>)> self, stk::ParallelMachine pm, unsigned int nx, unsigned int ny, bool generate_sidesets_in, 
            bool debug_geom_side_sets_as_blocks_in=False) -> QuadFixture_3
        __init__(stk::percept::QuadFixture<(double,shards::Triangle<(3)>)> self, stk::ParallelMachine pm, unsigned int nx, unsigned int ny, bool generate_sidesets_in) -> QuadFixture_3

        stk::percept::QuadFixture< Scalar, Topology
        >::QuadFixture(stk::ParallelMachine pm, unsigned nx, unsigned ny, bool
        generate_sidesets_in, bool debug_geom_side_sets_as_blocks_in=false) 
        """
        this = _PerceptMesh.new_QuadFixture_3(*args)
        try: self.this.append(this)
        except: self.this = this
    def set_bounding_box(self, *args):
        """
        set_bounding_box(QuadFixture_3 self, double xmin, double xmax, double ymin, double ymax)

        void stk::percept::QuadFixture< Scalar, Topology
        >::set_bounding_box(double xmin, double xmax, double ymin, double
        ymax) 
        """
        return _PerceptMesh.QuadFixture_3_set_bounding_box(self, *args)

    def generate_mesh(self, *args):
        """
        generate_mesh(QuadFixture_3 self)
        generate_mesh(QuadFixture_3 self, std::vector< stk::mesh::EntityId,std::allocator< stk::mesh::EntityId > > & element_ids_on_this_processor)

        void
        stk::percept::QuadFixture< Scalar, Topology
        >::generate_mesh(std::vector< stk::mesh::EntityId >
        &element_ids_on_this_processor) 
        """
        return _PerceptMesh.QuadFixture_3_generate_mesh(self, *args)

    __swig_getmethods__["meta_data"] = _PerceptMesh.QuadFixture_3_meta_data_get
    if _newclass:meta_data = _swig_property(_PerceptMesh.QuadFixture_3_meta_data_get)
    __swig_getmethods__["bulk_data"] = _PerceptMesh.QuadFixture_3_bulk_data_get
    if _newclass:bulk_data = _swig_property(_PerceptMesh.QuadFixture_3_bulk_data_get)
    __swig_getmethods__["quad_part"] = _PerceptMesh.QuadFixture_3_quad_part_get
    if _newclass:quad_part = _swig_property(_PerceptMesh.QuadFixture_3_quad_part_get)
    __swig_getmethods__["coord_field"] = _PerceptMesh.QuadFixture_3_coord_field_get
    if _newclass:coord_field = _swig_property(_PerceptMesh.QuadFixture_3_coord_field_get)
    __swig_getmethods__["coord_gather_field"] = _PerceptMesh.QuadFixture_3_coord_gather_field_get
    if _newclass:coord_gather_field = _swig_property(_PerceptMesh.QuadFixture_3_coord_gather_field_get)
    __swig_getmethods__["NX"] = _PerceptMesh.QuadFixture_3_NX_get
    if _newclass:NX = _swig_property(_PerceptMesh.QuadFixture_3_NX_get)
    __swig_getmethods__["NY"] = _PerceptMesh.QuadFixture_3_NY_get
    if _newclass:NY = _swig_property(_PerceptMesh.QuadFixture_3_NY_get)
    __swig_setmethods__["side_parts"] = _PerceptMesh.QuadFixture_3_side_parts_set
    __swig_getmethods__["side_parts"] = _PerceptMesh.QuadFixture_3_side_parts_get
    if _newclass:side_parts = _swig_property(_PerceptMesh.QuadFixture_3_side_parts_get, _PerceptMesh.QuadFixture_3_side_parts_set)
    __swig_setmethods__["generate_sidesets"] = _PerceptMesh.QuadFixture_3_generate_sidesets_set
    __swig_getmethods__["generate_sidesets"] = _PerceptMesh.QuadFixture_3_generate_sidesets_get
    if _newclass:generate_sidesets = _swig_property(_PerceptMesh.QuadFixture_3_generate_sidesets_get, _PerceptMesh.QuadFixture_3_generate_sidesets_set)
    __swig_setmethods__["debug_geom_side_sets_as_blocks"] = _PerceptMesh.QuadFixture_3_debug_geom_side_sets_as_blocks_set
    __swig_getmethods__["debug_geom_side_sets_as_blocks"] = _PerceptMesh.QuadFixture_3_debug_geom_side_sets_as_blocks_get
    if _newclass:debug_geom_side_sets_as_blocks = _swig_property(_PerceptMesh.QuadFixture_3_debug_geom_side_sets_as_blocks_get, _PerceptMesh.QuadFixture_3_debug_geom_side_sets_as_blocks_set)
    def node_id(self, *args):
        """
        node_id(QuadFixture_3 self, unsigned int ix, unsigned int iy) -> stk::mesh::EntityId

        stk::mesh::EntityId stk::percept::QuadFixture< Scalar, Topology
        >::node_id(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_3_node_id(self, *args)

    def elem_id(self, *args):
        """
        elem_id(QuadFixture_3 self, unsigned int ix, unsigned int iy) -> stk::mesh::EntityId

        stk::mesh::EntityId stk::percept::QuadFixture< Scalar, Topology
        >::elem_id(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_3_elem_id(self, *args)

    def node(self, *args):
        """
        node(QuadFixture_3 self, unsigned int ix, unsigned int iy) -> stk::mesh::Entity *

        stk::mesh::Entity* stk::percept::QuadFixture< Scalar, Topology
        >::node(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_3_node(self, *args)

    def node_ix_iy(self, *args):
        """
        node_ix_iy(QuadFixture_3 self, stk::mesh::EntityId entity_id, unsigned int & ix, unsigned int & iy)

        void
        stk::percept::QuadFixture< Scalar, Topology
        >::node_ix_iy(stk::mesh::EntityId entity_id, unsigned &ix, unsigned
        &iy) const 
        """
        return _PerceptMesh.QuadFixture_3_node_ix_iy(self, *args)

    def elem_ix_iy(self, *args):
        """
        elem_ix_iy(QuadFixture_3 self, stk::mesh::EntityId entity_id, unsigned int & ix, unsigned int & iy)

        void
        stk::percept::QuadFixture< Scalar, Topology
        >::elem_ix_iy(stk::mesh::EntityId entity_id, unsigned &ix, unsigned
        &iy) const 
        """
        return _PerceptMesh.QuadFixture_3_elem_ix_iy(self, *args)

    def elem(self, *args):
        """
        elem(QuadFixture_3 self, unsigned int ix, unsigned int iy) -> stk::mesh::Entity *

        stk::mesh::Entity* stk::percept::QuadFixture< Scalar, Topology
        >::elem(unsigned ix, unsigned iy) const 
        """
        return _PerceptMesh.QuadFixture_3_elem(self, *args)

    def generate_sides_meta(self):
        """
        generate_sides_meta(QuadFixture_3 self)

        void stk::percept::QuadFixture< Scalar, Topology
        >::generate_sides_meta() 
        """
        return _PerceptMesh.QuadFixture_3_generate_sides_meta(self)

    def generate_sides_bulk(self, *args):
        """
        generate_sides_bulk(QuadFixture_3 self, std::vector< stk::mesh::EntityId,std::allocator< stk::mesh::EntityId > > & element_ids_on_this_processor)

        void stk::percept::QuadFixture< Scalar, Topology
        >::generate_sides_bulk(std::vector< stk::mesh::EntityId >
        &element_ids_on_this_processor) 
        """
        return _PerceptMesh.QuadFixture_3_generate_sides_bulk(self, *args)

QuadFixture_3_swigregister = _PerceptMesh.QuadFixture_3_swigregister
QuadFixture_3_swigregister(QuadFixture_3)

def QuadFixture_3_get_entity_rank_names(*args):
  """QuadFixture_3_get_entity_rank_names(unsigned int dim) -> vectors"""
  return _PerceptMesh.QuadFixture_3_get_entity_rank_names(*args)

class WedgeFixture(_object):
    """Proxy of C++ stk::percept::WedgeFixture class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, WedgeFixture, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, WedgeFixture, name)
    __repr__ = _swig_repr
    def createMesh(self, *args):
        """
        createMesh(WedgeFixture self, stk::ParallelMachine parallel_machine, unsigned int n_nodes_x, unsigned int n_nodes_y, 
            unsigned int n_nodes_z, double xmin, double xmax, double ymin, double ymax, 
            double zmin, double zmax, std::string output_filename) -> BulkData

        mesh::BulkData*
        stk::percept::WedgeFixture::createMesh(stk::ParallelMachine
        parallel_machine, unsigned n_nodes_x, unsigned n_nodes_y, unsigned
        n_nodes_z, double xmin, double xmax, double ymin, double ymax, double
        zmin, double zmax, std::string output_filename) 
        """
        return _PerceptMesh.WedgeFixture_createMesh(self, *args)

    def getMetaData(self):
        """
        getMetaData(WedgeFixture self) -> FEMMetaData

        mesh::fem::FEMMetaData* stk::percept::WedgeFixture::getMetaData() 
        """
        return _PerceptMesh.WedgeFixture_getMetaData(self)

    def createBulkAfterMetaCommit(self, *args):
        """
        createBulkAfterMetaCommit(WedgeFixture self, stk::ParallelMachine parallel_machine)

        void
        stk::percept::WedgeFixture::createBulkAfterMetaCommit(stk::ParallelMachine
        parallel_machine) 
        """
        return _PerceptMesh.WedgeFixture_createBulkAfterMetaCommit(self, *args)

    def createFixedSizeMesh(self, *args):
        """
        createFixedSizeMesh(WedgeFixture self, stk::ParallelMachine parallel_machine, std::string output_filename)

        void
        stk::percept::WedgeFixture::createFixedSizeMesh(stk::ParallelMachine
        parallel_machine, std::string output_filename) 
        """
        return _PerceptMesh.WedgeFixture_createFixedSizeMesh(self, *args)

    def __init__(self): 
        """__init__(stk::percept::WedgeFixture self) -> WedgeFixture"""
        this = _PerceptMesh.new_WedgeFixture()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_WedgeFixture
WedgeFixture_swigregister = _PerceptMesh.WedgeFixture_swigregister
WedgeFixture_swigregister(WedgeFixture)

HET_FIX_INCLUDE_EXTRA_ELEM_TYPES = _PerceptMesh.HET_FIX_INCLUDE_EXTRA_ELEM_TYPES
class BeamFixture(_object):
    """
    Use case with mixed element topologies and field relations to provide
    fast access to node field data from an element.

    copied from stk_mesh and modified

    C++ includes: BeamFixture.hpp 
    """
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, BeamFixture, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, BeamFixture, name)
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_BeamFixture
    def __init__(self, *args): 
        """
        __init__(stk::percept::BeamFixture self, stk::ParallelMachine comm, bool doCommit=True) -> BeamFixture
        __init__(stk::percept::BeamFixture self, stk::ParallelMachine comm) -> BeamFixture

        stk::percept::BeamFixture::BeamFixture(stk::ParallelMachine comm, bool
        doCommit=true) 
        """
        this = _PerceptMesh.new_BeamFixture(*args)
        try: self.this.append(this)
        except: self.this = this
    def populate(self):
        """
        populate(BeamFixture self)

        void
        stk::percept::BeamFixture::populate() 
        """
        return _PerceptMesh.BeamFixture_populate(self)

    __swig_getmethods__["m_spatial_dimension"] = _PerceptMesh.BeamFixture_m_spatial_dimension_get
    if _newclass:m_spatial_dimension = _swig_property(_PerceptMesh.BeamFixture_m_spatial_dimension_get)
    __swig_getmethods__["m_metaData"] = _PerceptMesh.BeamFixture_m_metaData_get
    if _newclass:m_metaData = _swig_property(_PerceptMesh.BeamFixture_m_metaData_get)
    __swig_getmethods__["m_bulkData"] = _PerceptMesh.BeamFixture_m_bulkData_get
    if _newclass:m_bulkData = _swig_property(_PerceptMesh.BeamFixture_m_bulkData_get)
    __swig_getmethods__["m_block_beam"] = _PerceptMesh.BeamFixture_m_block_beam_get
    if _newclass:m_block_beam = _swig_property(_PerceptMesh.BeamFixture_m_block_beam_get)
    __swig_getmethods__["m_elem_rank"] = _PerceptMesh.BeamFixture_m_elem_rank_get
    if _newclass:m_elem_rank = _swig_property(_PerceptMesh.BeamFixture_m_elem_rank_get)
    __swig_getmethods__["m_coordinates_field"] = _PerceptMesh.BeamFixture_m_coordinates_field_get
    if _newclass:m_coordinates_field = _swig_property(_PerceptMesh.BeamFixture_m_coordinates_field_get)
    __swig_getmethods__["m_centroid_field"] = _PerceptMesh.BeamFixture_m_centroid_field_get
    if _newclass:m_centroid_field = _swig_property(_PerceptMesh.BeamFixture_m_centroid_field_get)
    __swig_getmethods__["m_temperature_field"] = _PerceptMesh.BeamFixture_m_temperature_field_get
    if _newclass:m_temperature_field = _swig_property(_PerceptMesh.BeamFixture_m_temperature_field_get)
    __swig_getmethods__["m_volume_field"] = _PerceptMesh.BeamFixture_m_volume_field_get
    if _newclass:m_volume_field = _swig_property(_PerceptMesh.BeamFixture_m_volume_field_get)
    __swig_getmethods__["m_element_node_coordinates_field"] = _PerceptMesh.BeamFixture_m_element_node_coordinates_field_get
    if _newclass:m_element_node_coordinates_field = _swig_property(_PerceptMesh.BeamFixture_m_element_node_coordinates_field_get)
BeamFixture_swigregister = _PerceptMesh.BeamFixture_swigregister
BeamFixture_swigregister(BeamFixture)

class HeterogeneousFixture(_object):
    """
    Use case with mixed element topologies and field relations to provide
    fast access to node field data from an element.

    copied from stk_mesh and modified

    C++ includes: HeterogeneousFixture.hpp 
    """
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, HeterogeneousFixture, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, HeterogeneousFixture, name)
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_HeterogeneousFixture
    def __init__(self, *args): 
        """
        __init__(stk::percept::HeterogeneousFixture self, stk::ParallelMachine comm, bool doCommit=True, bool do_sidesets=False) -> HeterogeneousFixture
        __init__(stk::percept::HeterogeneousFixture self, stk::ParallelMachine comm, bool doCommit=True) -> HeterogeneousFixture
        __init__(stk::percept::HeterogeneousFixture self, stk::ParallelMachine comm) -> HeterogeneousFixture

        stk::percept::HeterogeneousFixture::HeterogeneousFixture(stk::ParallelMachine
        comm, bool doCommit=true, bool do_sidesets=false) 
        """
        this = _PerceptMesh.new_HeterogeneousFixture(*args)
        try: self.this.append(this)
        except: self.this = this
    def populate(self):
        """
        populate(HeterogeneousFixture self)

        void stk::percept::HeterogeneousFixture::populate() 
        """
        return _PerceptMesh.HeterogeneousFixture_populate(self)

    __swig_getmethods__["m_spatial_dimension"] = _PerceptMesh.HeterogeneousFixture_m_spatial_dimension_get
    if _newclass:m_spatial_dimension = _swig_property(_PerceptMesh.HeterogeneousFixture_m_spatial_dimension_get)
    __swig_getmethods__["m_metaData"] = _PerceptMesh.HeterogeneousFixture_m_metaData_get
    if _newclass:m_metaData = _swig_property(_PerceptMesh.HeterogeneousFixture_m_metaData_get)
    __swig_getmethods__["m_bulkData"] = _PerceptMesh.HeterogeneousFixture_m_bulkData_get
    if _newclass:m_bulkData = _swig_property(_PerceptMesh.HeterogeneousFixture_m_bulkData_get)
    __swig_getmethods__["m_block_hex"] = _PerceptMesh.HeterogeneousFixture_m_block_hex_get
    if _newclass:m_block_hex = _swig_property(_PerceptMesh.HeterogeneousFixture_m_block_hex_get)
    __swig_getmethods__["m_block_wedge"] = _PerceptMesh.HeterogeneousFixture_m_block_wedge_get
    if _newclass:m_block_wedge = _swig_property(_PerceptMesh.HeterogeneousFixture_m_block_wedge_get)
    __swig_getmethods__["m_block_tet"] = _PerceptMesh.HeterogeneousFixture_m_block_tet_get
    if _newclass:m_block_tet = _swig_property(_PerceptMesh.HeterogeneousFixture_m_block_tet_get)
    __swig_getmethods__["m_block_pyramid"] = _PerceptMesh.HeterogeneousFixture_m_block_pyramid_get
    if _newclass:m_block_pyramid = _swig_property(_PerceptMesh.HeterogeneousFixture_m_block_pyramid_get)
    __swig_setmethods__["m_sideset_quad"] = _PerceptMesh.HeterogeneousFixture_m_sideset_quad_set
    __swig_getmethods__["m_sideset_quad"] = _PerceptMesh.HeterogeneousFixture_m_sideset_quad_get
    if _newclass:m_sideset_quad = _swig_property(_PerceptMesh.HeterogeneousFixture_m_sideset_quad_get, _PerceptMesh.HeterogeneousFixture_m_sideset_quad_set)
    __swig_setmethods__["m_sideset_quad_subset"] = _PerceptMesh.HeterogeneousFixture_m_sideset_quad_subset_set
    __swig_getmethods__["m_sideset_quad_subset"] = _PerceptMesh.HeterogeneousFixture_m_sideset_quad_subset_get
    if _newclass:m_sideset_quad_subset = _swig_property(_PerceptMesh.HeterogeneousFixture_m_sideset_quad_subset_get, _PerceptMesh.HeterogeneousFixture_m_sideset_quad_subset_set)
    __swig_setmethods__["m_sideset_tri"] = _PerceptMesh.HeterogeneousFixture_m_sideset_tri_set
    __swig_getmethods__["m_sideset_tri"] = _PerceptMesh.HeterogeneousFixture_m_sideset_tri_get
    if _newclass:m_sideset_tri = _swig_property(_PerceptMesh.HeterogeneousFixture_m_sideset_tri_get, _PerceptMesh.HeterogeneousFixture_m_sideset_tri_set)
    __swig_setmethods__["m_sideset_tri_subset"] = _PerceptMesh.HeterogeneousFixture_m_sideset_tri_subset_set
    __swig_getmethods__["m_sideset_tri_subset"] = _PerceptMesh.HeterogeneousFixture_m_sideset_tri_subset_get
    if _newclass:m_sideset_tri_subset = _swig_property(_PerceptMesh.HeterogeneousFixture_m_sideset_tri_subset_get, _PerceptMesh.HeterogeneousFixture_m_sideset_tri_subset_set)
    __swig_getmethods__["m_elem_rank"] = _PerceptMesh.HeterogeneousFixture_m_elem_rank_get
    if _newclass:m_elem_rank = _swig_property(_PerceptMesh.HeterogeneousFixture_m_elem_rank_get)
    __swig_getmethods__["m_coordinates_field"] = _PerceptMesh.HeterogeneousFixture_m_coordinates_field_get
    if _newclass:m_coordinates_field = _swig_property(_PerceptMesh.HeterogeneousFixture_m_coordinates_field_get)
    __swig_getmethods__["m_centroid_field"] = _PerceptMesh.HeterogeneousFixture_m_centroid_field_get
    if _newclass:m_centroid_field = _swig_property(_PerceptMesh.HeterogeneousFixture_m_centroid_field_get)
    __swig_getmethods__["m_temperature_field"] = _PerceptMesh.HeterogeneousFixture_m_temperature_field_get
    if _newclass:m_temperature_field = _swig_property(_PerceptMesh.HeterogeneousFixture_m_temperature_field_get)
    __swig_getmethods__["m_volume_field"] = _PerceptMesh.HeterogeneousFixture_m_volume_field_get
    if _newclass:m_volume_field = _swig_property(_PerceptMesh.HeterogeneousFixture_m_volume_field_get)
    __swig_getmethods__["m_element_node_coordinates_field"] = _PerceptMesh.HeterogeneousFixture_m_element_node_coordinates_field_get
    if _newclass:m_element_node_coordinates_field = _swig_property(_PerceptMesh.HeterogeneousFixture_m_element_node_coordinates_field_get)
HeterogeneousFixture_swigregister = _PerceptMesh.HeterogeneousFixture_swigregister
HeterogeneousFixture_swigregister(HeterogeneousFixture)


def verifyMesh(*args):
  """
    verifyMesh(BeamFixture mesh) -> bool
    verifyMesh(HeterogeneousFixture mesh) -> bool
    """
  return _PerceptMesh.verifyMesh(*args)
class GenericFunction(_object):
    """Proxy of C++ stk::percept::GenericFunction class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, GenericFunction, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, GenericFunction, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_GenericFunction
    def isSpatialOperator(self):
        """
        isSpatialOperator(GenericFunction self) -> bool

        bool
        stk::percept::GenericFunction::isSpatialOperator() 
        """
        return _PerceptMesh.GenericFunction_isSpatialOperator(self)

    def setIsSpatialOperator(self, *args):
        """
        setIsSpatialOperator(GenericFunction self, bool so)

        void
        stk::percept::GenericFunction::setIsSpatialOperator(bool so) 
        """
        return _PerceptMesh.GenericFunction_setIsSpatialOperator(self, *args)

    def getDomainDimensions(self):
        """
        getDomainDimensions(GenericFunction self) -> Dimensions

        Dimensions
        stk::percept::GenericFunction::getDomainDimensions() 
        """
        return _PerceptMesh.GenericFunction_getDomainDimensions(self)

    def getCodomainDimensions(self):
        """
        getCodomainDimensions(GenericFunction self) -> Dimensions

        Dimensions
        stk::percept::GenericFunction::getCodomainDimensions() 
        """
        return _PerceptMesh.GenericFunction_getCodomainDimensions(self)

    def getNewDomain(self):
        """
        getNewDomain(GenericFunction self) -> MDArray

        MDArray stk::percept::GenericFunction::getNewDomain() 
        """
        return _PerceptMesh.GenericFunction_getNewDomain(self)

    def getNewCodomain(self):
        """
        getNewCodomain(GenericFunction self) -> MDArray

        MDArray stk::percept::GenericFunction::getNewCodomain() const 
        """
        return _PerceptMesh.GenericFunction_getNewCodomain(self)

    def getNewMDArray(*args):
        """getNewMDArray(Dimensions dims) -> MDArray"""
        return _PerceptMesh.GenericFunction_getNewMDArray(*args)

    if _newclass:getNewMDArray = staticmethod(getNewMDArray)
    __swig_getmethods__["getNewMDArray"] = lambda x: getNewMDArray
GenericFunction_swigregister = _PerceptMesh.GenericFunction_swigregister
GenericFunction_swigregister(GenericFunction)

def GenericFunction_getNewMDArray(*args):
  """GenericFunction_getNewMDArray(Dimensions dims) -> MDArray"""
  return _PerceptMesh.GenericFunction_getNewMDArray(*args)

class Function(GenericFunction):
    """Proxy of C++ stk::percept::Function class"""
    __swig_setmethods__ = {}
    for _s in [GenericFunction]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, Function, name, value)
    __swig_getmethods__ = {}
    for _s in [GenericFunction]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, Function, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    def value(self, *args):
        """
        value(Function self, MDArray domain, MDArray MDOutVal, double time_value_optional=0.0)
        value(Function self, MDArray domain, MDArray MDOutVal)
        value(Function self, MDArray domain, double time_value_optional=0.0) -> MDArray
        value(Function self, MDArray domain) -> MDArray

        MDArray
        stk::percept::Function::value(MDArray &domain, double
        time_value_optional=0.0)

        this version creates a dummy output and returns it based on the
        codomain-dimensions specified at construction 
        """
        return _PerceptMesh.Function_value(self, *args)

    def derivative(self, *args):
        """
        derivative(Function self, MDArrayString deriv_spec) -> RCPFunction

        virtual
        Teuchos::RCP<Function >
        stk::percept::Function::derivative(MDArrayString &deriv_spec)

        Return a function that is the derivative of this function. The
        derivative is specified as a rank-2 array of strings that specify what
        derivative to take and how many derivatives. For example, 
        """
        return _PerceptMesh.Function_derivative(self, *args)

    def gradient(self, spatialDim=3):
        """
        gradient(Function self, int spatialDim=3) -> RCPFunction
        gradient(Function self) -> RCPFunction

        virtual
        Teuchos::RCP<Function > stk::percept::Function::gradient(int
        spatialDim=3) 
        """
        return _PerceptMesh.Function_gradient(self, spatialDim)

    def derivativeAtPoint(self, *args):
        """
        derivativeAtPoint(Function self, MDArrayString deriv_spec, MDArray domain, MDArray codomain, double time=0.0)
        derivativeAtPoint(Function self, MDArrayString deriv_spec, MDArray domain, MDArray codomain)

        void stk::percept::Function::derivativeAtPoint(MDArrayString
        &deriv_spec, MDArray &domain, MDArray &codomain, double time=0.0) 
        """
        return _PerceptMesh.Function_derivativeAtPoint(self, *args)

    def setIntegrationOrderDefault(*args):
        """setIntegrationOrderDefault(unsigned int integration_order)"""
        return _PerceptMesh.Function_setIntegrationOrderDefault(*args)

    if _newclass:setIntegrationOrderDefault = staticmethod(setIntegrationOrderDefault)
    __swig_getmethods__["setIntegrationOrderDefault"] = lambda x: setIntegrationOrderDefault
    def setSpatialDimDefault(*args):
        """setSpatialDimDefault(unsigned int spatialDim)"""
        return _PerceptMesh.Function_setSpatialDimDefault(*args)

    if _newclass:setSpatialDimDefault = staticmethod(setSpatialDimDefault)
    __swig_getmethods__["setSpatialDimDefault"] = lambda x: setSpatialDimDefault
    def setCodomainDimDefault(*args):
        """setCodomainDimDefault(unsigned int codomainDim)"""
        return _PerceptMesh.Function_setCodomainDimDefault(*args)

    if _newclass:setCodomainDimDefault = staticmethod(setCodomainDimDefault)
    __swig_getmethods__["setCodomainDimDefault"] = lambda x: setCodomainDimDefault
    def add_alias(self, *args):
        """
        add_alias(Function self, char const * alias) -> Function

        Function *
        stk::percept::Function::add_alias(const char *alias)

        allow this function to have one or more aliases 
        """
        return _PerceptMesh.Function_add_alias(self, *args)

    def getName(self):
        """
        getName(Function self) -> std::string &

        std::string&
        stk::percept::Function::getName() 
        """
        return _PerceptMesh.Function_getName(self)

    def setIntegrationOrder(self, *args):
        """
        setIntegrationOrder(Function self, unsigned int iord)

        void stk::percept::Function::setIntegrationOrder(unsigned iord) 
        """
        return _PerceptMesh.Function_setIntegrationOrder(self, *args)

    def getIntegrationOrder(self):
        """
        getIntegrationOrder(Function self) -> unsigned int

        unsigned stk::percept::Function::getIntegrationOrder(void) 
        """
        return _PerceptMesh.Function_getIntegrationOrder(self)

    def setDomainDimensions(self, *args):
        """
        setDomainDimensions(Function self, Dimensions dims)

        void stk::percept::Function::setDomainDimensions(const Dimensions
        dims) 
        """
        return _PerceptMesh.Function_setDomainDimensions(self, *args)

    def setCodomainDimensions(self, *args):
        """
        setCodomainDimensions(Function self, Dimensions dims)

        void stk::percept::Function::setCodomainDimensions(const Dimensions
        dims) 
        """
        return _PerceptMesh.Function_setCodomainDimensions(self, *args)

    def argsAreValid(self, *args):
        """
        argsAreValid(Function self, MDArray _in) -> bool

        bool
        stk::percept::Function::argsAreValid(const MDArray &in, const MDArray
        &out)

        Verify that the last dimensions of

        Parameters:
        -----------

        in:  and

        out:  are the same; this allows Functions to be invoked at multiple
        points where the first M indices represent an M-d array of points to
        evaluate the function, while the last N indices should match the
        Functions domain and codomain dimensions. 
        """
        return _PerceptMesh.Function_argsAreValid(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_Function
Function_swigregister = _PerceptMesh.Function_swigregister
Function_swigregister(Function)

def Function_setIntegrationOrderDefault(*args):
  """Function_setIntegrationOrderDefault(unsigned int integration_order)"""
  return _PerceptMesh.Function_setIntegrationOrderDefault(*args)

def Function_setSpatialDimDefault(*args):
  """Function_setSpatialDimDefault(unsigned int spatialDim)"""
  return _PerceptMesh.Function_setSpatialDimDefault(*args)

def Function_setCodomainDimDefault(*args):
  """Function_setCodomainDimDefault(unsigned int codomainDim)"""
  return _PerceptMesh.Function_setCodomainDimDefault(*args)
Function.Identity = _PerceptMesh.cvar.Function_Identity


def eval_func(*args):
  """
    eval_func(double x, double y, double z, double t, Function func) -> double
    eval_func(double x, double y, double z, double t, RCPFunction func) -> double
    """
  return _PerceptMesh.eval_func(*args)

def eval_func2(*args):
  """
    eval_func2(double x, double y, double t, Function func) -> double
    eval_func2(double x, double y, double t, RCPFunction func) -> double
    """
  return _PerceptMesh.eval_func2(*args)

def eval_print(*args):
  """
    eval_print(double x, double y, double z, double t, Function func)
    eval_print(double x, double y, double z, double t, RCPFunction func)
    """
  return _PerceptMesh.eval_print(*args)

def eval_print2(*args):
  """
    eval_print2(double x, double y, double t, Function func)
    eval_print2(double x, double y, double t, RCPFunction func)
    """
  return _PerceptMesh.eval_print2(*args)

def eval_vec3(*args):
  """
    eval_vec3(double x, double y, double z, double t, Function func) -> MDArray
    eval_vec3(double x, double y, double z, double t, RCPFunction func) -> MDArray
    """
  return _PerceptMesh.eval_vec3(*args)

def eval_vec3_print(*args):
  """
    eval_vec3_print(double x, double y, double z, double t, Function func)
    eval_vec3_print(double x, double y, double z, double t, RCPFunction func)
    """
  return _PerceptMesh.eval_vec3_print(*args)
class FieldFunction(Function):
    """
    Evaluate the function at this input point (or points) returning
    value(s) in output_field_values

    In the following, the arrays are dimensioned using the notation (from
    Intrepid's doc):

    [C] - num. integration domains (cells/elements) [F] - num. Intrepid
    "fields" (number of bases within an element == num. nodes typically)
    [P] - num. integration (or interpolation) points within the element
    [D] - spatial dimension [D1], [D2] - spatial dimension

    Locally, we introduce this notation:

    [DOF] - number of degrees-of-freedom per node of the interpolated stk
    Field. For example, a vector field in 3D has [DOF] = 3

    C++ includes: FieldFunction.hpp 
    """
    __swig_setmethods__ = {}
    for _s in [Function]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, FieldFunction, name, value)
    __swig_getmethods__ = {}
    for _s in [Function]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, FieldFunction, name)
    __repr__ = _swig_repr
    __swig_setmethods__["m_parallelEval"] = _PerceptMesh.FieldFunction_m_parallelEval_set
    __swig_getmethods__["m_parallelEval"] = _PerceptMesh.FieldFunction_m_parallelEval_get
    if _newclass:m_parallelEval = _swig_property(_PerceptMesh.FieldFunction_m_parallelEval_get, _PerceptMesh.FieldFunction_m_parallelEval_set)
    SIMPLE_SEARCH = _PerceptMesh.FieldFunction_SIMPLE_SEARCH
    STK_SEARCH = _PerceptMesh.FieldFunction_STK_SEARCH
    def get_field(self):
        """
        get_field(FieldFunction self) -> FieldBase

        mesh::FieldBase * stk::percept::FieldFunction::get_field() 
        """
        return _PerceptMesh.FieldFunction_get_field(self)

    def interpolateFrom(self, *args):
        """
        interpolateFrom(FieldFunction self, Function function)

        void stk::percept::FieldFunction::interpolateFrom(Function &function)

        for each node in the codomain, evaluate the function_to_interpolate's
        function, assign to the codomain field 
        """
        return _PerceptMesh.FieldFunction_interpolateFrom(self, *args)

    def __init__(self, *args): 
        """
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh mesh, int domain_dimension=3, int codomain_dimension=1, 
            stk::percept::FieldFunction::SearchType searchType=SIMPLE_SEARCH, 
            unsigned int integration_order=0) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh mesh, int domain_dimension=3, int codomain_dimension=1, 
            stk::percept::FieldFunction::SearchType searchType=SIMPLE_SEARCH) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh mesh, int domain_dimension=3, int codomain_dimension=1) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh mesh, int domain_dimension=3) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh mesh) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, BulkData bulk, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions(), 
            stk::percept::FieldFunction::SearchType searchType=SIMPLE_SEARCH, 
            unsigned int integration_order=0) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, BulkData bulk, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions(), 
            stk::percept::FieldFunction::SearchType searchType=SIMPLE_SEARCH) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, BulkData bulk, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions()) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, BulkData bulk, Dimensions domain_dimensions=stk::percept::Dimensions()) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, BulkData bulk) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh eMesh, Dimensions domain_dimensions, 
            Dimensions codomain_dimensions, stk::percept::FieldFunction::SearchType searchType=SIMPLE_SEARCH, 
            unsigned int integration_order=0) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh eMesh, Dimensions domain_dimensions, 
            Dimensions codomain_dimensions, stk::percept::FieldFunction::SearchType searchType=SIMPLE_SEARCH) -> FieldFunction
        __init__(stk::percept::FieldFunction self, char const * name, FieldBase field, PerceptMesh eMesh, Dimensions domain_dimensions, 
            Dimensions codomain_dimensions) -> FieldFunction

        stk::percept::FieldFunction::FieldFunction(const char *name,
        mesh::FieldBase *field, PerceptMesh &eMesh, Dimensions
        domain_dimensions, Dimensions codomain_dimensions, SearchType
        searchType=SIMPLE_SEARCH, unsigned integration_order=0) 
        """
        this = _PerceptMesh.new_FieldFunction(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_FieldFunction
    def derivative(self, *args):
        """
        derivative(FieldFunction self, MDArrayString deriv_spec) -> RCPFunction

        virtual Teuchos::RCP<Function >
        stk::percept::FieldFunction::derivative(MDArrayString &deriv_spec)

        Return a function that is the derivative of this function. The
        derivative is specified as a rank-2 array of strings that specify what
        derivative to take and how many derivatives. For example, 
        """
        return _PerceptMesh.FieldFunction_derivative(self, *args)

    def gradient(self, spatialDim=3):
        """
        gradient(FieldFunction self, int spatialDim=3) -> RCPFunction
        gradient(FieldFunction self) -> RCPFunction

        virtual
        Teuchos::RCP<Function > stk::percept::FieldFunction::gradient(int
        spatialDim=3) 
        """
        return _PerceptMesh.FieldFunction_gradient(self, spatialDim)

    def localEvaluation(self, *args):
        """
        localEvaluation(FieldFunction self, MDArray _in, double time_value_optional=0.0)
        localEvaluation(FieldFunction self, MDArray _in)

        void stk::percept::FieldFunction::localEvaluation(MDArray &in, MDArray
        &out, double time_value_optional=0.0) 
        """
        return _PerceptMesh.FieldFunction_localEvaluation(self, *args)

    def setup_searcher(self, *args):
        """
        setup_searcher(FieldFunction self, int D_)

        void stk::percept::FieldFunction::setup_searcher(int D_) 
        """
        return _PerceptMesh.FieldFunction_setup_searcher(self, *args)

    def get_bulk_data(self):
        """
        get_bulk_data(FieldFunction self) -> BulkData

        mesh::BulkData * stk::percept::FieldFunction::get_bulk_data() 
        """
        return _PerceptMesh.FieldFunction_get_bulk_data(self)

    def getFoundOnLocalOwnedPart(self):
        """
        getFoundOnLocalOwnedPart(FieldFunction self) -> bool

        bool
        stk::percept::FieldFunction::getFoundOnLocalOwnedPart() 
        """
        return _PerceptMesh.FieldFunction_getFoundOnLocalOwnedPart(self)

FieldFunction_swigregister = _PerceptMesh.FieldFunction_swigregister
FieldFunction_swigregister(FieldFunction)

EXTRA_PRINT_FF_HELPER = _PerceptMesh.EXTRA_PRINT_FF_HELPER
class ConstantFunction(Function):
    """Proxy of C++ stk::percept::ConstantFunction class"""
    __swig_setmethods__ = {}
    for _s in [Function]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, ConstantFunction, name, value)
    __swig_getmethods__ = {}
    for _s in [Function]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, ConstantFunction, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::ConstantFunction self, double value, char const * name, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions(), unsigned int integration_order=0) -> ConstantFunction
        __init__(stk::percept::ConstantFunction self, double value, char const * name, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions()) -> ConstantFunction
        __init__(stk::percept::ConstantFunction self, double value, char const * name, Dimensions domain_dimensions=stk::percept::Dimensions()) -> ConstantFunction
        __init__(stk::percept::ConstantFunction self, double value, char const * name) -> ConstantFunction

        stk::percept::ConstantFunction::ConstantFunction(double value, const
        char *name, Dimensions domain_dimensions=Dimensions(), Dimensions
        codomain_dimensions=Dimensions(), unsigned integration_order=0) 
        """
        this = _PerceptMesh.new_ConstantFunction(*args)
        try: self.this.append(this)
        except: self.this = this
    def getValue(self):
        """
        getValue(ConstantFunction self) -> double &

        double& stk::percept::ConstantFunction::getValue() 
        """
        return _PerceptMesh.ConstantFunction_getValue(self)

    def setValue(self, *args):
        """
        setValue(ConstantFunction self, double & v)

        void
        stk::percept::ConstantFunction::setValue(double &v) 
        """
        return _PerceptMesh.ConstantFunction_setValue(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_ConstantFunction
ConstantFunction_swigregister = _PerceptMesh.ConstantFunction_swigregister
ConstantFunction_swigregister(ConstantFunction)

class ConstantFunctionVec(Function):
    """Proxy of C++ stk::percept::ConstantFunctionVec class"""
    __swig_setmethods__ = {}
    for _s in [Function]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, ConstantFunctionVec, name, value)
    __swig_getmethods__ = {}
    for _s in [Function]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, ConstantFunctionVec, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::ConstantFunctionVec self, vectord value, char const * name, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions(), unsigned int integration_order=0) -> ConstantFunctionVec
        __init__(stk::percept::ConstantFunctionVec self, vectord value, char const * name, Dimensions domain_dimensions=stk::percept::Dimensions(), 
            Dimensions codomain_dimensions=stk::percept::Dimensions()) -> ConstantFunctionVec
        __init__(stk::percept::ConstantFunctionVec self, vectord value, char const * name, Dimensions domain_dimensions=stk::percept::Dimensions()) -> ConstantFunctionVec
        __init__(stk::percept::ConstantFunctionVec self, vectord value, char const * name) -> ConstantFunctionVec

        stk::percept::ConstantFunctionVec::ConstantFunctionVec(std::vector<
        double > &value, const char *name, Dimensions
        domain_dimensions=Dimensions(), Dimensions
        codomain_dimensions=Dimensions(), unsigned integration_order=0) 
        """
        this = _PerceptMesh.new_ConstantFunctionVec(*args)
        try: self.this.append(this)
        except: self.this = this
    def getValue(self):
        """
        getValue(ConstantFunctionVec self) -> vectord

        std::vector<double>& stk::percept::ConstantFunctionVec::getValue() 
        """
        return _PerceptMesh.ConstantFunctionVec_getValue(self)

    def setValue(self, *args):
        """
        setValue(ConstantFunctionVec self, vectord v)

        void stk::percept::ConstantFunctionVec::setValue(std::vector< double >
        &v) 
        """
        return _PerceptMesh.ConstantFunctionVec_setValue(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_ConstantFunctionVec
ConstantFunctionVec_swigregister = _PerceptMesh.ConstantFunctionVec_swigregister
ConstantFunctionVec_swigregister(ConstantFunctionVec)

class StringFunction(Function):
    """Proxy of C++ stk::percept::StringFunction class"""
    __swig_setmethods__ = {}
    for _s in [Function]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, StringFunction, name, value)
    __swig_getmethods__ = {}
    for _s in [Function]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, StringFunction, name)
    __repr__ = _swig_repr
    def getFunctionString(self):
        """
        getFunctionString(StringFunction self) -> std::string

        std::string stk::percept::StringFunction::getFunctionString() 
        """
        return _PerceptMesh.StringFunction_getFunctionString(self)

    def set_gradient_strings(self, *args):
        """
        set_gradient_strings(StringFunction self, std::string [3] gstring, int len)
        set_gradient_strings(StringFunction self, MDArrayString gstring)

        void
        stk::percept::StringFunction::set_gradient_strings(MDArrayString
        &gstring) 
        """
        return _PerceptMesh.StringFunction_set_gradient_strings(self, *args)

    def derivative(self, *args):
        """
        derivative(StringFunction self, MDArrayString deriv_spec) -> RCPFunction

        Teuchos::RCP< Function >
        stk::percept::StringFunction::derivative(MDArrayString &deriv_spec)

        Return a function that is the derivative of this function. The
        derivative is specified as a rank-2 array of strings that specify what
        derivative to take and how many derivatives. For example, 
        """
        return _PerceptMesh.StringFunction_derivative(self, *args)

    def gradient(self, spatialDim=3):
        """
        gradient(StringFunction self, int spatialDim=3) -> RCPFunction
        gradient(StringFunction self) -> RCPFunction

        Teuchos::RCP< Function > stk::percept::StringFunction::gradient(int
        spatialDim=3) 
        """
        return _PerceptMesh.StringFunction_gradient(self, spatialDim)

    def __init__(self, *args): 
        """
        __init__(stk::percept::StringFunction self, char const * function_string, Name name=stk::percept::Name("noname"), int domain_dimension=3, 
            int codomain_dimension=1, unsigned int integration_order=0) -> StringFunction
        __init__(stk::percept::StringFunction self, char const * function_string, Name name=stk::percept::Name("noname"), int domain_dimension=3, 
            int codomain_dimension=1) -> StringFunction
        __init__(stk::percept::StringFunction self, char const * function_string, Name name=stk::percept::Name("noname"), int domain_dimension=3) -> StringFunction
        __init__(stk::percept::StringFunction self, char const * function_string, Name name=stk::percept::Name("noname")) -> StringFunction
        __init__(stk::percept::StringFunction self, char const * function_string) -> StringFunction
        __init__(stk::percept::StringFunction self, char const * function_string, Name name, Dimensions domain_dimensions, Dimensions codomain_dimensions, 
            unsigned int integration_order=0) -> StringFunction
        __init__(stk::percept::StringFunction self, char const * function_string, Name name, Dimensions domain_dimensions, Dimensions codomain_dimensions) -> StringFunction
        __init__(stk::percept::StringFunction self, StringFunction s) -> StringFunction

        stk::percept::StringFunction::StringFunction(const StringFunction &s)

        """
        this = _PerceptMesh.new_StringFunction(*args)
        try: self.this.append(this)
        except: self.this = this
    def resolve(self, *args):
        """
        resolve(StringFunction self, stk::expreval::VariableMap::iterator & var_it)

        void
        stk::percept::StringFunction::resolve(stk::expreval::VariableMap::iterator
        &var_it) 
        """
        return _PerceptMesh.StringFunction_resolve(self, *args)

    def derivative_test(self, *args):
        """
        derivative_test(StringFunction self, MDArrayString deriv_spec) -> RCPFunction

        Teuchos::RCP< Function >
        stk::percept::StringFunction::derivative_test(MDArrayString
        &deriv_spec) 
        """
        return _PerceptMesh.StringFunction_derivative_test(self, *args)

    def derivative_test_fd(self, *args):
        """
        derivative_test_fd(StringFunction self, MDArrayString deriv_spec, double eps=1.e-6) -> RCPFunction
        derivative_test_fd(StringFunction self, MDArrayString deriv_spec) -> RCPFunction

        Teuchos::RCP<
        Function >
        stk::percept::StringFunction::derivative_test_fd(MDArrayString
        &deriv_spec, double eps=1.e-6) 
        """
        return _PerceptMesh.StringFunction_derivative_test_fd(self, *args)

    def __add__(self, *args):
        """__add__(StringFunction self, StringFunction rhs) -> StringFunction"""
        return _PerceptMesh.StringFunction___add__(self, *args)

    def __div__(self, *args):
        """__div__(StringFunction self, StringFunction rhs) -> StringFunction"""
        return _PerceptMesh.StringFunction___div__(self, *args)

    def __mul__(self, *args):
        """__mul__(StringFunction self, StringFunction rhs) -> StringFunction"""
        return _PerceptMesh.StringFunction___mul__(self, *args)

    def __sub__(self, *args):
        """__sub__(StringFunction self, StringFunction rhs) -> StringFunction"""
        return _PerceptMesh.StringFunction___sub__(self, *args)

    __swig_destroy__ = _PerceptMesh.delete_StringFunction
StringFunction_swigregister = _PerceptMesh.StringFunction_swigregister
StringFunction_swigregister(StringFunction)


def __add__(*args):
  """__add__(StringFunction lhs, StringFunction rhs) -> StringFunction"""
  return _PerceptMesh.__add__(*args)

def __div__(*args):
  """__div__(StringFunction lhs, StringFunction rhs) -> StringFunction"""
  return _PerceptMesh.__div__(*args)

def __mul__(*args):
  """__mul__(StringFunction lhs, StringFunction rhs) -> StringFunction"""
  return _PerceptMesh.__mul__(*args)

def __sub__(*args):
  """
    __sub__(StringFunction lhs, StringFunction rhs) -> StringFunction
    __sub__(StringFunction lhs) -> StringFunction
    """
  return _PerceptMesh.__sub__(*args)

def square(*args):
  """square(double x) -> double"""
  return _PerceptMesh.square(*args)
class MaxOfNodeValues(Function):
    """Proxy of C++ stk::percept::MaxOfNodeValues class"""
    __swig_setmethods__ = {}
    for _s in [Function]: __swig_setmethods__.update(getattr(_s,'__swig_setmethods__',{}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, MaxOfNodeValues, name, value)
    __swig_getmethods__ = {}
    for _s in [Function]: __swig_getmethods__.update(getattr(_s,'__swig_getmethods__',{}))
    __getattr__ = lambda self, name: _swig_getattr(self, MaxOfNodeValues, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_setmethods__["m_integrand"] = _PerceptMesh.MaxOfNodeValues_m_integrand_set
    __swig_getmethods__["m_integrand"] = _PerceptMesh.MaxOfNodeValues_m_integrand_get
    if _newclass:m_integrand = _swig_property(_PerceptMesh.MaxOfNodeValues_m_integrand_get, _PerceptMesh.MaxOfNodeValues_m_integrand_set)
    __swig_setmethods__["maxVal"] = _PerceptMesh.MaxOfNodeValues_maxVal_set
    __swig_getmethods__["maxVal"] = _PerceptMesh.MaxOfNodeValues_maxVal_get
    if _newclass:maxVal = _swig_property(_PerceptMesh.MaxOfNodeValues_maxVal_get, _PerceptMesh.MaxOfNodeValues_maxVal_set)
    __swig_destroy__ = _PerceptMesh.delete_MaxOfNodeValues
MaxOfNodeValues_swigregister = _PerceptMesh.MaxOfNodeValues_swigregister
MaxOfNodeValues_swigregister(MaxOfNodeValues)

class L2Norm(_object):
    """This class will evaluate the L2 norm of a function."""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, L2Norm, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, L2Norm, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, std::string partName, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, std::string partName, TurboOption turboOpt=TURBO_NONE) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, std::string partName) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, MDArrayString partNames, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, MDArrayString partNames, TurboOption turboOpt=TURBO_NONE) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, MDArrayString partNames) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, Part part=None, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, Part part=None, TurboOption turboOpt=TURBO_NONE) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData, Part part=None) -> L2Norm
        __init__(stk::percept::Norm<(2)> self, BulkData bulkData) -> L2Norm

        stk::percept::Norm<
        Power >::Norm(mesh::BulkData &bulkData, mesh::Selector *selector,
        TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=false) 
        """
        this = _PerceptMesh.new_L2Norm(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_L2Norm
    def setCubDegree(self, *args):
        """
        setCubDegree(L2Norm self, unsigned int cubDegree)

        void
        stk::percept::Norm< Power >::setCubDegree(unsigned cubDegree) 
        """
        return _PerceptMesh.L2Norm_setCubDegree(self, *args)

    def getCubDegree(self):
        """
        getCubDegree(L2Norm self) -> unsigned int

        unsigned
        stk::percept::Norm< Power >::getCubDegree() 
        """
        return _PerceptMesh.L2Norm_getCubDegree(self)

    def set_is_surface_norm(self, *args):
        """
        set_is_surface_norm(L2Norm self, bool is_surface_norm)

        void
        stk::percept::Norm< Power >::set_is_surface_norm(bool is_surface_norm)

        """
        return _PerceptMesh.L2Norm_set_is_surface_norm(self, *args)

    def get_is_surface_norm(self):
        """
        get_is_surface_norm(L2Norm self) -> bool

        bool
        stk::percept::Norm< Power >::get_is_surface_norm() 
        """
        return _PerceptMesh.L2Norm_get_is_surface_norm(self)

    def evaluate(self, *args):
        """
        evaluate(L2Norm self, Function integrand) -> double

        double
        stk::percept::Norm< Power >::evaluate(Function &integrand) 
        """
        return _PerceptMesh.L2Norm_evaluate(self, *args)

    def error_check_is_surface_norm(self):
        """
        error_check_is_surface_norm(L2Norm self)

        void stk::percept::Norm< Power >::error_check_is_surface_norm()

        if a Selector is specified with part(s) that are not auto-declared,
        make sure all parts are of the same rank, and that m_is_surface_norm
        is set correctly (if not, warn...) 
        """
        return _PerceptMesh.L2Norm_error_check_is_surface_norm(self)

L2Norm_swigregister = _PerceptMesh.L2Norm_swigregister
L2Norm_swigregister(L2Norm)

class L1Norm(_object):
    """This class will evaluate the L1 norm of a function."""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, L1Norm, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, L1Norm, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, std::string partName, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, std::string partName, TurboOption turboOpt=TURBO_NONE) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, std::string partName) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, MDArrayString partNames, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, MDArrayString partNames, TurboOption turboOpt=TURBO_NONE) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, MDArrayString partNames) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, Part part=None, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, Part part=None, TurboOption turboOpt=TURBO_NONE) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData, Part part=None) -> L1Norm
        __init__(stk::percept::Norm<(1)> self, BulkData bulkData) -> L1Norm

        stk::percept::Norm<
        Power >::Norm(mesh::BulkData &bulkData, mesh::Selector *selector,
        TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=false) 
        """
        this = _PerceptMesh.new_L1Norm(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_L1Norm
    def setCubDegree(self, *args):
        """
        setCubDegree(L1Norm self, unsigned int cubDegree)

        void
        stk::percept::Norm< Power >::setCubDegree(unsigned cubDegree) 
        """
        return _PerceptMesh.L1Norm_setCubDegree(self, *args)

    def getCubDegree(self):
        """
        getCubDegree(L1Norm self) -> unsigned int

        unsigned
        stk::percept::Norm< Power >::getCubDegree() 
        """
        return _PerceptMesh.L1Norm_getCubDegree(self)

    def set_is_surface_norm(self, *args):
        """
        set_is_surface_norm(L1Norm self, bool is_surface_norm)

        void
        stk::percept::Norm< Power >::set_is_surface_norm(bool is_surface_norm)

        """
        return _PerceptMesh.L1Norm_set_is_surface_norm(self, *args)

    def get_is_surface_norm(self):
        """
        get_is_surface_norm(L1Norm self) -> bool

        bool
        stk::percept::Norm< Power >::get_is_surface_norm() 
        """
        return _PerceptMesh.L1Norm_get_is_surface_norm(self)

    def evaluate(self, *args):
        """
        evaluate(L1Norm self, Function integrand) -> double

        double
        stk::percept::Norm< Power >::evaluate(Function &integrand) 
        """
        return _PerceptMesh.L1Norm_evaluate(self, *args)

    def error_check_is_surface_norm(self):
        """
        error_check_is_surface_norm(L1Norm self)

        void stk::percept::Norm< Power >::error_check_is_surface_norm()

        if a Selector is specified with part(s) that are not auto-declared,
        make sure all parts are of the same rank, and that m_is_surface_norm
        is set correctly (if not, warn...) 
        """
        return _PerceptMesh.L1Norm_error_check_is_surface_norm(self)

L1Norm_swigregister = _PerceptMesh.L1Norm_swigregister
L1Norm_swigregister(L1Norm)

class LInfNorm(_object):
    """This class will evaluate the L-inf norm of a function."""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, LInfNorm, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, LInfNorm, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, std::string partName, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, std::string partName, TurboOption turboOpt=TURBO_NONE) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, std::string partName) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, MDArrayString partNames, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, MDArrayString partNames, TurboOption turboOpt=TURBO_NONE) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, MDArrayString partNames) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, Part part=None, TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=False) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, Part part=None, TurboOption turboOpt=TURBO_NONE) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData, Part part=None) -> LInfNorm
        __init__(stk::percept::Norm<(-1)> self, BulkData bulkData) -> LInfNorm

        stk::percept::Norm<
        Power >::Norm(mesh::BulkData &bulkData, mesh::Selector *selector,
        TurboOption turboOpt=TURBO_NONE, bool is_surface_norm=false) 
        """
        this = _PerceptMesh.new_LInfNorm(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_LInfNorm
    def setCubDegree(self, *args):
        """
        setCubDegree(LInfNorm self, unsigned int cubDegree)

        void
        stk::percept::Norm< Power >::setCubDegree(unsigned cubDegree) 
        """
        return _PerceptMesh.LInfNorm_setCubDegree(self, *args)

    def getCubDegree(self):
        """
        getCubDegree(LInfNorm self) -> unsigned int

        unsigned
        stk::percept::Norm< Power >::getCubDegree() 
        """
        return _PerceptMesh.LInfNorm_getCubDegree(self)

    def set_is_surface_norm(self, *args):
        """
        set_is_surface_norm(LInfNorm self, bool is_surface_norm)

        void
        stk::percept::Norm< Power >::set_is_surface_norm(bool is_surface_norm)

        """
        return _PerceptMesh.LInfNorm_set_is_surface_norm(self, *args)

    def get_is_surface_norm(self):
        """
        get_is_surface_norm(LInfNorm self) -> bool

        bool
        stk::percept::Norm< Power >::get_is_surface_norm() 
        """
        return _PerceptMesh.LInfNorm_get_is_surface_norm(self)

    def evaluate(self, *args):
        """
        evaluate(LInfNorm self, Function integrand) -> double

        double
        stk::percept::Norm< Power >::evaluate(Function &integrand) 
        """
        return _PerceptMesh.LInfNorm_evaluate(self, *args)

    def error_check_is_surface_norm(self):
        """
        error_check_is_surface_norm(LInfNorm self)

        void stk::percept::Norm< Power >::error_check_is_surface_norm()

        if a Selector is specified with part(s) that are not auto-declared,
        make sure all parts are of the same rank, and that m_is_surface_norm
        is set correctly (if not, warn...) 
        """
        return _PerceptMesh.LInfNorm_error_check_is_surface_norm(self)

LInfNorm_swigregister = _PerceptMesh.LInfNorm_swigregister
LInfNorm_swigregister(LInfNorm)

NEW_FIX_ELEMENT_SIDES = _PerceptMesh.NEW_FIX_ELEMENT_SIDES
STK_ADAPT_URP_LOCAL_NODE_COMPS = _PerceptMesh.STK_ADAPT_URP_LOCAL_NODE_COMPS
FORCE_TRACE_PRINT_ONLY = _PerceptMesh.FORCE_TRACE_PRINT_ONLY
TRACE_STAGE_PRINT_ON = _PerceptMesh.TRACE_STAGE_PRINT_ON
DEBUG_URP_HPP = _PerceptMesh.DEBUG_URP_HPP
class SierraPort(_object):
    """Proxy of C++ stk::adapt::SierraPort class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SierraPort, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SierraPort, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::adapt::SierraPort self) -> SierraPort"""
        this = _PerceptMesh.new_SierraPort()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_SierraPort
SierraPort_swigregister = _PerceptMesh.SierraPort_swigregister
SierraPort_swigregister(SierraPort)

class STK_Adapt_Auto_Part(_object):
    """
    signifies a part that has been defined automatically during adaptivity

    C++ includes: UniformRefinerPattern.hpp 
    """
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, STK_Adapt_Auto_Part, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, STK_Adapt_Auto_Part, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(stk::adapt::STK_Adapt_Auto_Part self) -> STK_Adapt_Auto_Part"""
        this = _PerceptMesh.new_STK_Adapt_Auto_Part()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_STK_Adapt_Auto_Part
STK_Adapt_Auto_Part_swigregister = _PerceptMesh.STK_Adapt_Auto_Part_swigregister
STK_Adapt_Auto_Part_swigregister(STK_Adapt_Auto_Part)

class UniformRefinerPatternBase(_object):
    """
    The base class for all refinement patterns
    ------------------------------------------------------------------------------------------------------------------------

    C++ includes: UniformRefinerPattern.hpp 
    """
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, UniformRefinerPatternBase, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, UniformRefinerPatternBase, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _PerceptMesh.delete_UniformRefinerPatternBase
    def doBreak(self):
        """
        doBreak(UniformRefinerPatternBase self)

        virtual void stk::adapt::UniformRefinerPatternBase::doBreak()=0 
        """
        return _PerceptMesh.UniformRefinerPatternBase_doBreak(self)

    def getFromTypeKey(self):
        """
        getFromTypeKey(UniformRefinerPatternBase self) -> unsigned int

        virtual
        unsigned stk::adapt::UniformRefinerPatternBase::getFromTypeKey()=0 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getFromTypeKey(self)

    def getFromTopology(self):
        """
        getFromTopology(UniformRefinerPatternBase self) -> CellTopologyData const *

        virtual const
        CellTopologyData*
        stk::adapt::UniformRefinerPatternBase::getFromTopology()=0 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getFromTopology(self)

    def getToTopology(self):
        """
        getToTopology(UniformRefinerPatternBase self) -> CellTopologyData const *

        virtual const
        CellTopologyData*
        stk::adapt::UniformRefinerPatternBase::getToTopology()=0 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getToTopology(self)

    def getPrimaryEntityRank(self):
        """
        getPrimaryEntityRank(UniformRefinerPatternBase self) -> stk::mesh::EntityRank

        stk::mesh::EntityRank
        stk::adapt::UniformRefinerPatternBase::getPrimaryEntityRank() 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getPrimaryEntityRank(self)

    def fillNeededEntities(self, *args):
        """
        fillNeededEntities(UniformRefinerPatternBase self, std::vector< NeededEntityType,std::allocator< NeededEntityType > > & needed_entities)

        virtual
        void
        stk::adapt::UniformRefinerPatternBase::fillNeededEntities(std::vector<
        NeededEntityType > &needed_entities)=0

        must be provided by derived classes
        ------------------------------------------------------------------------------------------------------------------------
        supplies the ranks of the sub entities needed during refinement (eg.
        m_eMesh.face_rank(), m_eMesh.edge_rank(),..) 10/02/10 and the number
        of nodes needed for each sub entity 
        """
        return _PerceptMesh.UniformRefinerPatternBase_fillNeededEntities(self, *args)

    def setNeededParts(self, *args):
        """
        setNeededParts(UniformRefinerPatternBase self, PerceptMesh eMesh, vectorvs block_names_ranks, bool sameTopology=True)
        setNeededParts(UniformRefinerPatternBase self, PerceptMesh eMesh, vectorvs block_names_ranks)

        virtual void
        stk::adapt::UniformRefinerPatternBase::setNeededParts(percept::PerceptMesh
        &eMesh, BlockNamesType block_names_ranks, bool sameTopology=true) 
        """
        return _PerceptMesh.UniformRefinerPatternBase_setNeededParts(self, *args)

    def getNumNewElemPerElem(self):
        """
        getNumNewElemPerElem(UniformRefinerPatternBase self) -> unsigned int

        virtual
        unsigned
        stk::adapt::UniformRefinerPatternBase::getNumNewElemPerElem()=0

        supply the number of new elements per element during refinement 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getNumNewElemPerElem(self)

    def createNewElements(self, *args):
        """
        createNewElements(UniformRefinerPatternBase self, PerceptMesh eMesh, NodeRegistry & nodeRegistry, stk::mesh::Entity & element, stk::adapt::NewSubEntityNodesType & new_sub_entity_nodes, 
            vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > >::iterator & element_pool, 
            FieldBase proc_rank_field=None)
        createNewElements(UniformRefinerPatternBase self, PerceptMesh eMesh, NodeRegistry & nodeRegistry, stk::mesh::Entity & element, stk::adapt::NewSubEntityNodesType & new_sub_entity_nodes, 
            vector< stk::mesh::Entity *,std::allocator< stk::mesh::Entity * > >::iterator & element_pool)

        virtual
        void
        stk::adapt::UniformRefinerPatternBase::createNewElements(percept::PerceptMesh
        &eMesh, NodeRegistry &nodeRegistry, stk::mesh::Entity &element,
        NewSubEntityNodesType &new_sub_entity_nodes, vector< stk::mesh::Entity
        * >::iterator &element_pool, stk::mesh::FieldBase
        *proc_rank_field=0)=0

        given the node database ( NodeRegistry), and the newly created nodes,
        and an iterator for the elements in the element pool, create all new
        sub-elements of the refined element 
        """
        return _PerceptMesh.UniformRefinerPatternBase_createNewElements(self, *args)

    def set_parent_child_relations(self, *args):
        """
        set_parent_child_relations(UniformRefinerPatternBase self, PerceptMesh eMesh, stk::mesh::Entity & old_owning_elem, stk::mesh::Entity & newElement, 
            unsigned int ordinal, unsigned int * numChild=None)
        set_parent_child_relations(UniformRefinerPatternBase self, PerceptMesh eMesh, stk::mesh::Entity & old_owning_elem, stk::mesh::Entity & newElement, 
            unsigned int ordinal)

        void
        stk::adapt::UniformRefinerPatternBase::set_parent_child_relations(percept::PerceptMesh
        &eMesh, stk::mesh::Entity &old_owning_elem, stk::mesh::Entity
        &newElement, unsigned ordinal, unsigned *numChild=0)

        if numChild is passed in as non-null, use that value, else use
        getNumNewElemPerElem() as size of child vector 
        """
        return _PerceptMesh.UniformRefinerPatternBase_set_parent_child_relations(self, *args)

    def interpolateElementFields(self, *args):
        """
        interpolateElementFields(UniformRefinerPatternBase self, PerceptMesh eMesh, stk::mesh::Entity & old_owning_elem, stk::mesh::Entity & newElement)

        void
        stk::adapt::UniformRefinerPatternBase::interpolateElementFields(percept::PerceptMesh
        &eMesh, stk::mesh::Entity &old_owning_elem, stk::mesh::Entity
        &newElement) 
        """
        return _PerceptMesh.UniformRefinerPatternBase_interpolateElementFields(self, *args)

    def findSideRelations(self, *args):
        """
        findSideRelations(UniformRefinerPatternBase self, PerceptMesh eMesh, stk::mesh::Entity * parent, stk::mesh::Entity * child) -> bool

        bool
        stk::adapt::UniformRefinerPatternBase::findSideRelations(percept::PerceptMesh
        &eMesh, stk::mesh::Entity *parent, stk::mesh::Entity *child)

        given a new element (child) that is a child of an original element
        (parent), look at parent's side to elem relations and from the
        children of the element, choose an element to connect the new side to
        (using connectSides) 
        """
        return _PerceptMesh.UniformRefinerPatternBase_findSideRelations(self, *args)

    def connectSides(self, *args):
        """
        connectSides(UniformRefinerPatternBase self, PerceptMesh eMesh, stk::mesh::Entity * element, stk::mesh::Entity * side_elem) -> bool

        bool
        stk::adapt::UniformRefinerPatternBase::connectSides(percept::PerceptMesh
        &eMesh, stk::mesh::Entity *element, stk::mesh::Entity *side_elem) 
        """
        return _PerceptMesh.UniformRefinerPatternBase_connectSides(self, *args)

    def setSubPatterns(self, *args):
        """
        setSubPatterns(UniformRefinerPatternBase self, std::vector< stk::adapt::UniformRefinerPatternBase *,std::allocator< stk::adapt::UniformRefinerPatternBase * > > & bp, 
            PerceptMesh eMesh)

        virtual void
        stk::adapt::UniformRefinerPatternBase::setSubPatterns(std::vector<
        UniformRefinerPatternBase * > &bp, percept::PerceptMesh &eMesh)

        optionally overridden (must be overridden if sidesets are to work
        properly) to provide info on which sub pattern should be used to
        refine side sets (and edge sets)

        default is only this pattern 
        """
        return _PerceptMesh.UniformRefinerPatternBase_setSubPatterns(self, *args)

    def fixSurfaceAndEdgeSetNamesMap(self):
        """
        fixSurfaceAndEdgeSetNamesMap(UniformRefinerPatternBase self) -> stk::adapt::StringStringMap

        virtual StringStringMap
        stk::adapt::UniformRefinerPatternBase::fixSurfaceAndEdgeSetNamesMap()

        for i/o to work properly, supply string replacements such as for
        hex-->tet breaking, you would supply "quad"-->"tri" etc. string
        maps 
        """
        return _PerceptMesh.UniformRefinerPatternBase_fixSurfaceAndEdgeSetNamesMap(self)

    def getFromTopoPartName(self):
        """
        getFromTopoPartName(UniformRefinerPatternBase self) -> std::string

        virtual
        std::string
        stk::adapt::UniformRefinerPatternBase::getFromTopoPartName()=0

        provided by this class
        ------------------------------------------------------------------------------------------------------------------------

        """
        return _PerceptMesh.UniformRefinerPatternBase_getFromTopoPartName(self)

    def getToTopoPartName(self):
        """
        getToTopoPartName(UniformRefinerPatternBase self) -> std::string

        virtual
        std::string
        stk::adapt::UniformRefinerPatternBase::getToTopoPartName()=0 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getToTopoPartName(self)

    def getName(self):
        """
        getName(UniformRefinerPatternBase self) -> std::string

        virtual std::string stk::adapt::UniformRefinerPatternBase::getName()

        """
        return _PerceptMesh.UniformRefinerPatternBase_getName(self)

    def getToParts(self):
        """
        getToParts(UniformRefinerPatternBase self) -> vectorparts

        stk::mesh::PartVector&
        stk::adapt::UniformRefinerPatternBase::getToParts() 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getToParts(self)

    def getFromParts(self):
        """
        getFromParts(UniformRefinerPatternBase self) -> vectorparts

        stk::mesh::PartVector&
        stk::adapt::UniformRefinerPatternBase::getFromParts() 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getFromParts(self)

    def getAppendConvertString(self):
        """
        getAppendConvertString(UniformRefinerPatternBase self) -> std::string const &

        const
        std::string&
        stk::adapt::UniformRefinerPatternBase::getAppendConvertString() 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getAppendConvertString(self)

    def getAppendOriginalString(self):
        """
        getAppendOriginalString(UniformRefinerPatternBase self) -> std::string const &

        const
        std::string&
        stk::adapt::UniformRefinerPatternBase::getAppendOriginalString() 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getAppendOriginalString(self)

    def getOldElementsPartName():
        """getOldElementsPartName() -> std::string const &"""
        return _PerceptMesh.UniformRefinerPatternBase_getOldElementsPartName()

    if _newclass:getOldElementsPartName = staticmethod(getOldElementsPartName)
    __swig_getmethods__["getOldElementsPartName"] = lambda x: getOldElementsPartName
    def setToOne(self, *args):
        """
        setToOne(UniformRefinerPatternBase self, std::vector< NeededEntityType,std::allocator< NeededEntityType > > & needed_entities)

        void stk::adapt::UniformRefinerPatternBase::setToOne(std::vector<
        NeededEntityType > &needed_entities)

        utilities --------- sets the needed number of nodes on each sub-entity
        to 1 - this is just a helper - in general, edges and faces have 1 new
        node for linear elements, and multiple new nodes in the case of
        quadratic elements 
        """
        return _PerceptMesh.UniformRefinerPatternBase_setToOne(self, *args)

    def midPoint(self, *args):
        """
        midPoint(UniformRefinerPatternBase self, double const * p1, double const * p2, int spatialDim, double * x) -> double *

        double* stk::adapt::UniformRefinerPatternBase::midPoint(const double
        *p1, const double *p2, int spatialDim, double *x) 
        """
        return _PerceptMesh.UniformRefinerPatternBase_midPoint(self, *args)

    def getCentroid(self, *args):
        """
        getCentroid(UniformRefinerPatternBase self, double *[] pts, int len, int spatialDim, double * x) -> double *

        double*
        stk::adapt::UniformRefinerPatternBase::getCentroid(double *pts[], int
        len, int spatialDim, double *x) 
        """
        return _PerceptMesh.UniformRefinerPatternBase_getCentroid(self, *args)

    def getTopoDim(*args):
        """getTopoDim(shards::CellTopology & cell_topo) -> int"""
        return _PerceptMesh.UniformRefinerPatternBase_getTopoDim(*args)

    if _newclass:getTopoDim = staticmethod(getTopoDim)
    __swig_getmethods__["getTopoDim"] = lambda x: getTopoDim
    def createPattern(*args):
        """createPattern(std::string refine, std::string enrich, std::string convert, PerceptMesh eMesh, vectorvs block_names) -> Teuchos::RCP< stk::adapt::UniformRefinerPatternBase >"""
        return _PerceptMesh.UniformRefinerPatternBase_createPattern(*args)

    if _newclass:createPattern = staticmethod(createPattern)
    __swig_getmethods__["createPattern"] = lambda x: createPattern
    __swig_setmethods__["s_convert_options"] = _PerceptMesh.UniformRefinerPatternBase_s_convert_options_set
    __swig_getmethods__["s_convert_options"] = _PerceptMesh.UniformRefinerPatternBase_s_convert_options_get
    if _newclass:s_convert_options = _swig_property(_PerceptMesh.UniformRefinerPatternBase_s_convert_options_get, _PerceptMesh.UniformRefinerPatternBase_s_convert_options_set)
    __swig_setmethods__["s_refine_options"] = _PerceptMesh.UniformRefinerPatternBase_s_refine_options_set
    __swig_getmethods__["s_refine_options"] = _PerceptMesh.UniformRefinerPatternBase_s_refine_options_get
    if _newclass:s_refine_options = _swig_property(_PerceptMesh.UniformRefinerPatternBase_s_refine_options_get, _PerceptMesh.UniformRefinerPatternBase_s_refine_options_set)
    __swig_setmethods__["s_enrich_options"] = _PerceptMesh.UniformRefinerPatternBase_s_enrich_options_set
    __swig_getmethods__["s_enrich_options"] = _PerceptMesh.UniformRefinerPatternBase_s_enrich_options_get
    if _newclass:s_enrich_options = _swig_property(_PerceptMesh.UniformRefinerPatternBase_s_enrich_options_get, _PerceptMesh.UniformRefinerPatternBase_s_enrich_options_set)
UniformRefinerPatternBase_swigregister = _PerceptMesh.UniformRefinerPatternBase_swigregister
UniformRefinerPatternBase_swigregister(UniformRefinerPatternBase)

def UniformRefinerPatternBase_getOldElementsPartName():
  """UniformRefinerPatternBase_getOldElementsPartName() -> std::string const &"""
  return _PerceptMesh.UniformRefinerPatternBase_getOldElementsPartName()

def UniformRefinerPatternBase_getTopoDim(*args):
  """UniformRefinerPatternBase_getTopoDim(shards::CellTopology & cell_topo) -> int"""
  return _PerceptMesh.UniformRefinerPatternBase_getTopoDim(*args)

def UniformRefinerPatternBase_createPattern(*args):
  """UniformRefinerPatternBase_createPattern(std::string refine, std::string enrich, std::string convert, PerceptMesh eMesh, vectorvs block_names) -> Teuchos::RCP< stk::adapt::UniformRefinerPatternBase >"""
  return _PerceptMesh.UniformRefinerPatternBase_createPattern(*args)

LINE2_LINE2_2 = _PerceptMesh.LINE2_LINE2_2
BEAM2_BEAM2_2 = _PerceptMesh.BEAM2_BEAM2_2
SHELLLINE2_SHELLLINE2_2 = _PerceptMesh.SHELLLINE2_SHELLLINE2_2
SHELLLINE3_SHELLLINE3_2 = _PerceptMesh.SHELLLINE3_SHELLLINE3_2
QUAD4_QUAD4_4_OLD = _PerceptMesh.QUAD4_QUAD4_4_OLD
QUAD4_QUAD4_4 = _PerceptMesh.QUAD4_QUAD4_4
QUAD4_QUAD4_4_SIERRA = _PerceptMesh.QUAD4_QUAD4_4_SIERRA
TRI3_TRI3_4 = _PerceptMesh.TRI3_TRI3_4
SHELLTRI3_SHELLTRI3_4 = _PerceptMesh.SHELLTRI3_SHELLTRI3_4
SHELLTRI6_SHELLTRI6_4 = _PerceptMesh.SHELLTRI6_SHELLTRI6_4
SHELLQUAD4_SHELLQUAD4_4 = _PerceptMesh.SHELLQUAD4_SHELLQUAD4_4
SHELLQUAD8_SHELLQUAD8_4 = _PerceptMesh.SHELLQUAD8_SHELLQUAD8_4
TET4_TET4_8 = _PerceptMesh.TET4_TET4_8
HEX8_HEX8_8 = _PerceptMesh.HEX8_HEX8_8
WEDGE6_WEDGE6_8 = _PerceptMesh.WEDGE6_WEDGE6_8
PYRAMID5_PYRAMID5_10 = _PerceptMesh.PYRAMID5_PYRAMID5_10
LINE3_LINE3_2 = _PerceptMesh.LINE3_LINE3_2
BEAM3_BEAM3_2 = _PerceptMesh.BEAM3_BEAM3_2
TRI6_TRI6_4 = _PerceptMesh.TRI6_TRI6_4
QUAD9_QUAD9_4 = _PerceptMesh.QUAD9_QUAD9_4
QUAD8_QUAD8_4 = _PerceptMesh.QUAD8_QUAD8_4
HEX27_HEX27_8 = _PerceptMesh.HEX27_HEX27_8
HEX20_HEX20_8 = _PerceptMesh.HEX20_HEX20_8
TET10_TET10_8 = _PerceptMesh.TET10_TET10_8
WEDGE15_WEDGE15_8 = _PerceptMesh.WEDGE15_WEDGE15_8
WEDGE18_WEDGE18_8 = _PerceptMesh.WEDGE18_WEDGE18_8
PYRAMID13_PYRAMID13_10 = _PerceptMesh.PYRAMID13_PYRAMID13_10
QUAD4_QUAD9_1 = _PerceptMesh.QUAD4_QUAD9_1
QUAD4_QUAD8_1 = _PerceptMesh.QUAD4_QUAD8_1
BEAM2_BEAM3_1 = _PerceptMesh.BEAM2_BEAM3_1
SHELLQUAD4_SHELLQUAD8_1 = _PerceptMesh.SHELLQUAD4_SHELLQUAD8_1
TRI3_TRI6_1 = _PerceptMesh.TRI3_TRI6_1
TET4_TET10_1 = _PerceptMesh.TET4_TET10_1
HEX8_HEX27_1 = _PerceptMesh.HEX8_HEX27_1
HEX8_HEX20_1 = _PerceptMesh.HEX8_HEX20_1
WEDGE6_WEDGE15_1 = _PerceptMesh.WEDGE6_WEDGE15_1
WEDGE6_WEDGE18_1 = _PerceptMesh.WEDGE6_WEDGE18_1
PYRAMID5_PYRAMID13_1 = _PerceptMesh.PYRAMID5_PYRAMID13_1
QUAD4_TRI3_2 = _PerceptMesh.QUAD4_TRI3_2
QUAD4_TRI3_4 = _PerceptMesh.QUAD4_TRI3_4
QUAD4_TRI3_6 = _PerceptMesh.QUAD4_TRI3_6
HEX8_TET4_24 = _PerceptMesh.HEX8_TET4_24
HEX8_TET4_6_12 = _PerceptMesh.HEX8_TET4_6_12
UNIFORM_REF_REMOVE_OLD_STD_SET = _PerceptMesh.UNIFORM_REF_REMOVE_OLD_STD_SET
UNIFORM_REF_REMOVE_OLD_STD_VECTOR = _PerceptMesh.UNIFORM_REF_REMOVE_OLD_STD_VECTOR
UNIFORM_REF_REMOVE_OLD_BOOST_SET = _PerceptMesh.UNIFORM_REF_REMOVE_OLD_BOOST_SET
class Refiner(_object):
    """
    e.g. UniformRefiner<shards::Hex<8>, shards::Tet<4> >

    C++ includes: Refiner.hpp 
    """
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Refiner, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Refiner, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(stk::adapt::Refiner self, PerceptMesh eMesh, UniformRefinerPatternBase bp, FieldBase proc_rank_field=None) -> Refiner
        __init__(stk::adapt::Refiner self, PerceptMesh eMesh, UniformRefinerPatternBase bp) -> Refiner
        __init__(stk::adapt::Refiner self, PerceptMesh eMesh, stk::adapt::Pattern refine_type, FieldBase proc_rank_field=None) -> Refiner
        __init__(stk::adapt::Refiner self, PerceptMesh eMesh, stk::adapt::Pattern refine_type) -> Refiner

        stk::adapt::Refiner::Refiner(percept::PerceptMesh &eMesh, Pattern
        refine_type, stk::mesh::FieldBase *proc_rank_field=0) 
        """
        this = _PerceptMesh.new_Refiner(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _PerceptMesh.delete_Refiner
    def doBreak(self):
        """
        doBreak(Refiner self)

        void
        stk::adapt::Refiner::doBreak()

        FIXME add part info

        communicate all-to-all the new node creation information which also
        updates the node registry so it can be queried locally now for any
        ghost or non-ghost element

        Global element ops: here's where we e.g. connect the new elements by
        declaring new relations
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        Global node loop operations: this is where we perform ops like adding
        new nodes to the right parts, interpolating fields, etc.
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        """
        return _PerceptMesh.Refiner_doBreak(self)

    def setRemoveOldElements(self, *args):
        """
        setRemoveOldElements(Refiner self, bool do_remove)

        void stk::adapt::Refiner::setRemoveOldElements(bool do_remove) 
        """
        return _PerceptMesh.Refiner_setRemoveOldElements(self, *args)

    def getRemoveOldElements(self):
        """
        getRemoveOldElements(Refiner self) -> bool

        bool stk::adapt::Refiner::getRemoveOldElements() 
        """
        return _PerceptMesh.Refiner_getRemoveOldElements(self)

    def setGeometryFile(self, *args):
        """
        setGeometryFile(Refiner self, std::string file_name)

        void
        stk::adapt::Refiner::setGeometryFile(std::string file_name) 
        """
        return _PerceptMesh.Refiner_setGeometryFile(self, *args)

    def setSmoothGeometry(self, *args):
        """
        setSmoothGeometry(Refiner self, bool do_smooth)

        void
        stk::adapt::Refiner::setSmoothGeometry(bool do_smooth) 
        """
        return _PerceptMesh.Refiner_setSmoothGeometry(self, *args)

    def getSmoothGeometry(self):
        """
        getSmoothGeometry(Refiner self) -> bool

        bool
        stk::adapt::Refiner::getSmoothGeometry() 
        """
        return _PerceptMesh.Refiner_getSmoothGeometry(self)

    def setRemoveGeometryBlocks(self, *args):
        """
        setRemoveGeometryBlocks(Refiner self, bool do_remove)

        void stk::adapt::Refiner::setRemoveGeometryBlocks(bool do_remove) 
        """
        return _PerceptMesh.Refiner_setRemoveGeometryBlocks(self, *args)

    def getRemoveGeometryBlocks(self):
        """
        getRemoveGeometryBlocks(Refiner self) -> bool

        bool stk::adapt::Refiner::getRemoveGeometryBlocks() 
        """
        return _PerceptMesh.Refiner_getRemoveGeometryBlocks(self)

    def setIgnoreSideSets(self, *args):
        """
        setIgnoreSideSets(Refiner self, bool ignore_sidesets)

        void
        stk::adapt::Refiner::setIgnoreSideSets(bool ignore_sidesets) 
        """
        return _PerceptMesh.Refiner_setIgnoreSideSets(self, *args)

    def getIgnoreSideSets(self):
        """
        getIgnoreSideSets(Refiner self) -> bool

        bool
        stk::adapt::Refiner::getIgnoreSideSets() 
        """
        return _PerceptMesh.Refiner_getIgnoreSideSets(self)

    def getRefinementInfoByType(self):
        """
        getRefinementInfoByType(Refiner self) -> std::vector< RefinementInfoByType,std::allocator< RefinementInfoByType > > &

        std::vector< RefinementInfoByType > &
        stk::adapt::Refiner::getRefinementInfoByType() 
        """
        return _PerceptMesh.Refiner_getRefinementInfoByType(self)

    def setQueryPassOnly(self, *args):
        """
        setQueryPassOnly(Refiner self, bool doQueryOnly)

        void
        stk::adapt::Refiner::setQueryPassOnly(bool doQueryOnly) 
        """
        return _PerceptMesh.Refiner_setQueryPassOnly(self, *args)

    def setDoProgressMeter(self, *args):
        """
        setDoProgressMeter(Refiner self, bool do_progress)

        void
        stk::adapt::Refiner::setDoProgressMeter(bool do_progress) 
        """
        return _PerceptMesh.Refiner_setDoProgressMeter(self, *args)

    def getDoProgressMeter(self):
        """
        getDoProgressMeter(Refiner self) -> bool

        bool
        stk::adapt::Refiner::getDoProgressMeter() 
        """
        return _PerceptMesh.Refiner_getDoProgressMeter(self)

    def unrefineTheseElements(self, *args):
        """
        unrefineTheseElements(Refiner self, stk::adapt::ElementUnrefineCollection & elements_to_unref)

        void
        stk::adapt::Refiner::unrefineTheseElements(ElementUnrefineCollection
        &elements_to_unref) 
        """
        return _PerceptMesh.Refiner_unrefineTheseElements(self, *args)

    def unrefineAll(self):
        """
        unrefineAll(Refiner self)

        void
        stk::adapt::Refiner::unrefineAll() 
        """
        return _PerceptMesh.Refiner_unrefineAll(self)

    def setAlwaysInitializeNodeRegistry(self, *args):
        """
        setAlwaysInitializeNodeRegistry(Refiner self, bool do_init)

        void
        stk::adapt::Refiner::setAlwaysInitializeNodeRegistry(bool do_init) 
        """
        return _PerceptMesh.Refiner_setAlwaysInitializeNodeRegistry(self, *args)

    def getAlwaysInitializeNodeRegistry(self):
        """
        getAlwaysInitializeNodeRegistry(Refiner self) -> bool

        bool
        stk::adapt::Refiner::getAlwaysInitializeNodeRegistry() 
        """
        return _PerceptMesh.Refiner_getAlwaysInitializeNodeRegistry(self)

    def deleteParentElements(self):
        """
        deleteParentElements(Refiner self)

        void stk::adapt::Refiner::deleteParentElements()

        Delete all elements that aren't child elements. 
        """
        return _PerceptMesh.Refiner_deleteParentElements(self)

    def check_db(self, msg=""):
        """
        check_db(Refiner self, std::string msg="")
        check_db(Refiner self)

        void
        stk::adapt::Refiner::check_db(std::string msg="") 
        """
        return _PerceptMesh.Refiner_check_db(self, msg)

    def check_sidesets(self, msg=""):
        """
        check_sidesets(Refiner self, std::string msg="")
        check_sidesets(Refiner self)

        void
        stk::adapt::Refiner::check_sidesets(std::string msg="") 
        """
        return _PerceptMesh.Refiner_check_sidesets(self, msg)

    def check_sidesets_1(self, *args):
        """
        check_sidesets_1(Refiner self, std::string msg)

        void
        stk::adapt::Refiner::check_sidesets_1(std::string msg) 
        """
        return _PerceptMesh.Refiner_check_sidesets_1(self, *args)

    def check_sidesets_2(self, *args):
        """
        check_sidesets_2(Refiner self, std::string msg)

        void
        stk::adapt::Refiner::check_sidesets_2(std::string msg) 
        """
        return _PerceptMesh.Refiner_check_sidesets_2(self, *args)

    def fix_side_sets_1(self):
        """
        fix_side_sets_1(Refiner self)

        void
        stk::adapt::Refiner::fix_side_sets_1() 
        """
        return _PerceptMesh.Refiner_fix_side_sets_1(self)

    def fix_side_sets_2(self):
        """
        fix_side_sets_2(Refiner self)

        void
        stk::adapt::Refiner::fix_side_sets_2() 
        """
        return _PerceptMesh.Refiner_fix_side_sets_2(self)

    def fix_side_sets_3(self, *args):
        """
        fix_side_sets_3(Refiner self, bool checkParentChild, stk::adapt::SidePartMap & side_part_map)

        void
        stk::adapt::Refiner::fix_side_sets_3(bool checkParentChild,
        SidePartMap &side_part_map) 
        """
        return _PerceptMesh.Refiner_fix_side_sets_3(self, *args)

    def get_side_part_relations(self, *args):
        """
        get_side_part_relations(Refiner self, bool checkParentChild, stk::adapt::SidePartMap & side_part_map)

        void stk::adapt::Refiner::get_side_part_relations(bool
        checkParentChild, SidePartMap &side_part_map)

        determine side part to elem part relations 
        """
        return _PerceptMesh.Refiner_get_side_part_relations(self, *args)

    def connectSides(self, *args):
        """
        connectSides(Refiner self, stk::mesh::Entity * element, stk::mesh::Entity * side_elem, stk::adapt::SidePartMap * side_part_map=None) -> bool
        connectSides(Refiner self, stk::mesh::Entity * element, stk::mesh::Entity * side_elem) -> bool

        bool
        stk::adapt::Refiner::connectSides(stk::mesh::Entity *element,
        stk::mesh::Entity *side_elem, SidePartMap *side_part_map=0) 
        """
        return _PerceptMesh.Refiner_connectSides(self, *args)

    def fixElementSides2(self):
        """
        fixElementSides2(Refiner self)

        void
        stk::adapt::Refiner::fixElementSides2() 
        """
        return _PerceptMesh.Refiner_fixElementSides2(self)

    def fixSides(self, *args):
        """
        fixSides(Refiner self, stk::mesh::Entity * parent)

        void
        stk::adapt::Refiner::fixSides(stk::mesh::Entity *parent) 
        """
        return _PerceptMesh.Refiner_fixSides(self, *args)

    def getNodeRegistry(self):
        """
        getNodeRegistry(Refiner self) -> NodeRegistry &

        NodeRegistry& stk::adapt::Refiner::getNodeRegistry() 
        """
        return _PerceptMesh.Refiner_getNodeRegistry(self)

    def getMesh(self):
        """
        getMesh(Refiner self) -> PerceptMesh

        percept::PerceptMesh& stk::adapt::Refiner::getMesh() 
        """
        return _PerceptMesh.Refiner_getMesh(self)

    def __basePatterns__(self):
        """
        __basePatterns__(Refiner self)

        The following is a list of patterns that may be used to create a Refiner object:
        BEAM2_BEAM2_2    
        BEAM2_BEAM3_1    
        BEAM3_BEAM3_2
        HEX20_HEX20_8   
        HEX27_HEX27_8   
        HEX8_HEX20_1
        HEX8_HEX27_1   
        HEX8_HEX8_8      
        HEX8_TET4_24
        HEX8_TET4_6_12    
        LINE2_LINE2_2     
        LINE3_LINE3_2
        QUAD4_QUAD4_4   
        QUAD4_QUAD4_4_OLD   
        QUAD4_QUAD4_4_SIERRA
        QUAD4_QUAD8_1   
        QUAD4_QUAD9_1      
        QUAD4_TRI3_2
        QUAD4_TRI3_4    
        QUAD4_TRI3_6        
        QUAD8_QUAD8_4
        QUAD9_QUAD9_4
        SHELLLINE2_SHELLLINE2_2            
        SHELLLINE3_SHELLLINE3_2
        SHELLLINE3_SHELLLINE3_2            
        SHELLQUAD4_SHELLQUAD8_1
        SHELLQUAD8_SHELLQUAD8_4            
        SHELLTRI3_SHELLTRI3_4
        SHELLTRI6_SHELLTRI6_4
        TET10_TET10_8    
        TET4_TET10_1    
        TET4_TET4_8
        TRI3_TRI3_4      
        TRI3_TRI6_1     
        TRI6_TRI6_4
        WEDGE15_WEDGE15_8    
        WEDGE18_WEDGE18_8
        WEDGE6_WEDGE15_1     
        WEDGE6_WEDGE18_1
        WEDGE6_WEDGE6_8
        """
        return _PerceptMesh.Refiner___basePatterns__(self)

Refiner_swigregister = _PerceptMesh.Refiner_swigregister
Refiner_swigregister(Refiner)

# This file is compatible with both classic and new-style classes.


